// Copyright (c) 2001-2006, Scalable Network Technologies, Inc.  All Rights Reserved.
//                          6701 Center Drive West
//                          Suite 520
//                          Los Angeles, CA 90045
//                          sales@scalable-networks.com
//
// This source code is licensed, not sold, and is subject to a written
// license agreement.  Among other things, no portion of this source
// code may be copied, transmitted, disclosed, displayed, distributed,
// translated, used as the basis for a derivative work, or used, in
// whole or in part, for any program or purpose other than its intended
// use in compliance with the license agreement as part of the QualNet
// software.  This source code and certain of the algorithms contained
// within it are confidential trade secrets of Scalable Network
// Technologies, Inc. and may not be used as the basis for any other
// software, hardware, product or service.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "api.h"
#include "partition.h"
#include "mac_802_11.h"
#include "network_ip.h"
#include "ipv6.h"

#ifdef ENTERPRISE_LIB
#include "mpls_shim.h"
#endif // ENTERPRISE_LIB


#define DEBUG 0

// Used to force remote nodes to yield longer for the data exchange.
// Also needed since when packets will arrive at the same time a timer
// times out.  Add delay to allow packets to be process first before
// timers get a chance to expire.  Therefore, this value to be at
// least 1.

#define EPSILON_DELAY 1


BOOL Mac802_11CanHandleDueCfpBeacon(
    Node* node, MacData802_11* M802);
void Mac802_11CfpPcBeaconTransmitted(
    Node* node, MacData802_11* M802);
void Mac802_11CfpFrameTransmitted(
    Node* node, MacData802_11* M802);
void Mac802_11CfpCancelSomeTimersBecauseChannelHasBecomeBusy(
    Node* node, MacData802_11* M802);
void Mac802_11CfpPhyStatusIsNowIdleStartSendTimers(
    Node* node, MacData802_11* M802);
void Mac802_11CfpReceivePacketFromPhy(
    Node* node, MacData802_11* M802, Message* msg);
void Mac802_11CfpStationInitiateCfPeriod(
    Node* node, MacData802_11* M802);
void Mac802_11ProcessCfpBeacon(
    Node* node, MacData802_11* M802, Message* msg);
void Mac802_11ProcessCfpEnd(
    Node* node, MacData802_11* M802, Message* msg);
void Mac802_11CfpHandleTimeout(
    Node* node, MacData802_11* M802);
BOOL Mac802_11CfpPcAttemptToTransmitCfBeacon(
    Node* node, MacData802_11* M802);
void Mac802_11CfpHandleEndTimer(
    Node* node, MacData802_11* M802);
void Mac802_11CfpInit(
    Node* node, const NodeInput* nodeInput, MacData802_11* M802,
    MacSubnetMemberData* subnetList, int nodesInSubnet,
    int subnetListIndex, NodeAddress subnetAddress, int numHostBits,
    int networkType = NETWORK_IPV4,
    /*unsigned int tla = 0,
    unsigned int nla = 0,
    unsigned int sla = 0*/
    in6_addr *subnetAddr = 0,
    unsigned int subnetPrefixLen = 0);

void Mac802_11CfpPrintStats(
    Node* node, MacData802_11* M802, int interfaceIndex);
void Mac802_11CfpFreeStorage(
    Node* node, int interfaceIndex);

//--------------------------------------------------------------------------
//  NAME:        Mac802_11MicroToNanosecond
//  PURPOSE:     Convert a value in micro-seconds to clocktype.
//  PARAMETERS:  int us
//                  Value in micro-seconds
//  RETURN:      Value in nano-seconds
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
clocktype Mac802_11MicroToNanosecond(int us)
{
    return (clocktype) us * MICRO_SECOND;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11NanoToMicrosecond
//  PURPOSE:     Convert a value from clocktype to micro-seconds.
//               Return value is rounded up for fractional micro-seconds.
//  PARAMETERS:  clocktype ns
//                  Value in nano-seconds
//  RETURN:      Value in micro-seconds
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
int Mac802_11NanoToMicrosecond(clocktype ns)
{
    return (int) ((ns + 999) / MICRO_SECOND);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TUsToClocktype
//  PURPOSE:     Convert a value in Time Units to clocktype
//               (1 TU = 1024 micro-seconds)
//  PARAMETERS:  unsigned short tu
//                  Value in TUs (1 to 32767)
//  RETURN:      Value in nanoseconds
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
clocktype Mac802_11TUsToClocktype(unsigned short tu)
{
    return (clocktype) tu * M802_11_TIME_UNIT;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ClocktypeToTUs
//  PURPOSE:     Convert time from clocktype to Time Units
//               (1 TU = 1024 micro-seconds)
//  PARAMETERS:  clocktype timeValue
//                  Value in nanoseconds
//  RETURN:      Value in TUs
//  ASSUMPTION:  Integer division
//--------------------------------------------------------------------------
static
unsigned short Mac802_11ClocktypeToTUs(
    clocktype timeValue)
{
    return (unsigned short) (timeValue / M802_11_TIME_UNIT);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ResetCurrentMessageVariables
//  PURPOSE:     Reset variables related to single message buffer at
//               the 802.11 interface.
//               Called after a broadcast is sent or when a unicast is
//               backed or dropped.
//
//  PARAMETERS:  MacData802_11* const M802
//                  Pointer to M802 structure.
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11ResetCurrentMessageVariables(
    MacData802_11* const M802)
{
    M802->currentMessage = NULL;
    M802->currentNextHopAddress = ANY_DEST;
    M802->ipNextHopAddr = ANY_DEST;
    M802->ipDestAddr = ANY_DEST;
    M802->ipSourceAddr = ANY_DEST;
    M802->dataRateInfo = NULL;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ResetAckVariables
//  PURPOSE:     Reset variables once an ack is received.
//  PARAMETERS:  MacData802_11* const M802
//                  Pointer to M802 structure.
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11ResetAckVariables(
    MacData802_11* const M802)
{
    M802->SSRC = 0;
    M802->SLRC = 0;
    M802->waitingForAckOrCtsFromAddress = INVALID_ADDRESS;
}

//--------------------------------------------------------------------------
// FUNCTION		Mac802_11UpdateCurrentSignalDataANTS01
// PURPOSE		Allow 802.11 to update the current value of physical layer 
//				data. These data can then later be used to update data at 
//				the routing layer, once we know which kind of reception we
//				are dealing with, and who is the sender
// PARAMETERS	Node *node;
//				int interfaceIndex
//				double power
//				double sensitivity
//				double interference
//				double noise
// RETURN		None
// NOTES		None
//--------------------------------------------------------------------------

void Mac802_11UpdateCurrentSignalDataANTS01(Node *node, 
											int interfaceIndex,
											double power, 
											double sensitivity, 
											double interference, 
											double noise,
											double sinr)
{
	MacData802_11 *M802 = (MacData802_11*)node->macData[interfaceIndex]->macVar;
	M802->currentPower = power;
	M802->currentSensitivity = sensitivity;
	M802->currentInterference = interference;
	M802->currentNoise = noise;
	M802->currentSinr = sinr;
}

//--------------------------------------------------------------------------
// FUNCTION     Mac802_11NotifySendReceiveANTS01
// PURPOSE		Notify the ANTS01 protocol of a received acknowledgement
// PARAMETERS	Node *node
//				NodeAddress nextHop
// RETURN		None
// NOTES		None
//--------------------------------------------------------------------------

void Mac802_11NotifySendReceiveANTS01(Node *node, NodeAddress nextHop)
{
  if (NetworkIpGetRoutingProtocol(node, ROUTING_PROTOCOL_ANTHOCNET) != NULL)
    {
      Message *msg = MESSAGE_Alloc(node, NETWORK_LAYER, ROUTING_PROTOCOL_ANTHOCNET,
				   MAC_ANTHOCNET_SendReceive);
      
      HandleSpecialMacLayerStatusEvents(node,
					msg,
					nextHop,
					DEFAULT_INTERFACE);
    }
}

//--------------------------------------------------------------------------
// FUNCTION     Mac802_11UpdateAvgSendTimeANTS01
// PURPOSE      To Update the Average Send Time of unicast packets, used for
//              estimates in ANTS01.
// PARAMETERS   Node* node
//              MacData802_11* M802
// RETURN       None
// NOTES        None
//--------------------------------------------------------------------------
void Mac802_11UpdateAvgSendTimeANTS01(Node* node, MacData802_11* M802)
{
	M802->avgSendTimeANTS01 = (clocktype)(M802->avgSendTimeANTS01 * M802->avgSendTimeDiscountANTS01);
	M802->avgSendTimeANTS01 += (clocktype)((1 - M802->avgSendTimeDiscountANTS01) *
		(getSimTime(node) - M802->currentStartTimeANTS01));
}

//--------------------------------------------------------------------------
// FUNCTION     Mac802_11InitAvgSendTimeANTS01
// PURPOSE	    Initialize avgSendTimeANTS01
// PARAMETERS	Node* node
//				int interfaceIndex
//              double initVal
//              double discount
// RETURN       None
// NOTES        None
//--------------------------------------------------------------------------
void Mac802_11InitAvgSendTimeANTS01(Node* node, int interfaceIndex, clocktype initVal, double discount)
{
	MacData802_11* M802 = (MacData802_11*)node->macData[interfaceIndex]->macVar;
  M802->avgSendTimeANTS01 = initVal;
	M802->avgSendTimeDiscountANTS01 = discount;
}

//--------------------------------------------------------------------------
// FUNCTION     Mac802_11GetAvgSendTimeANTS01
// PURPOSE      To Get the Average Send Time of unicast packets, used for
//              estimates in ANTS01.
// PARAMETERS   Node* node, int interfaceIndex
// RETURN       the average send time per unicast packet
// NOTES        None
//--------------------------------------------------------------------------
clocktype Mac802_11GetAvgSendTimeANTS01(Node* node,int interfaceIndex)
{
	MacData802_11* M802 = (MacData802_11*)node->macData[interfaceIndex]->macVar;
	return M802->avgSendTimeANTS01;
}

//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsInCfp
//  PURPOSE:     Determine if within contention free period.
//               Used to handle events accordingly.
//
//  PARAMETERS:  M802_11_MacStates state
//                  Current DCF state
//  RETURN:      TRUE if within contention free period
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsInCfp(
    M802_11_MacStates state)
{
    return state == M802_11_CFP_START;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpIsInTransmitState
//  PURPOSE:     Determine if currently in CFP transmit state.
//               Used for asserts when receiving frames.
//
//  PARAMETERS:  M802_11_CfpStates state
//                  Current CFP state
//  RETURN:      TRUE if in tranmit state
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpIsInTransmitState(
    M802_11_CfpStates state)
{
    return state >= M802_11_X_CFP_BROADCAST
        && state <= M802_11_X_CFP_END;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpSetState
//  PURPOSE:     Set the CFP state. Save previous state.
//  PARAMETERS:  MacData802_11 * const M802
//                  Pointer to M802 structure.
//               M802_11_CfpStates state
//                  New CFP state to be set
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpSetState(
    MacData802_11* const M802,
    M802_11_CfpStates state)
{
    M802->cfpPrevState = M802->cfpState;
    M802->cfpState = state;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsCfFrameAnAckType
//  PURPOSE:     Determine if the CF frame is one of those which
//               acknowledges previously sent data e.g. CF_DATA_ACK
//               or CF_END_ACK.
//
//  PARAMETERS:  M802_11MacFrameType frameType
//                  Frame type
//  RETURN:      TRUE if frame type has a piggy-back ack
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsCfFrameAnAckType(
    M802_11MacFrameType frameType)
{
    BOOL isAckType = FALSE;

    switch (frameType) {
        // This check could use the bit pattern in frame type.
        case M802_11_CF_DATA_ACK:
        case M802_11_CF_DATA_POLL_ACK:
        case M802_11_CF_ACK:
        case M802_11_CF_POLL_ACK:
        case M802_11_CF_END_ACK:
            isAckType = TRUE;
            break;

        default:
            break;
    } //switch

    return isAckType;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsBssStation
//  PURPOSE:     Determine if station is a BSS station.
//               Other types of stations are AP, PC or IBSS station.
//               The station type is a user input.
//
//  PARAMETERS:  const MacData802_11 * const M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE for stations within a BSS
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsBssStation(
    const MacData802_11* const M802)
{
    return M802->stationType == M802_11_STA_BSS;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsAP
//  PURPOSE:     Determine if station is either an Access Point or a
//               Point Coordinator.
//  PARAMETERS:  const MacData802_11 * const M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE for Access Points and Point Coordinators
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsAP(
    const MacData802_11* const M802)
{
    return M802->stationType == M802_11_STA_AP
        || M802->stationType == M802_11_STA_PC;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsPC
//  PURPOSE:     Determine if station is a Point Coordinator.
//  PARAMETERS:  const MacData802_11* const M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE for Access Points and Point Coordinators
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsPC(
    const MacData802_11 * const M802)
{
    return M802->stationType == M802_11_STA_PC;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsFrameFromMyAP
//  PURPOSE:     Determine if frame is from the AP or PC of the BSS
//               by matching source address.
//               M802_11_FROM_DS is not matched.
//
//  PARAMETERS:  const MacData802_11* const M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Source address
//  RETURN:      TRUE if frame is from AP or PC
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsFrameFromMyAP(
    const MacData802_11* const M802,
    NodeAddress sourceAddr)
{
    return sourceAddr == M802->bssAddr;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsFrameFromStationInMyBSS
//  PURPOSE:     Determine if frame is from station within the
//               same BSS.
//  PARAMETERS:  const MacData802_11* const M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Source address
//               unsigned char frameFlags
//                  Frame flags
//  RETURN:      TRUE if frame is destined for AP/PC of the BSS
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsFrameFromStationInMyBSS(
    const MacData802_11* const M802,
    NodeAddress destAddr,
    unsigned char frameFlags)
{
    return destAddr == M802->bssAddr
        && frameFlags & M802_11_TO_DS;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsMyUnicastDataFrame
//  PURPOSE:     Determine if unicast data frame should be received.
//  PARAMETERS:  const MacData802_11 * const M802,
//                  Pointer to 802.11 structure
//               const Message * const msg
//                  Frame message
//  RETURN:      TRUE when BSS station receives from its AP/PC
//                    when AP/PC receives from BSS station
//                    when AP/PC receives directed frame in relay mode
//                    when IBSS station receives directed frame
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsMyUnicastDataFrame(
    const MacData802_11* const M802,
    const Message* const msg)
{
    BOOL isMyFrame = FALSE;
    M802_11FrameHdr* fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);

    if (M802->selfAddr != fHdr->destAddr) {
        return FALSE;
    }

    if (M802->selfAddr != M802->bssAddr) {
        // For a BSS station, check that the frame is from own AP
        if (fHdr->frameFlags & M802_11_FROM_DS
            && fHdr->sourceAddr == M802->bssAddr)
        {
            isMyFrame = TRUE;
        }
    }
    else {
        // An IBSS station will process frames from any source.
        // As will an AP/PC in relay mode.
        isMyFrame = TRUE;

        // For an AP/PC in non-relay mode, check the ToDS flag
        if (Mac802_11IsAP(M802)
            && !M802->relayFrames
            && !(fHdr->frameFlags & M802_11_TO_DS))
        {
            isMyFrame = FALSE;
        }
    }

    return isMyFrame;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsMyBroadcastDataFrame
//  PURPOSE:     Determine if broadcast data frame should be received.
//  PARAMETERS:  const MacData802_11 * const M802,
//                  Pointer to 802.11 structure
//               const Message * const msg
//                  Frame message
//  RETURN:      TRUE when BSS station receives from its AP/PC
//                       except when the source ip address was itself
//                    when AP/PC receives any broadcast in relay mode
//                    when IBSS station receives any broadcast
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsMyBroadcastDataFrame(
    const MacData802_11* const M802,
    const Message* const msg)
{
    BOOL isMyFrame = FALSE;
    M802_11FrameHdr* fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);

    ERROR_Assert(fHdr->destAddr == ANY_DEST,
        "Mac802_11IsMyBroadcastDataFrame: Not a broadcast frame.\n");

    if (M802->selfAddr != M802->bssAddr)
    {
        // For a BSS station, check that the frame is from own AP
        if (fHdr->frameFlags & M802_11_FROM_DS
            && fHdr->sourceAddr == M802->bssAddr)
        {
            isMyFrame = TRUE;

            // Check that this is not own broadcast resent by the AP
            if (fHdr->address3 == M802->selfAddr)
            {
                isMyFrame = FALSE;
            }
        }
    }
    else
    {
        isMyFrame = TRUE;

        // Do not process data broadcasts from outside the BSS.
        // Stations within BSS do not send broadcasts.
        if (Mac802_11IsAP(M802)
            && !M802->relayFrames)
        {
            isMyFrame = FALSE;
        }
    }
    return isMyFrame;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IsFrameAUnicastDataTypeForPromiscuousMode
//  PURPOSE:     Determine if the frame is one of those which
//               carries data e.g. CF_DATA_ACK or M802_11_CF_DATA_POLL.
//
//               A directed frame from a BSS station sent to an AP/PC
//               with final destination as broadcast in not considered
//               a unicast data type for promiscuous mode.
//
//  PARAMETERS:  M802_11MacFrameType frameType
//                  Frame type.
//               Message* msg
//                  Received message.
//  RETURN:      TRUE if frame type is one of those which contains data
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11IsFrameAUnicastDataTypeForPromiscuousMode(
    M802_11MacFrameType frameType,
    Message* msg)
{
    BOOL isDataType = FALSE;
    M802_11FrameHdr* fHdr;

    switch (frameType)
    {
        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
        case M802_11_CF_DATA_POLL:
        case M802_11_CF_DATA_POLL_ACK:
            isDataType = TRUE;
            break;

        default:
            break;

    } //switch

    if (isDataType) {
        // Filter out broadcasts sent by BSS stations to
        // the AP/PC as unicasts.
        fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);
        if (fHdr->address3 == ANY_DEST) {
            isDataType = FALSE;
        }
    }

    return isDataType;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11SetFromAndToDsInHdr
//  PURPOSE:     Set the FromDS/ToDS flag in data frame header
//               depending on whether the AP/PC or BSS station is
//               the sender.
//
//               In case of relay, there could be a check for
//               destination which is outside BSS. However, in the
//               implementation, an IBSS station would ignore the
//               FromDS setting.
//
//  PARAMETERS:  const MacData802_11 * const M802,
//                  Pointer to 802.11 structure
//               const Message * const msg
//                  Frame message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11SetFromAndToDsInHdr(
    const MacData802_11* const M802,
    M802_11FrameHdr* const fHdr)
{
    // Set to zero since AddHeader does not memset to 0
    // Modify when other flags are implemented.
    fHdr->frameFlags = 0;

    switch (fHdr->frameType)
    {
        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
        case M802_11_CF_POLL:
        case M802_11_CF_POLL_ACK:
        case M802_11_CF_DATA_POLL:
        case M802_11_CF_DATA_POLL_ACK:
        case M802_11_CF_NULLDATA:
        case M802_11_CF_ACK:
            if (Mac802_11IsAP(M802))
            {
                fHdr->frameFlags |= M802_11_FROM_DS;
            }
            else if (Mac802_11IsBssStation(M802))
            {
                fHdr->frameFlags |= M802_11_TO_DS;
            }
            break;
        default:
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11SetFieldsInDataFrameHdr
//  PURPOSE:     Set frame type, frame flag and address fields of the
//               data frame being sent.
//
//  PARAMETERS:  const MacData802_11 * const M802,
//                  Pointer to 802.11 structure
//               M802_11FrameHdr * const fHdr,
//                  Frame header
//               NodeAddress destAddr
//                  Destination address
//               M802_11MacFrameType frameType
//                  frame type
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11SetFieldsInDataFrameHdr(
    const MacData802_11* const M802,
    M802_11FrameHdr* const fHdr,
    NodeAddress destAddr,
    M802_11MacFrameType frameType)
{
    fHdr->frameType = (unsigned char) frameType;
    Mac802_11SetFromAndToDsInHdr(M802, fHdr);

    fHdr->destAddr = destAddr;
    fHdr->sourceAddr = M802->selfAddr;
    fHdr->address3 = M802->bssAddr;
    if (M802->selfAddr != M802->bssAddr)
    {
        fHdr->address3 = M802->ipDestAddr;
    }
    else if (Mac802_11IsAP(M802))
    {
        fHdr->address3 = M802->ipSourceAddr;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpAdjustFrameTypeForAck
//  PURPOSE:     Adjust the PCF frame type in case an ack is to
//               piggy-back on it.
//  PARAMETERS:  const MacData802_11 * const M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType * const frameType
//                  Frame type
//  RETURN:      Adjusted frame type
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpAdjustFrameTypeForAck(
    const MacData802_11* const M802,
    M802_11MacFrameType* const frameType)
{
    if (*frameType == M802_11_CF_NONE) {
        return;
    }

    if (M802->cfpAckIsDue) {
        // Could use bit patterns of frame types (*frameType++)
        //
        switch (*frameType) {

            case M802_11_DATA:
                *frameType = M802_11_CF_DATA_ACK;
                break;

            case M802_11_CF_DATA_POLL:
                *frameType = M802_11_CF_DATA_POLL_ACK;
                break;

            case M802_11_CF_POLL:
                *frameType = M802_11_CF_POLL_ACK;
                break;

            case M802_11_CF_END:
                *frameType = M802_11_CF_END_ACK;
                break;

            default:
                break;

        } //switch
    } //if
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcSetTransmitState
//  PURPOSE:     Set the state of the PC based on the frame about to
//               be transmitted during CFP.
//  PARAMETERS:  const MacData802_11 * const M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType * const frameType
//                  Frame type
//               NodeAddress destAddr
//                  Destination address
//  RETURN:      Adjusted frame type
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcSetTransmitState(
    MacData802_11* const M802,
    M802_11MacFrameType frameType,
    NodeAddress destAddr)
{
    M802->cfpLastFrameType = frameType;

    switch (frameType) {

        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
            if (destAddr == ANY_DEST) {
                Mac802_11CfpSetState(M802, M802_11_X_CFP_BROADCAST);
            }
            else {
                Mac802_11CfpSetState(M802, M802_11_X_CFP_UNICAST);
            }
            break;

        case M802_11_CF_DATA_POLL:
        case M802_11_CF_DATA_POLL_ACK:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_UNICAST_POLL);
            break;

        case M802_11_CF_POLL:
        case M802_11_CF_POLL_ACK:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_POLL);
            break;

        case M802_11_CF_END:
        case M802_11_CF_END_ACK:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_END);
            break;

        case M802_11_ACK:
            // To cater to relay
            Mac802_11CfpSetState(M802, M802_11_X_CFP_ACK);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpPcSetTransmitState: "
                "Unknown frame type.\n");
            break;

    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcSetPostTransmitState
//  PURPOSE:     Set that state of PC during CFP after frame
//               transmission is complete.
//
//               Also reset ack flag is the sent frame was an ack type.
//
//  PARAMETERS:  MacData802_11 * const M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcSetPostTransmitState(
    MacData802_11* const M802)
{
    switch (M802->cfpState) {

        case M802_11_X_CFP_BROADCAST:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        case M802_11_X_CFP_UNICAST:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_WF_ACK);
            break;

        case M802_11_X_CFP_UNICAST_POLL:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_WF_DATA_ACK);
            break;

        case M802_11_X_CFP_POLL:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_WF_DATA);
            break;

        case M802_11_X_CFP_END:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        case M802_11_X_CFP_ACK:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpPcSetPostTransmitState: "
                "Unknown transmit state.\n");
            break;

    } //switch

    switch (M802->cfpLastFrameType) {
        case M802_11_CF_DATA_ACK:
        case M802_11_CF_DATA_POLL_ACK:
        case M802_11_CF_POLL_ACK:
        case M802_11_CF_END_ACK:
            M802->cfpAckIsDue = FALSE;
            break;
        default:
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationSetTransmitState
//  PURPOSE:     Set the state of the BSS station based on the frame
//               about to be transmitted during CFP.
//
//  PARAMETERS:  Frame type
//               Destination address
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationSetTransmitState(
    MacData802_11* const M802,
    M802_11MacFrameType frameType)
{
    M802->cfpLastFrameType = frameType;

    switch (frameType) {

        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_UNICAST);
            break;

        case M802_11_CF_NULLDATA:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_NULLDATA);
            break;

        case M802_11_CF_ACK:
            Mac802_11CfpSetState(M802, M802_11_X_CFP_ACK);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationSetTransmitState: "
                "Unknown frame type.\n");
            break;

    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationSetPostTransmitState
//  PURPOSE:     Set that state of BSS station during CFP after frame
//               transmission is complete.
//
//               Also reset ack flag is the sent frame was an ack type.
//
//  PARAMETERS:  MacData802_11 * const M802
//                  Pointer to 802.11 strcuture
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationSetPostTransmitState(
    MacData802_11* const M802)
{
    switch (M802->cfpState) {

        case M802_11_X_CFP_UNICAST:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_WF_ACK);
            break;

        case M802_11_X_CFP_NULLDATA:
        case M802_11_X_CFP_ACK:
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationSetPostTransmitState: "
                "Unknown transmit state.\n");
            break;

    } //switch

    switch (M802->cfpLastFrameType) {
        case M802_11_CF_DATA_ACK:
        case M802_11_CF_ACK:
            M802->cfpAckIsDue = FALSE;
            break;
        default:
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcSendsPolls
//  PURPOSE:     Determine if PC delivery mode is one in which it sends
//               polls and has a polling list.
//
//  PARAMETERS:  const M802_11_PcVars * const pc
//                  PC structure
//  RETURN:      TRUE if PC operates in one of the polling modes
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpPcSendsPolls(
    const M802_11_PcVars* const pc)
{
    ERROR_Assert(pc != NULL,
        "Mac802_11CfpPcSendsPolls: PC variable is NULL.\n");

    return (pc->deliveryMode == M802_11_PC_POLL_AND_DELIVER
        ||  pc->deliveryMode == M802_11_PC_POLL_ONLY);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11StartTimerOfGivenType
//  PURPOSE:     Set a timer of given message/event type.
//
//               Similar to Mac802_11StartTimer which sets timer for
//               MSG_MAC_TimerExpired only.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               clocktype timerDelay
//                  Delay
//               int timerType
//                  Delay for the timer
//               timer type
//                  type of timer
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11StartTimerOfGivenType(
    Node* node,
    MacData802_11* M802,
    clocktype timerDelay,
    int timerType)
{
    Message* newMsg;

    M802->timerSequenceNumber++;

    newMsg = MESSAGE_Alloc(node, MAC_LAYER, MAC_PROTOCOL_802_11,
                           timerType);
    MESSAGE_SetInstanceId(newMsg, (short) M802->myMacData->interfaceIndex);

    MESSAGE_InfoAlloc(node, newMsg, sizeof(M802->timerSequenceNumber));
    *((int*)MESSAGE_ReturnInfo(newMsg)) = M802->timerSequenceNumber;

    MESSAGE_Send(node, newMsg, timerDelay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListInit
//  PURPOSE:     Initialize the poll link list data structure.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollList **list
//                  Pointer to polling list
//  RETURN:      Poll list with memory allocated
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListInit(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollList** list)
{
    M802_11_CfPollList* tmpList;

    tmpList = (M802_11_CfPollList*) MEM_malloc(sizeof(M802_11_CfPollList));

    tmpList->size = 0;
    tmpList->first = tmpList->last = NULL;
    *list = tmpList;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListEmpty
//  PURPOSE:     See if the list is empty.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollList* list
//                  Pointer to polling list
//  RETURN:      TRUE if size is zero
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfPollListIsEmpty(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollList* list)
{
    return (list->size == 0);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListGetSize
//  PURPOSE:     Query the size of poll list
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      Number of items in poll list
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
int Mac802_11CfPollListGetSize(
    Node* node,
    MacData802_11* M802)
{
    return M802->pcVars->cfPollList->size;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListInsert
//  PURPOSE:     Insert an item to the end of the list.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollList* list
//                  Pointer to polling list
//               M802_11_CfPollStation* data
//                  Poll station data for item to insert
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListInsert(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollList* list,
    M802_11_CfPollStation* data)
{
    M802_11_CfPollListItem* listItem =
        (M802_11_CfPollListItem*)
        MEM_malloc(sizeof(M802_11_CfPollListItem));

    listItem->data = data;

    listItem->next = NULL;

    if (list->size == 0) {
        // Only item in the list
        listItem->prev = NULL;
        list->last = listItem;
        list->first = list->last;
    }
    else {
        // Insert at end of list
        listItem->prev = list->last;
        list->last->next = listItem;
        list->last = listItem;
    }

    list->size++;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListItemRemove
//  PURPOSE:     Remove an item from the poll list.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollList* list
//                  Pointer to polling list
//               M802_11_CfPollListItem* listItem
//                  List item to remove
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListItemRemove(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollList* list,
    M802_11_CfPollListItem* listItem)
{
    M802_11_CfPollListItem* nextListItem;

    if (list == NULL || listItem == NULL) {
        return;
    }

    if (list->size == 0) {
        return;
    }

    nextListItem = listItem->next;

    if (list->size == 1) {
        list->first = list->last = NULL;
    }
    else if (list->first == listItem) {
        list->first = listItem->next;
        list->first->prev = NULL;
    }
    else if (list->last == listItem) {
        list->last = listItem->prev;
        if (list->last != NULL) {
            list->last->next = NULL;
        }
    }
    else {
        listItem->prev->next = listItem->next;
        if (listItem->prev->next != NULL) {
            listItem->next->prev = listItem->prev;
        }
    }

    list->size--;

    if (listItem->data != NULL) {
        MEM_free(listItem->data);
    }

    MEM_free(listItem);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListFree
//  PURPOSE:     Free the entire list.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollList* list
//                  Pointer to polling list
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListFree(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollList* list)
{
    M802_11_CfPollListItem *item;
    M802_11_CfPollListItem *tempItem;

    item = list->first;

    while (item) {
        tempItem = item;
        item = item->next;

        MEM_free(tempItem->data);
        MEM_free(tempItem);
    }

    if (list != NULL) {
        MEM_free(list);
    }

    list = NULL;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListGetFirst
//  PURPOSE:     Retrieve the first item in the poll list
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      First list item
//               NULL for empty list
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
M802_11_CfPollListItem * Mac802_11CfPollListGetFirst(
    Node* node,
    MacData802_11* M802)
{
    return M802->pcVars->cfPollList->first;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListGetNextItem
//  PURPOSE:     Retrieve the next item given the previous item.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollListItem *prev
//                  Pointer to prev item.
//  RETURN:      First item when previous item is NULL
//                          when previous item is the last
//                          when there is only one item in the list
//               NULL if the list is empty
//               Next item after previous item
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
M802_11_CfPollListItem * Mac802_11CfPollListGetNextItem(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollListItem *prev)
{
    M802_11_CfPollListItem* next = NULL;
    M802_11_CfPollList* pollList = M802->pcVars->cfPollList;

    if (!Mac802_11CfPollListIsEmpty(node, M802, pollList)) {
        if (prev == pollList->last || prev == NULL) {
            next = pollList->first;
        }
        else {
            next = prev->next;
        }
    }

    return next;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListResetAtStartOfCfp
//  PURPOSE:     Reset values of the poll list at start of CFP
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListResetAtStartOfCfp(
    Node* node,
    MacData802_11* M802)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollListItem* next = NULL;
    M802_11_CfPollListItem* first;

    ERROR_Assert(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc),
        "Mac802_11CfPollListResetAtStartOfCfp: "
        "Poll list not applicable to PC configuration.\n");

    if (!Mac802_11CfPollListIsEmpty(node, M802, pc->cfPollList)) {
        // It is simpler to reset all items
        next = first = Mac802_11CfPollListGetFirst(node, M802);

        do {
            next->data->flags = 0;
            next = Mac802_11CfPollListGetNextItem(node, M802, next);

        } while (next != first);

    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListGetItemWithGivenAddress
//  PURPOSE:     Retrieve the list item related to given address
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress lookupAddress
//                  Address to lookup
//  RETURN:      List item corresponding to address
//               NULL otherwise
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
M802_11_CfPollListItem* Mac802_11CfPollListGetItemWithGivenAddress(
    Node* node,
    MacData802_11* M802,
    NodeAddress lookupAddress)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollListItem* next = NULL;
    M802_11_CfPollListItem* first;

    if (!Mac802_11CfPollListIsEmpty(node, M802, pc->cfPollList)) {
        next = first = Mac802_11CfPollListGetFirst(node, M802);

        do {
            if (next->data->macAddr == lookupAddress) {
                return next;
            }
            next = Mac802_11CfPollListGetNextItem(node, M802, next);

        } while (next != first);

    }

    return NULL;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListIsAddressPollable
//  PURPOSE:     Determine if given address is pollable
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address,
//                  Node address
//  RETURN:      TRUE if address is found and is pollable
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfPollListIsAddressPollable(
    Node* node,
    MacData802_11* M802,
    NodeAddress address)
{
    M802_11_PcVars* pc = M802->pcVars;
    BOOL isPollable = FALSE;
    M802_11_CfPollListItem* item = NULL;

    ERROR_Assert(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc),
        "Mac802_11CfPollListIsAddressPollable: "
        "Poll list not applicable to PC configuration.\n");

    if (address != ANY_DEST) {
        item = Mac802_11CfPollListGetItemWithGivenAddress(node,
                    M802, address);

        if (item != NULL
            && item->data->pollType == M802_11_STA_POLLABLE_POLL)
        {
            isPollable = TRUE;
        }
    }
    return isPollable;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListUpdateForNullDataResponse
//  PURPOSE:     Update poll list when station responds with null data
//               and also when PC timeouts in response to a poll.
//
//               When poll save is by count, increment counter upto
//               a max value.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address,
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListUpdateForNullDataResponse(
    Node* node,
    MacData802_11* M802,
    NodeAddress address)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollListItem* matchedListItem;

    ERROR_Assert(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc),
        "Mac802_11CfPollListUpdateForNullDataResponse: "
        "Poll list not applicable to PC configuration.\n");

    if (pc->pollSaveMode == M802_11_POLL_SAVE_BYCOUNT) {
        matchedListItem = Mac802_11CfPollListGetItemWithGivenAddress(
            node, M802, address);

        // Increment nulldata count upto a max value
        if (matchedListItem
            && matchedListItem->data->nullDataCount
                < pc->pollSaveMaxCount)
        {
            matchedListItem->data->nullDataCount++;
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListUpdateForUnicastReceived
//  PURPOSE:     Update poll list when PC receives a unicast from BSS
//               station.
//
//               When poll save is by count, reset nulldata counter.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Node address of unicast sender station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListUpdateForUnicastReceived(
    Node* node,
    MacData802_11* M802,
    NodeAddress sourceAddr)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollListItem* matchedListItem;

    if (!(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc))) {
        return;
    }

    if (pc->pollSaveMode == M802_11_POLL_SAVE_BYCOUNT) {
        matchedListItem = Mac802_11CfPollListGetItemWithGivenAddress(
            node, M802, sourceAddr);

        if (matchedListItem) {
            matchedListItem->data->nullDataCount = 0;
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11Trace
//  PURPOSE:     Common entry routine for trace.
//
//               Placeholder for code to be inserted.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Flag which is either "Send" or "Receive" or comment
//               FILE* fp
//                  File pointer
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11Trace(
    Node* node,
    MacData802_11* M802,
    Message* msg,
    char *flag)
{
    return;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TraceInit
//  PURPOSE:     Initialize trace setting from user configuration file.
//
//               Placeholder for code to be inserted.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11TraceInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802)
{
    return;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfPollListCreateFromUserInput
//  PURPOSE:     Use the SUBNET parsed user input to create a polling list.
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               MacSubnetMemberData* subnetList
//                  Pointer to subnet list
//               int nodesInSubnet
//                  Number of nodes in subnet
//               int subnetListIndex
//                  Subnet list index
//               NodeAddress subnetAddress
//                  Subnet address
//               int numHostBits
//                  Number of host bits
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfPollListCreateFromUserInput(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType = NETWORK_IPV4,
    /*unsigned int tla = 0,
    unsigned int nla = 0,
    unsigned int sla = 0*/
    in6_addr *subnetAddr = 0,
    unsigned int subnetPrefixLen = 0)
{
    M802_11_PcVars* pc = M802->pcVars;
    char input[MAX_STRING_LENGTH];
    BOOL wasFound = FALSE;
    M802_11_CfPollStation pollStation;
    M802_11_CfPollListItem* listItem = NULL;
    int listSize = 0;
    int i;
    Address address;
    if (networkType != NETWORK_IPV6)
    {
        ERROR_Assert(pow(2.0, numHostBits) >= nodesInSubnet,
        "Mac802_11CfPollListCreateFromUserInput: "
        "Number of host bits not large enough for input node count.\n");
    }

    // Allocate initial memory for poll list
    Mac802_11CfPollListInit(node, M802, &(pc->cfPollList));

    for (i = 0; i < nodesInSubnet; i++) {
        // Avoid including self
        if (i != subnetListIndex) {

            if (networkType == NETWORK_IPV6)
            {
                in6_addr tempAddr = {{{0}}};
                int interfaceIndex = 0;
                if (!MAPPING_GetInterfaceAddressForSubnet(
                        node,
                        subnetList[i].nodeId,
                        subnetAddr,
                        subnetPrefixLen,
                        &tempAddr,
                        &interfaceIndex))
                {
                    ERROR_ReportError(
                        "Mac802_11CfPollListCreateFromUserInput: "
                        "Invalid mapping while creating polling list.\n");
                }
                pollStation.macAddr = MAPPING_CreateIpv6LinkLayerAddr(
                                subnetList[i].nodeId,
                                interfaceIndex);
                SetIPv6AddressInfo(&address, tempAddr);
            }
            else
            {
                pollStation.macAddr = MAPPING_GetInterfaceAddressForSubnet(
                    node, subnetList[i].nodeId,
                    subnetAddress, numHostBits);

                if (pollStation.macAddr == INVALID_MAPPING) {
                    ERROR_ReportError(
                        "Mac802_11CfPollListCreateFromUserInput: "
                        "Invalid mapping while creating polling list.\n");
                }
                SetIPv4AddressInfo(&address, pollStation.macAddr);
            }

            // Read station poll type
            //      MAC-802.11-STATION-POLL-TYPE NOT-POLLABLE
            //                                   | POLLABLE-DONT-POLL
            //                                   | POLLABLE
            // Default is M802_11_STATION_POLL_TYPE_DEFAULT

            pollStation.nullDataCount = 0;
            pollStation.pollCount = 0;

            IO_ReadString(
                subnetList[i].nodeId,
                &address,
                nodeInput,
                "MAC-802.11-STATION-POLL-TYPE",
                &wasFound,
                input);

            if (wasFound)
            {
                if (!strcmp(input, "NOT-POLLABLE")) {
                    pollStation.pollType = M802_11_STA_NOT_POLLABLE;
                }
                else if (!strcmp(input, "POLLABLE-DONT-POLL")) {
                    pollStation.pollType = M802_11_STA_POLLABLE_DONT_POLL;
                }
                else if (!strcmp(input, "POLLABLE")) {
                    pollStation.pollType = M802_11_STA_POLLABLE_POLL;
                }
                else {
                    ERROR_ReportError(
                        "Mac802_11CfPollListCreateFromUserInput: "
                        "Invalid value for MAC-802.11-STATION-POLL-TYPE "
                        "in configuration file.\n"
                        "Expecting NOT-POLLABLE | POLLABLE-DONT-POLL |"
                        "POLLABLE \n");
                }
            }
            else {
                pollStation.pollType = M802_11_STATION_POLL_TYPE_DEFAULT;
            }

            if (pollStation.pollType == M802_11_STA_POLLABLE_DONT_POLL
                || pollStation.pollType == M802_11_STA_POLLABLE_POLL)
            {
                M802_11_CfPollStation *aStation = (M802_11_CfPollStation*)
                    MEM_malloc(sizeof(M802_11_CfPollStation));
                memcpy(aStation, &pollStation,
                    sizeof(M802_11_CfPollStation));
                Mac802_11CfPollListInsert(node, M802,
                    pc->cfPollList, aStation);
            }
        }
    } // for i

    // Count the number of CF-Pollable stations.
    // Initialize polling variables
    listItem = NULL;
    listSize = Mac802_11CfPollListGetSize(node, M802);

    pc->prevPolledItem = NULL;
    pc->pollableCount = 0;

    for (i = 0; i < listSize; i++) {
        listItem = Mac802_11CfPollListGetNextItem(node, M802, listItem);
        if (listItem->data->pollType == M802_11_STA_POLLABLE_POLL) {
            pc->pollableCount++;

            // This effectively initializes the item to be the last
            // pollable one on the list
            pc->prevPolledItem = listItem;
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcPollSaveInit
//  PURPOSE:     Initialize PC poll save mode values from user configuration
//  PARAMETERS:  Node* node
//                  Pointer to node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcPollSaveInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802)
{
    M802_11_PcVars* pc = M802->pcVars;
    char input[MAX_STRING_LENGTH];
    BOOL wasFound = FALSE;
    int pollSaveMin, pollSaveMax;
    Address address;
    // Read PC poll save mode.
    // Format is :
    //       MAC-802.11-PC-POLL-SAVE NONE | BY-COUNT
    // Default is M802_11_POLL_SAVE_MODE_DEFAULT
    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);
    IO_ReadString(
         node->nodeId,
         &address, //M802->selfAddr,
         nodeInput,
         "MAC-802.11-PC-POLL-SAVE",
         &wasFound,
         input);

    if (wasFound == TRUE) {
        if (!strcmp(input, "NONE")) {
            pc->pollSaveMode = M802_11_POLL_SAVE_NONE;
        }
        else if (!strcmp(input, "BY-COUNT")) {
            pc->pollSaveMode = M802_11_POLL_SAVE_BYCOUNT;
        }
        else {
            ERROR_ReportError("Mac802_11CfpPcPollSaveInit: "
                "Invalid value for MAC-802.11-PC-POLL-SAVE "
                "in configuration file.\n"
                "Expecting NONE or BY-COUNT\n");
        }
    }
    else {
        pc->pollSaveMode = M802_11_POLL_SAVE_MODE_DEFAULT;
    }

    if (pc->pollSaveMode == M802_11_POLL_SAVE_BYCOUNT) {
        // Read min threshold for the BY-COUNT PC poll save mode.
        // Format is :
        //      MAC-802.11-PC-POLL-SAVE-MIN 1
        // Default is M802_11_POLL_SAVE_MIN_COUNT_DEFAULT
        IO_ReadInt(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-PC-POLL-SAVE-MIN",
            &wasFound,
            &pollSaveMin);

        if (wasFound) {
            ERROR_Assert(pollSaveMin >= 0,
                "Mac802_11CfpPcPollSaveInit: "
                "MAC-802.11-PC-POLL-SAVE-MIN in "
                "configuration file is not positive.\n");
            pc->pollSaveMinCount = pollSaveMin;
        }
        else {
            pc->pollSaveMinCount = M802_11_POLL_SAVE_MIN_COUNT_DEFAULT;
        }

        // Read min threshold for the BY-COUNT PC poll save mode.
        // Format is :
        //       MAC-802.11-PC-POLL-SAVE-MAX 10
        //  Default is M802_11_POLL_SAVE_MAX_COUNT_DEFAULT
        IO_ReadInt(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-PC-POLL-SAVE-MAX",
            &wasFound,
            &pollSaveMax);

        if (wasFound) {
            ERROR_Assert(pollSaveMax >= 0,
                "Mac802_11CfpPcPollSaveInit: "
                "MAC-802.11-PC-POLL-SAVE-MAX in "
                "configuration file is not a positive value.\n");
            pc->pollSaveMaxCount = pollSaveMax;
        }
        else {
            pc->pollSaveMaxCount = M802_11_POLL_SAVE_MAX_COUNT_DEFAULT;
        }

        ERROR_Assert(pc->pollSaveMinCount < pc->pollSaveMaxCount,
            "Mac802_11CfpPcPollSaveInit: "
            "MAC-802.11-PC-POLL-SAVE-MAX is <= "
            "MAC-802.11-PC-POLL-SAVE-MIN.\n");
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpDurationInit
//  PURPOSE:     Read the user specified duration of the CFP.
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               clocktype* cfpMaxDuration
//                  Read or default value of max. duration.
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpDurationInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    clocktype* cfpMaxDuration)

{
    BOOL wasFound = FALSE;
    int cfpDuration;
    Address address;
    // The contention free duration is specified in TUs (Time units)
    // where each time unit is 1024US
    //    [0.6] MAC-802.11-PC-CONTENTION-FREE-DURATION 50
    // and should be in the range 1 to M802_11_BEACON_INTERVAL_MAX TUs
    // Default is M802_11_CFP_MAX_DURATION_DEFAULT
    //
    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);
    if (address.networkType == NETWORK_IPV6)
    {
        int interfaceIndex = M802->bssAddr & 0xff;
        in6_addr addr6 = {{{0}}};

        MAPPING_GetIpv6GlobalAddressForInterface(
            node,
            MAPPING_GetNodeIdFromLinkLayerAddr(node, M802->bssAddr),
            interfaceIndex,
            &addr6);
        SetIPv6AddressInfo(&address, addr6);
    }
    else
    {
        SetIPv4AddressInfo(&address, M802->bssAddr);
    }

    IO_ReadInt(
        MAPPING_GetNodeIdFromLinkLayerAddr(node, M802->bssAddr),
        &address, //M802->bssAddr,
        nodeInput,
        "MAC-802.11-PC-CONTENTION-FREE-DURATION",
        &wasFound,
        &cfpDuration);

    if (wasFound)
    {
        ERROR_Assert(cfpDuration > 0
            && cfpDuration < M802_11_BEACON_INTERVAL_MAX,
            "Mac802_11CfpPcInit: "
            "Out of range value for MAC-802.11-PC-CONTENTION-FREE-DURATION "
            "in configuration file.\n");
    }
    else
    {
        cfpDuration = M802_11_CFP_MAX_DURATION_DEFAULT;
    }

    *cfpMaxDuration = Mac802_11TUsToClocktype((unsigned short) cfpDuration);
    return;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcInit
//  PURPOSE:     Initialize Point Coordinator related user configuration.
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               MacSubnetMemberData* subnetList
//                  Pointer to subnet list
//               int nodesInSubnet
//                  Number of nodes in subnet
//               int subnetListIndex
//                  Subnet list index
//               NodeAddress subnetAddress
//                  Subnet address
//               int numHostBits
//                  Number of host bits
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType = NETWORK_IPV4,
    /*unsigned int tla = 0,
    unsigned int nla = 0,
    unsigned int sla = 0*/
    in6_addr *subnetAddr = 0,
    unsigned int subnetPrefixLen = 0)
{
    M802_11_PcVars* pc = M802->pcVars;
    char input[MAX_STRING_LENGTH];
    BOOL wasFound = FALSE;
    clocktype cfpDuration;
    Address address;
    // A PC may operate as
    // send only where
    //      it does not poll stations but only delivers queued data
    // or as poll-only where
    //      buffered data may piggyback on the polls
    // or as poll+deliver where
    //      the poll only mode is followed by send only
    //
    // MAC-802.11-PC-DELIVERY-MODE DELIVER-ONLY
    //                             | POLL-ONLY
    //                             | POLL-AND-DELIVER
    // Default is M802_11_PC_DELIVERY_MODE_DEFAULT

    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);
    if (address.networkType == NETWORK_IPV6)
    {
        if (M802->selfAddr != INVALID_ADDRESS)
        {
            int interfaceIndex = M802->selfAddr & 0xff;
            MAPPING_GetIpv6GlobalAddressForInterface(
                node,
                node->nodeId,
                interfaceIndex,
                &address.interfaceAddr.ipv6);
        }
    }
    else
    {
        address.interfaceAddr.ipv4 = M802->selfAddr;
    }

    IO_ReadString(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-PC-DELIVERY-MODE",
        &wasFound,
        input);

    if (wasFound) {
        if (!strcmp(input, "DELIVER-ONLY")) {
            pc->deliveryMode = M802_11_PC_DELIVER_ONLY;
        }
        else if (!strcmp(input, "POLL-AND-DELIVER")) {
            pc->deliveryMode = M802_11_PC_POLL_AND_DELIVER;
        }
        else if (!strcmp(input, "POLL-ONLY")) {
            pc->deliveryMode = M802_11_PC_POLL_ONLY;
        }
        else {
            ERROR_ReportError("Mac802_11CfpPcInit: "
                "Invalid value for MAC-802.11-PC-DELIVERY-MODE "
                "in configuration file.\n "
                "Expecting DELIVER-ONLY | POLL-ONLY | POLL-AND-DELIVER.\n");
        }
    }
    else {
        pc->deliveryMode = M802_11_PC_DELIVERY_MODE_DEFAULT;
    }


    if (Mac802_11CfpPcSendsPolls(pc)) {
        // For these modes, need to create the polling list.
        // To be later change based on configuration
        //        MAC-801-11-PC-POLLING-LIST USER-INPUT | ASSOCIATION
        if (networkType == NETWORK_IPV6)
        {
            Mac802_11CfPollListCreateFromUserInput(
                node, nodeInput, M802, subnetList, nodesInSubnet,
                subnetListIndex, 0, 0,
                networkType,subnetAddr,subnetPrefixLen);
        }
        else
        {
        Mac802_11CfPollListCreateFromUserInput(
            node, nodeInput, M802, subnetList, nodesInSubnet,
            subnetListIndex, subnetAddress, numHostBits);
    }
    }

    // Read the poll save mode
    if (Mac802_11CfpPcSendsPolls(pc)) {
        Mac802_11CfpPcPollSaveInit(node, nodeInput, M802);
    }


    // Read in the contention free duration, e.g.
    //    [0.6] MAC-802.11-PC-CONTENTION-FREE-DURATION 50
    //
    Mac802_11CfpDurationInit(node, nodeInput, M802, &M802->cfpMaxDuration);

    // The duration should be smaller than the beacon interval
    // to allow at least one data frame to be sent during the
    // contention period. Note that min BO is added here.
    //
    cfpDuration =
        (M802->extraPropDelay + M802->difs
         + M802->cwMin * M802->slotTime
         + PHY_GetTransmissionDuration(node, M802->myMacData->phyNumber,
                                       M802->lowestDataRateType,
                                       M802_11_LONG_CTRL_FRAME_SIZE)
         + M802->extraPropDelay + M802->sifs
         + PHY_GetTransmissionDuration(node, M802->myMacData->phyNumber,
                                       M802->lowestDataRateType,
                                       M802_11_SHORT_CTRL_FRAME_SIZE)
         + M802->extraPropDelay + M802->sifs
         + PHY_GetTransmissionDuration(node, M802->myMacData->phyNumber,
                                       M802->lowestDataRateType,
                                       GetNetworkIPFragUnit(node))
         + M802->extraPropDelay + M802->sifs
         + PHY_GetTransmissionDuration(node, M802->myMacData->phyNumber,
                                       M802->lowestDataRateType,
                                       M802_11_SHORT_CTRL_FRAME_SIZE)
         + M802->extraPropDelay);
    ERROR_Assert(
        M802->cfpBeaconInterval - M802->cfpMaxDuration >= cfpDuration,
        "Mac802_11CfpPcInit: "
        "The contention free duration does not allow "
        "enough time for the contention period.\n"
        "Either reduce the value of MAC-802.11-PC-CONTENTION-FREE-DURATION\n"
        "or increase the beacon interval (MAC-802.11-BEACON-INTERVAL).\n");

    // The duration should be large enough to allow 2 full
    // frames plus time for a beacon and CF-End
    // The duration should be large enough to allow 2 full frames
    // plus time for a beacon and CF-End
    cfpDuration =
       (M802->pifs + M802->extraPropDelay
        + PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            M802->lowestDataRateType,
            sizeof(M802_11_BeaconFrame))
        + M802->sifs
        + 2 * (M802->extraPropDelay
               + PHY_GetTransmissionDuration(
                   node, M802->myMacData->phyNumber,
                   M802->lowestDataRateType,
                   GetNetworkIPFragUnit(node))
               + M802->sifs
               + M802->extraPropDelay)
        + PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            M802->lowestDataRateType,
            M802_11_LONG_CTRL_FRAME_SIZE)
        + M802->extraPropDelay);

    ERROR_Assert(M802->cfpMaxDuration > cfpDuration,
        "Mac802_11CfpPcInit: "
        "The contention free duration is too small for packet exchange.\n"
        "Increase the value of MAC-802.11-PC-CONTENTION-FREE-DURATION.\n");
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationInit
//  PURPOSE:     Initialize Station related values from user configuration.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802)
{
    // For static association, read the contention free duration of the PC
    //    [0.6] MAC-802.11-PC-CONTENTION-FREE-DURATION 50
    //
    Mac802_11CfpDurationInit(node, nodeInput, M802, &M802->cfpMaxDuration);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11BeaconInit
//  PURPOSE:     Initialize beacon related variables from user configuration
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11BeaconInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802)
{
    BOOL wasFound = FALSE;
    int beaconInterval;
    int beaconStart;
    Address address;
    NodeId nodeId;
    // The beacon interval is specified in TUs (Time units)
    //  where each time unit is 1024US
    //       [0.6] MAC-802.11-BEACON-INTERVAL 200
    //  and should be in the range 1 to M802_11_BEACON_INTERVAL_MAX TUs
    //  Default is M802_11_BEACON_INTERVAL_DEFAULT

    beaconInterval = M802_11_BEACON_INTERVAL_DEFAULT;

    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);

    if (address.networkType == NETWORK_IPV6)
    {
        nodeId = MAPPING_GetNodeIdFromLinkLayerAddr(node, M802->bssAddr);
    }
    else
    {
        nodeId = MAPPING_GetNodeIdFromInterfaceAddress(node, M802->bssAddr);
    }

    IO_ReadInt(
        nodeId,
        &address,  //M802->bssAddr,
        nodeInput,
        "MAC-802.11-BEACON-INTERVAL",
        &wasFound,
        &beaconInterval);

    if (wasFound) {
        ERROR_Assert(beaconInterval > 0
            && beaconInterval <= M802_11_BEACON_INTERVAL_MAX,
            "Mac802_11BeaconInit: "
            "Out of range value for MAC-802.11-BEACON-INTERVAL in "
            "configuration file.\n"
            "Valid range is 1 to 32767.\n");
    }

    M802->cfpBeaconInterval = Mac802_11TUsToClocktype(
        (unsigned short) beaconInterval);

    // For overlapping BSSs, the beacon start time (in TUs)
    // is best configured to eliminate overlapping CF periods.
    // The value offsets the first beacon from time 0 and
    // should be in the range 1 to beacon interval.
    //       [0.6] MAC-802.11-BEACON-START-TIME 30
    //  Default is M802_11_BEACON_START_DEFAULT

    beaconStart = M802_11_BEACON_START_DEFAULT;

    IO_ReadInt(
        nodeId,
        &address,
        nodeInput,
        "MAC-802.11-BEACON-START-TIME",
        &wasFound,
        &beaconStart);

    if (wasFound) {
        ERROR_Assert(beaconStart > 0
            && beaconStart
                <= Mac802_11ClocktypeToTUs(M802->cfpBeaconInterval),
            "Mac802_11BeaconStartInit: "
            "Out of range value for MAC-802.11-BEACON-START-TIME "
            "in configuration file.\n"
            "Valid range is 1 to beacon interval.\n");
    }

    M802->cfpBeaconTime =
            Mac802_11TUsToClocktype((unsigned short)beaconStart);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ApInit
//  PURPOSE:     Initialize AP related variables from user configuration.
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               MacSubnetMemberData* subnetList
//                  Pointer to subnet list
//               int nodesInSubnet
//                  Number of nodes in subnet
//               int subnetListIndex
//                  Subnet list index
//               NodeAddress subnetAddress
//                  Subnet address
//               int numHostBits
//                  Number of host bits
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11ApInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType = NETWORK_IPV4,
    /*unsigned int tla = 0,
    unsigned int nla = 0,
    unsigned int sla = 0*/
    in6_addr *subnetAddr = 0,
    unsigned int subnetPrefixLen = 0)
{
    BOOL wasFound = FALSE;
    char input[MAX_STRING_LENGTH];
    Address address;
    // Get beacon related variables.
    // Though the implementation currently handles only CFP beacons,
    // they could be extended to APs as well for dynamic association.
    Mac802_11BeaconInit(node, nodeInput, M802);

    // An AP (or PC) can relay frames to/from stations outside the BSS.
    // Such stations can be other AP/PCs or IBSS stations.
    // BSS stations never relay. IBSS stations always relay.
    //       MAC-802.11-RELAY-FRAMES YES | NO
    // Default is M802_11_RELAY_FRAMES_DEFAULT

    M802->relayFrames = M802_11_RELAY_FRAMES_DEFAULT;

    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);
    if (address.networkType == NETWORK_IPV6)
    {
        if (M802->selfAddr != INVALID_ADDRESS)
        {
            int interfaceIndex = M802->selfAddr & 0xff;
            MAPPING_GetIpv6GlobalAddressForInterface(
                node,
                node->nodeId,
                interfaceIndex,
                &address.interfaceAddr.ipv6);
        }
    }
    else
    {
        address.interfaceAddr.ipv4 = M802->selfAddr;
    }

    IO_ReadString(
        node->nodeId,
        &address,    //M802->selfAddr,
        nodeInput,
        "MAC-802.11-RELAY-FRAMES",
        &wasFound,
        input);

    if (wasFound) {
        if (!strcmp(input, "YES")) {
            M802->relayFrames = TRUE;
        }
        else if (!strcmp(input, "NO")) {
            M802->relayFrames = FALSE;
        }
        else {
            ERROR_ReportError("Mac802_11ApInit: "
                "Invalid value for MAC-802.11-RELAY-FRAMES "
                "in configuration file.\n"
                "Expecting YES or NO.\n");
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11BssInit
//  PURPOSE:     Initialize BSS related variables.
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               MacSubnetMemberData* subnetList
//                  Pointer to subnet list
//               int nodesInSubnet
//                  Number of nodes in subnet
//               int subnetListIndex
//                  Subnet list index
//               NodeAddress subnetAddress
//                  Subnet address
//               int numHostBits
//                  Number of host bits
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11BssInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType = NETWORK_IPV4,
    /*unsigned int tla = 0,
    unsigned int nla = 0,
    unsigned int sla = 0*/
    in6_addr *subnetAddr = 0,
    unsigned int subnetPrefixLen = 0)
{
    char input[MAX_STRING_LENGTH];
    NodeAddress anApAddress;
    in6_addr anApAddress6;
    Address address;
    int interfaceIndex = M802->myMacData->interfaceIndex;
    BOOL wasFound = FALSE;

    // For an infrastructure network, the access point is given by
    // MAC-802.11-AP e.g.
    //       SUBNET N8-1.0 (20 thru 40)
    //       [N8-1.0] MAC-802.11-AP 25
    // would indicate that node 25 was the access point.
    //
    // If not specified, the subnet is assumed to be an IBSS.
    NetworkGetInterfaceInfo(node, interfaceIndex, &address);

    IO_ReadString(
        node->nodeId,
        &address,
        nodeInput,
        "MAC-802.11-AP",
        &wasFound,
        input);

    if (wasFound) {
        NodeId isNodeId = 0;
        if (networkType == NETWORK_IPV6)
        {
            anApAddress6 = address.interfaceAddr.ipv6;
            IO_ParseNodeIdOrHostAddress(
            input,
                &anApAddress6,
            &isNodeId);
        }
        else
        {
            anApAddress = address.interfaceAddr.ipv4;
            IO_ParseNodeIdOrHostAddress(
                input,
                &anApAddress,
                (BOOL*)&isNodeId);
        }
        if (isNodeId) {
            // Input for ACCESS-POINT is a node ID

            // In this case, verify that access point is one of
            // the nodes in subnet by checking that the address of
            // one of its interfaces is part of subnetList.
            if (networkType == NETWORK_IPV6)
            {
                anApAddress = MAPPING_GetInterfaceAddressForSubnet(
                    node, atoi(input),subnetAddr,subnetPrefixLen,
                    &anApAddress6, &interfaceIndex);
            }
            else
            {
                anApAddress = MAPPING_GetInterfaceAddressForSubnet(
                    node, atoi(input), subnetAddress, numHostBits);
            }
            ERROR_Assert(anApAddress != INVALID_MAPPING,
                "Mac802_11BssInit: "
                "Erroneous MAC-802.11-AP in configuration file.\n"
                "The node for the AP is not a node in the SUBNET list.\n");

            if (networkType == NETWORK_IPV6) {
                M802->bssAddr = MAPPING_CreateIpv6LinkLayerAddr(isNodeId, //node->nodeId,
                                    interfaceIndex);
            }
            else {
            M802->bssAddr = anApAddress;
        }

        }
        else  {
            // Input for ACCESS-POINT is an IP address.

            // Verify that access point address is part of subnetList
            if (networkType == NETWORK_IPV6)
            {
                int i;
                int infIndex;
                in6_addr aListAddress;
                anApAddress6 = GetIPv6Address(address);
                for (i = 0; i < nodesInSubnet; i++) {
                    MAPPING_GetInterfaceAddressForSubnet(
                        node, subnetList[i].nodeId,
                       subnetAddr,subnetPrefixLen, &aListAddress, &infIndex);
                    if (SAME_ADDR6(aListAddress, anApAddress6)) {
                        break;
                    }
                }
                ERROR_Assert(SAME_ADDR6(aListAddress, anApAddress6),
                "Mac802_11BssInit: "
                "Erroneous MAC-802.11-AP in configuration file.\n"
                "The address for the AP in not part of the SUBNET list.\n");

                M802->bssAddr = MAPPING_CreateIpv6LinkLayerAddr(node->nodeId,
                                interfaceIndex);
            }
            else
            {
                int i;
                NodeAddress aListAddress;
                for (i = 0; i < nodesInSubnet; i++) {
                    aListAddress = MAPPING_GetInterfaceAddressForSubnet(
                        node, subnetList[i].nodeId,
                        subnetAddress, numHostBits);
                    if (aListAddress == anApAddress) {
                        break;
                    }
                }
                aListAddress = anApAddress;

                ERROR_Assert(anApAddress == aListAddress,
                             "Mac802_11BssInit: "
                             "Erroneous MAC-802.11-AP in configuration file.\n"
                             "The address for the AP in not part of the SUBNET list.\n");

                M802->bssAddr = anApAddress;
            }
        }

        if (M802->selfAddr == M802->bssAddr) {
            M802->stationType = M802_11_STA_AP;

            // Init access point variables
            if (networkType == NETWORK_IPV6) {
                Mac802_11ApInit(node, nodeInput, M802,
                    subnetList, nodesInSubnet, subnetListIndex,
                    0, 0, networkType,subnetAddr,subnetPrefixLen);
            }
            else
            {
            Mac802_11ApInit(node, nodeInput, M802,
                subnetList, nodesInSubnet, subnetListIndex,
                subnetAddress, numHostBits);
        }
        }
        else {
            M802->stationType = M802_11_STA_BSS;

            // For static association, get beacon related variables.
            Mac802_11BeaconInit(node, nodeInput, M802);
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcPrintStats
//  PURPOSE:     Print CFP statistics for a PC.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               int interfaceIndex
//                  Interface index
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcPrintStats(
    Node* node,
    MacData802_11* M802,
    int interfaceIndex)
{
    char buf[MAX_STRING_LENGTH];

    sprintf(buf, "Unicasts sent and acked = %d",
           M802->unicastPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Broadcasts sent = %d",
           M802->broadcastPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Unicasts received = %d",
           M802->unicastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Broadcasts received = %d",
           M802->broadcastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data-Polls transmitted = %d",
           M802->dataPollPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Polls transmitted = %d",
           M802->pollPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data packets transmitted = %d",
           M802->dataPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data packets received = %d",
           M802->dataPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "NullData received = %d",
           M802->nulldataPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Beacons transmitted = %d",
           M802->beaconPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "CF Ends transmitted = %d",
           M802->endPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Beacons received = %d",
           M802->beaconPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "CF Ends received = %d",
           M802->endPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    //sprintf(buf, "Polls saved = %d",
    //       M802->pollsSavedCfp);
    //IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    ERROR_Assert(
        M802->nulldataPacketsSentCfp == 0 &&
        M802->dataPollPacketsGotCfp == 0 &&
        M802->pollPacketsGotCfp == 0,
        "Mac802_11PrintStatsPc: "
        "One or more stat values is unexpectedly non-zero.\n");
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationPrintStats
//  PURPOSE:     Print CFP statistics for a station in BSS.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               int interfaceIndex
//                  Interface index
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationPrintStats(
    Node* node,
    MacData802_11* M802,
    int interfaceIndex)
{
    char buf[MAX_STRING_LENGTH];

    sprintf(buf, "Unicasts sent and acked = %d",
           M802->unicastPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Unicasts received = %d",
           M802->unicastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Broadcasts received = %d",
           M802->broadcastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data packets transmitted = %d",
           M802->dataPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "NullData transmitted = %d",
           M802->nulldataPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data-Polls received = %d",
           M802->dataPollPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Polls received = %d",
           M802->pollPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Data packets received = %d",
           M802->dataPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Beacons received = %d",
           M802->beaconPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "CF Ends received = %d",
           M802->endPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11PCF", ANY_DEST, interfaceIndex, buf);

    ERROR_Assert(
        M802->beaconPacketsSentCfp == 0 &&
        M802->endPacketsSentCfp == 0 &&
        M802->broadcastPacketsSentCfp == 0 &&
        M802->dataPollPacketsSentCfp == 0 &&
        M802->pollPacketsSentCfp == 0 &&
        M802->nulldataPacketsGotCfp == 0 &&
        M802->pollsSavedCfp == 0,
        "Mac802_11CfpStationPrintStats: "
        "One or more stat values is unexpectedly non-zero.\n");
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11GetDataRateEntry.
//  PURPOSE:     Returns the data rate entry for given destAddr.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//  RETURN:      M802_11DataRateEntry Pointer to Entry for given destAddr.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
M802_11DataRateEntry* Mac802_11GetDataRateEntry(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr)
{
    M802_11DataRateEntry* entry = M802->dataRateHead;
    M802_11DataRateEntry* prev = NULL;

    if (!entry) {
        entry = (M802_11DataRateEntry*)
                MEM_malloc(sizeof(M802_11DataRateEntry));
        entry->ipAddress = destAddr;
        if (destAddr != ANY_DEST) {
            entry->dataRateType = M802->lowestDataRateType;
        }
        else {
            entry->dataRateType = M802->highestDataRateTypeForBC;
        }
        entry->firstTxAtNewRate = FALSE;
        entry->numAcksFailed = 0;
        entry->numAcksInSuccess = 0;
        entry->dataRateTimer = getSimTime(node) + M802->rateAdjustmentTime;
        entry->next = NULL;
        M802->dataRateHead = entry;
        return entry;
    }

    while (entry) {
        if (entry->ipAddress == destAddr) {
            return entry;
        }
        else {
            prev = entry;
            entry = entry->next;
        }
    }

    entry = (M802_11DataRateEntry*)
            MEM_malloc(sizeof(M802_11DataRateEntry));
    entry->ipAddress = destAddr;
    if (destAddr != ANY_DEST) {
        entry->dataRateType = M802->lowestDataRateType;
    }
    else {
        entry->dataRateType = M802->highestDataRateTypeForBC;
    }
    entry->firstTxAtNewRate = FALSE;
    entry->numAcksFailed = 0;
    entry->numAcksInSuccess = 0;
    entry->dataRateTimer = getSimTime(node) + M802->rateAdjustmentTime;
    entry->next = NULL;

    prev->next = entry;

    return entry;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11AdjustDataRateForNewOutgoingPacket
//  PURPOSE:     When a packet is dequeued to the local buffer,
//               check for and adjust data rate.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11AdjustDataRateForNewOutgoingPacket(
    Node* node,
    MacData802_11* M802)
{
    if ((M802->dataRateInfo->ipAddress != ANY_ADDRESS &&
         M802->dataRateInfo->dataRateTimer < getSimTime(node)) ||
        (M802->dataRateInfo->numAcksInSuccess ==
         M802_11_NUM_ACKS_FOR_RATE_INCREASE))
    {
        //if (M802->dataRateInfo->dataRateType != M802->highestDataRateType) {
        //    printf("%lld: node %d increase rate to %d to node %d\n",
        //           getSimTime(node), node->nodeId,
        //           M802->dataRateInfo->dataRateType + 1,
        //           M802->dataRateInfo->ipAddress);
        //}

        M802->dataRateInfo->dataRateType =
            MIN(M802->dataRateInfo->dataRateType + 1,
                M802->highestDataRateType);

        M802->dataRateInfo->firstTxAtNewRate = TRUE;
        M802->dataRateInfo->numAcksFailed = 0;
        M802->dataRateInfo->numAcksInSuccess = 0;
        M802->dataRateInfo->dataRateTimer =
            getSimTime(node) + M802->rateAdjustmentTime;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11AdjustDataRateForResponseTimeout
//  PURPOSE:     When a RTS or CTS timeout occurs, check and
//               adjust values related to data rate.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11AdjustDataRateForResponseTimeout(
    Node* node,
    MacData802_11* M802)
{
    if ((M802->dataRateInfo->numAcksFailed ==
         M802_11_NUM_ACKS_FOR_RATE_DECREASE) ||
        (M802->dataRateInfo->numAcksFailed == 1 &&
         M802->dataRateInfo->firstTxAtNewRate))
    {
        //if (M802->dataRateInfo->dataRateType !=
        //    M802->lowestDataRateType)
        //{
        //    printf("%lld: node %d decrease rate to %d to node %d\n",
        //           getSimTime(node), node->nodeId,
        //           M802->dataRateInfo->dataRateType - 1,
        //           M802->dataRateInfo->ipAddress);
        //}

        M802->dataRateInfo->dataRateType =
            MAX(M802->dataRateInfo->dataRateType - 1,
                M802->lowestDataRateType);

        M802->dataRateInfo->numAcksFailed = 0;
        M802->dataRateInfo->numAcksInSuccess = 0;
        M802->dataRateInfo->firstTxAtNewRate = FALSE;
        M802->dataRateInfo->dataRateTimer =
            getSimTime(node) + M802->rateAdjustmentTime;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer
//  PURPOSE:     Move a packet from the network layer to the buffer
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer(
    Node* node,
    MacData802_11* M802)
{
    BOOL dequeued;
    TosType priority;
    int networkType;
    NodeAddress srcAddr;
    NodeAddress destAddr;

    int interfaceIndex = M802->myMacData->interfaceIndex;

    ERROR_Assert(M802->currentMessage == NULL,
        "Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer: "
        "A packet exists in the local buffer.\n");

    dequeued =
        MAC_OutputQueueDequeuePacket(
            node,
            interfaceIndex,
            &(M802->currentMessage),
            &(M802->currentNextHopAddress),
            &networkType, &priority);

    if (dequeued == FALSE) {
        return FALSE;
    }

// ANTS01 begin:
	M802->currentStartTimeANTS01 = getSimTime(node);
// ANTS01 end

    M802->pktsToSend++;

    // For a BSS station, change next hop address to be that of the AP
    M802->ipNextHopAddr = M802->currentNextHopAddress;
    if (Mac802_11IsBssStation(M802)) {
        M802->currentNextHopAddress = M802->bssAddr;
    }

    // Extract the source/dest address from the packet
    // Note: Only IP4 and MPLS assumed here.
    if (M802->myMacData->mplsVar == NULL)  {
        // Treat as an IP v4/v6 message
        // If this is IPv6 packet.
        if (MAPPING_GetNetworkTypeForInterface(node,
            interfaceIndex) == NETWORK_IPV6)
        {
            ip6_hdr* ip6Header =
                (ip6_hdr*)MESSAGE_ReturnPacket(M802->currentMessage);

            srcAddr = ANY_DEST;
            if (!IS_MULTIADDR6(ip6Header->ip6_src))
            {
                char addressstr[80];
                IO_ConvertIpAddressToString(&ip6Header->ip6_src, addressstr);

                NodeId nodeId = INVALID_MAPPING;

                if (IS_LINKLADDR6(ip6Header->ip6_src))
                {
                    nodeId = node->nodeId;
                }else
                {
                    nodeId = MAPPING_GetNodeIdFromGlobalAddr(
                                    node, ip6Header->ip6_src);
                }

                if (nodeId == INVALID_MAPPING)
                {
                    ERROR_Assert(FALSE, "IPv6 Source Address error "
                        "unable to found source address\n");
                }
                srcAddr = MAPPING_CreateIpv6LinkLayerAddr(nodeId,
                            interfaceIndex);
            }
            destAddr = ANY_DEST;
            if (!IS_MULTIADDR6(ip6Header->ip6_dst))
            {
                NodeId nodeId = MAPPING_GetNodeIdFromGlobalAddr(
                                node, ip6Header->ip6_dst);

                if (nodeId == INVALID_MAPPING)
                {
                    ERROR_Assert(FALSE,"IPv6 Destination Address error "
                        "unable to found source address\n");
                }
                destAddr = MAPPING_CreateIpv6LinkLayerAddr(nodeId,
                            interfaceIndex);
            }
        }
        else
        {   // This is IPv4 packet.
            IpHeaderType* ipHeader;
        ipHeader =
            (IpHeaderType*)MESSAGE_ReturnPacket(M802->currentMessage);
            srcAddr = ipHeader->ip_src;
            destAddr = ipHeader->ip_dst;
        }
    }
#ifdef ENTERPRISE_LIB
    else {
        // Treat this packet as an MPLS message
        IpHeaderType* ipHeader;
        char *aHeader;
        aHeader = (MESSAGE_ReturnPacket(M802->currentMessage)
            + sizeof(Mpls_Shim_LabelStackEntry));
        ipHeader = (IpHeaderType*)aHeader;
        srcAddr = ipHeader->ip_src;
        destAddr = ipHeader->ip_dst;
    }
#endif // ENTERPRISE_LIB


    M802->ipSourceAddr = srcAddr;
    M802->ipDestAddr = destAddr;

    // Ensure that network broadcast addresses are treated correctly
    if (M802->currentNextHopAddress ==
        NetworkIpGetInterfaceBroadcastAddress(node, interfaceIndex))
    {
        M802->currentNextHopAddress = ANY_DEST;
    }

    M802->dataRateInfo =
        Mac802_11GetDataRateEntry(node, M802, M802->currentNextHopAddress);

    ERROR_Assert(M802->dataRateInfo->ipAddress ==
                 M802->currentNextHopAddress,
                 "Address does not match");

    Mac802_11AdjustDataRateForNewOutgoingPacket(node, M802);

    return TRUE;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11GetLastSignalsAngleOfArrivalFromRadio
//  PURPOSE:     Get the last signal's AOA from the radio
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      AOA in double
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
double Mac802_11GetLastSignalsAngleOfArrivalFromRadio(
    Node *node,
    MacData802_11 *M802)
{
    int phyIndex = M802->myMacData->phyNumber;
    return PHY_GetLastSignalsAngleOfArrival(node, phyIndex);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11SetPhySensingDirectionForNextOutgoingPacket
//  PURPOSE:     Set the sensing direction for the next outgoing packet
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static //inline//
void Mac802_11SetPhySensingDirectionForNextOutgoingPacket(
    Node *node,
    MacData802_11 *M802)
{
    BOOL haveDirection = FALSE;
    double directionAngleToSend = 0.0;

    if (M802->currentMessage == NULL) {
        Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer(node, M802);
    }//if//


    if ((M802->currentMessage != NULL) &&
        (M802->currentNextHopAddress != ANY_DEST))
    {
        AngleOfArrivalCache_GetAngle(
            &M802->directionalInfo->angleOfArrivalCache,
            M802->currentNextHopAddress,
            getSimTime(node),
            &haveDirection,
            &directionAngleToSend);
    }//if//

    if (haveDirection) {
        PHY_SetSensingDirection(
            node, M802->myMacData->phyNumber, directionAngleToSend);
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11UpdateDirectionCacheWithCurrentSignal
//  PURPOSE:     Update the direction of the neighbor in the cache
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destinationAddress
//                  Destination address
//               BOOL* nodeIsInCache
//                  Returns whether the node is in cache
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11UpdateDirectionCacheWithCurrentSignal(
    Node *node,
    MacData802_11 *M802,
    NodeAddress destinationAddress,
    BOOL* nodeIsInCache)
{
    AngleOfArrivalCache_Update(
        &M802->directionalInfo->angleOfArrivalCache,
        destinationAddress,
        Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802),
        nodeIsInCache,
        (getSimTime(node) + M802->directionalInfo->defaultCacheExpirationDelay),
        getSimTime(node));
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11AddDirectionCacheWithCurrentSignal
//  PURPOSE:     Add an entry for the neighbor to the cache
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destinationAddress
//                  Destination address
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11AddDirectionCacheWithCurrentSignal(
    Node *node,
    MacData802_11* M802,
    NodeAddress destinationAddress)
{
    AngleOfArrivalCache_Add(
        &M802->directionalInfo->angleOfArrivalCache,
        destinationAddress,
        Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802),
        (getSimTime(node) + M802->directionalInfo->defaultCacheExpirationDelay));
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11WaitForNAV.
//  PURPOSE:     See if this node is holding for NAV.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE if holding for NAV, FALSE otherwise.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static //inline//
BOOL Mac802_11WaitForNAV(
    Node* node,
    MacData802_11* M802)
{
    return (M802->NAV > getSimTime(node));
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11SetState.
//  PURPOSE:     Set the state of the node.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_MacStates state
//                  The state that is being set to.
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11SetState(
    Node* node,
    MacData802_11* M802,
    M802_11_MacStates state)
{
    M802->prevState = M802->state;
    M802->state = state;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11StartTimer.
//  PURPOSE:     Set various timers in 802.11.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               clocktype timerDelay
//                  Duration of the timer being set.
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11StartTimer(
    Node* node,
    MacData802_11* M802,
    clocktype timerDelay)
{
    Message *newMsg;

    M802->timerSequenceNumber++;

    newMsg = MESSAGE_Alloc(node, MAC_LAYER, MAC_PROTOCOL_802_11,
                           MSG_MAC_TimerExpired);
    MESSAGE_SetInstanceId(newMsg, (short) M802->myMacData->interfaceIndex);

    MESSAGE_InfoAlloc(node, newMsg, sizeof(M802->timerSequenceNumber));
    *((int*)MESSAGE_ReturnInfo(newMsg)) = M802->timerSequenceNumber;

    MESSAGE_Send(node, newMsg, timerDelay);

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CancelTimer.
//  PURPOSE:     Cancel the current timer.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static //inline//
void Mac802_11CancelTimer(
    Node* node,
    MacData802_11* M802)
{
    // Has the effect of making the current timer "obsolete".
    M802->timerSequenceNumber++;

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11StartTransmittingPacket
//  PURPOSE:     Start transmitting packet.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* packet
//                  Packet to be transmitted
//               clocktype delay
//                  Delay before transmission
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11StartTransmittingPacket(
    Node* node,
    MacData802_11* M802,
    Message* packet,
    clocktype delay)
{
    if (DEBUG) {
        char timeStr[100], delayStr[100];
        M802_11LongControlFrame* hdr =
            (M802_11LongControlFrame*)MESSAGE_ReturnPacket(packet);

        ctoa(getSimTime(node), timeStr);
        ctoa(delay, delayStr);

        printf("%d Sending to %d (size %d) at %s %s\n", node->nodeId,
               hdr->destAddr, MESSAGE_ReturnPacketSize(packet),
               timeStr, delayStr);
    }

    Mac802_11Trace(node, M802, packet, "Send");

    PHY_StartTransmittingSignal(
        node, M802->myMacData->phyNumber, packet, TRUE, delay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11StartTransmittingPacketDirectionally
//  PURPOSE:     Start transmitting packet directionally
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* packet
//                  Packet to be transmitted
//               clocktype delay
//                  Delay before transmission
//               double directionAngle
//                  Direction of transmission
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11StartTransmittingPacketDirectionally(
    Node* node,
    MacData802_11* M802,
    Message* packet,
    clocktype delay,
    double directionAngle)
{
    if (DEBUG) {
        char timeStr[100], delayStr[100];
        M802_11LongControlFrame* hdr =
            (M802_11LongControlFrame *)MESSAGE_ReturnPacket(packet);

        ctoa(getSimTime(node), timeStr);
        ctoa(delay, delayStr);

        printf("%d Sending to %d (size %d) at %s %s\n", node->nodeId,
               hdr->destAddr, MESSAGE_ReturnPacketSize(packet),
               timeStr, delayStr);
    }

    Mac802_11Trace(node, M802, packet, "Send");

    PHY_StartTransmittingSignalDirectionally(
        node, M802->myMacData->phyNumber,  packet, TRUE, delay,
        directionAngle);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11PhyStatus
//  PURPOSE:     Get the physical status.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      The physical status of the medium
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
PhyStatusType Mac802_11PhyStatus(
    Node* node,
    MacData802_11* M802)
{
    return PHY_GetStatus(node, M802->myMacData->phyNumber);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11AttemptToGoIntoWaitForDifsOrEifsState
//  PURPOSE:     Attempt to go into DIFS or EIFS state.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(
    Node* node,
    MacData802_11* M802)
{
    BOOL mustWaitForNAV = FALSE;
    clocktype navTimeoutTime  = 0;

    if (!M802->useDvcs) {
        if (Mac802_11PhyStatus(node, M802) != PHY_IDLE) {
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);

            return;
        }//if//

        // Should disregard NAV in Extended Ifs Mode. section 9.2.3.4
        if (!M802->IsInExtendedIfsMode) {
            if (Mac802_11WaitForNAV(node, M802)) {

                // Set timer to wait for NAV to finish.
                mustWaitForNAV = TRUE;
                navTimeoutTime = M802->NAV;
            }//if//
        }
        else {
            M802->NAV = 0;
        }//if//
    }
    else {
        BOOL haveDirection = FALSE;
        double directionAngleToSend = 0.0;

        if (Mac802_11PhyStatus(node, M802) == PHY_RECEIVING) {
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);

            return;
        }//if//

        if (M802->currentNextHopAddress != ANY_DEST) {
            AngleOfArrivalCache_GetAngle(
                &M802->directionalInfo->angleOfArrivalCache,
                M802->currentNextHopAddress,
                getSimTime(node),
                &haveDirection,
                &directionAngleToSend);
        }//if//

        if (haveDirection) {
            PHY_SetSensingDirection(
                node, M802->myMacData->phyNumber, directionAngleToSend);
        }//if//

        if (Mac802_11PhyStatus(node, M802) == PHY_SENSING) {
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);
            return;
        }//if//

        ERROR_Assert(Mac802_11PhyStatus(node, M802) == PHY_IDLE,
                     "Current state should be IDLE");

        if (((haveDirection) &&
             (DirectionalNav_SignalIsNaved(
                 &M802->directionalInfo->directionalNav,
                 directionAngleToSend, getSimTime(node))))
                  ||
            ((!haveDirection) &&
             (!DirectionalNav_ThereAreNoNavs(
                 &M802->directionalInfo->directionalNav, getSimTime(node)))))
        {
            // Should disregard NAV in Extended Ifs Mode. section 9.2.3.4
            if (!M802->IsInExtendedIfsMode) {
                mustWaitForNAV = TRUE;
                DirectionalNav_GetEarliestNavExpirationTime(
                    &M802->directionalInfo->directionalNav,
                    getSimTime(node),
                    &navTimeoutTime);
            }
            else {
                M802->NAV = 0;
            }//if//
        }//if//
    }//if//


    if (mustWaitForNAV) {
        // Set timer to wait for NAV to finish.
        Mac802_11SetState(node, M802, M802_11_S_WFNAV);
        Mac802_11StartTimer(node, M802, (navTimeoutTime - getSimTime(node)));
    }
    else {
        clocktype extraWait = 0;

        Mac802_11SetState(node, M802, M802_11_S_WF_DIFS_OR_EIFS);

        if (getSimTime(node) < M802->noOutgoingPacketsUntilTime) {
            extraWait = M802->noOutgoingPacketsUntilTime - getSimTime(node);
        }//if//

        if (M802->IsInExtendedIfsMode) {
            Mac802_11StartTimer(
                node, M802, (extraWait + M802->extendedIfsDelay));
        }
        else {
            Mac802_11StartTimer(
                node, M802, (extraWait + M802->txDifs));
        }//if//
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11IncreaseCW.
//  PURPOSE:     Increase control window since node has to retransmit.
//  PARAMETERS:  Node* node
//                  Pointer to node thatis increasing its contention window
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11IncreaseCW(
    Node* node,
    MacData802_11* M802)
{
    M802->CW = MIN((((M802->CW + 1) * 2) - 1), M802->cwMax);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11GetSeqNo.
//  PURPOSE:     Return the entry for given destAddr.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//  RETURN:      M802_11SeqNoEntry Pointer to Entry for given destAddr.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
M802_11SeqNoEntry* Mac802_11GetSeqNo(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr)
{
    M802_11SeqNoEntry* entry = M802->seqNoHead;
    M802_11SeqNoEntry* prev = NULL;

    if (!entry) {
        entry = (M802_11SeqNoEntry*) MEM_malloc(sizeof(M802_11SeqNoEntry));
        entry->ipAddress = destAddr;
        entry->fromSeqNo = 0;
        entry->toSeqNo = 0;
        entry->next = NULL;
        M802->seqNoHead = entry;
        return entry;
    }

    while (entry) {
        if (entry->ipAddress == destAddr) {
            return entry;
        }
        else {
            prev = entry;
            entry = entry->next;
        }
    }

    entry = (M802_11SeqNoEntry*) MEM_malloc(sizeof(M802_11SeqNoEntry));
    entry->ipAddress = destAddr;
    entry->fromSeqNo = 0;
    entry->toSeqNo = 0;
    entry->next = NULL;

    prev->next = entry;

    return entry;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11SetBackoffIfZero.
//  PURPOSE:     Set the backoff counter for this node.
//  PARAMETERS:  Node* node
//                  Pointer to node that is being backed off.
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11SetBackoffIfZero(
    Node* node,
    MacData802_11* M802)
{
    if (M802->BO == 0) {
        M802->BO = (RANDOM_nrand(M802->seed) % (M802->CW + 1)) * M802->slotTime;
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11HandOffSuccessfullyReceivedUnicast
//  PURPOSE:     Remove unicast header, forward the message to upper layer
//               and update sequence number in list.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11HandOffSuccessfullyReceivedUnicast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11SeqNoEntry *entry;
    M802_11LongControlFrame* lHdr =
        (M802_11LongControlFrame*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = lHdr->sourceAddr;

    MESSAGE_RemoveHeader(node,
                         msg,
                         M802_11_DATA_FRAME_HDR_SIZE,
                         TRACE_802_11);

    MAC_HandOffSuccessfullyReceivedPacket(node,
        M802->myMacData->interfaceIndex, msg, sourceAddr);

    // Upate sequence number
    entry = Mac802_11GetSeqNo(node, M802, sourceAddr);
    ERROR_Assert(entry,
        "Mac802_11HandOffSuccessfullyReceivedUnicast: "
        "Unable to create or find sequence number entry.\n");
    entry->fromSeqNo += 1;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11HandOffSuccessfullyReceivedBroadcast
//  PURPOSE:     Strip 802.11 header and forward message to upper layer.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11HandOffSuccessfullyReceivedBroadcast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11LongControlFrame* lHdr =
        (M802_11LongControlFrame*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = lHdr->sourceAddr;

    MESSAGE_RemoveHeader(node,
                         msg,
                         M802_11_DATA_FRAME_HDR_SIZE,
                         TRACE_802_11);

    MAC_HandOffSuccessfullyReceivedPacket(node,
        M802->myMacData->interfaceIndex, msg, sourceAddr);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11UpdateForSuccessfullyReceivedAck
//  PURPOSE:     Clear message in buffer and related variables
//               once unicast is acknowledged.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Address of the source
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11UpdateForSuccessfullyReceivedAck(
    Node* node,
    MacData802_11* M802,
    NodeAddress sourceAddr)
{
    M802_11SeqNoEntry *entry;

    // Reset retry counts since frame is acknowledged.
    Mac802_11ResetAckVariables(M802);

    MAC_MacLayerAcknowledgement(node, M802->myMacData->interfaceIndex,
        M802->currentMessage, sourceAddr);

    // Remove frame from queue since already acknowledged.
    MESSAGE_Free(node, M802->currentMessage);
    Mac802_11ResetCurrentMessageVariables(M802);

    // Update exchange sequence number.
    entry = Mac802_11GetSeqNo(node, M802, sourceAddr);
    ERROR_Assert(entry,
        "Mac802_11UpdateForSuccessfullyReceivedAck: "
        "Unable to find sequence number entry.\n");
    entry->toSeqNo += 1;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CheckForOutgoingPacket.
//  PURPOSE:     If there's a packet in the queue that needs to be sent out,
//               this should only be called right after the medium was
//               busy. So assume 802.11 will backoff.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               BOOL backoff
//                  Force backoff even with no message to transmit next
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11CheckForOutgoingPacket(
    Node* node,
    MacData802_11* M802,
    BOOL backoff)
{
    if (M802->currentMessage == NULL) {
        Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer(node, M802);
    }

    if (M802->cfpBeaconIsDue && Mac802_11CanHandleDueCfpBeacon(node, M802)) {
        return;
    }

    // If the Queue is NonEmpty, or we are currently working on transmitting
    // a message (currentMessage)
    if (backoff ||
        M802->BO != 0 ||
        M802->currentMessage != NULL)
    {
        // Have something to send. Set Backoff if not already set and
        // start sending process
        Mac802_11SetBackoffIfZero(node, M802);
        Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(node, M802);
    }
    else {
        ERROR_Assert(M802->BO == 0,
            "Mac802_11CheckForOutgoingPacket: "
            "Backoff should be zero to enter IDLE state.\n");
        Mac802_11SetState(node, M802, M802_11_S_IDLE);
        Mac802_11CancelTimer(node, M802);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11InformNetworkOfPktDrop.
//  PURPOSE:     Inform Network layer that a packet is dropped because of
//               unsuccessful delivery.
//  PARAMETERS:  Node* node
//                  Pointer to node that was unable to deliver the network
//                   packet.
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                   Network packet that got dropped
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11InformNetworkOfPktDrop(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    MAC_NotificationOfPacketDrop(node,
                                 M802->ipNextHopAddr,
                                 M802->myMacData->interfaceIndex,
                                 msg);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ResetCW.
//  PURPOSE:     Reset congestion window since transmission was successful.
//  PARAMETERS:  Node* node
//                  Pointer to node that is resetting its contention window
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11ResetCW(
    Node* node,
    MacData802_11* M802)
{
    M802->CW = M802->cwMin;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11Retransmit.
//  PURPOSE:     Retransmit frame when CTS or ACK timeout occurs.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11Retransmit(
    Node* node,
    MacData802_11* M802)
{
    BOOL UseShortPacketRetryCounter;
    BOOL RetryTheTransmission = FALSE;

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11Retransmit: "
        "There is no message in the local buffer.\n");

    // Long retry count only applies to data frames greater
    // than RTS_THRESH.  RTS uses short retry count.
    UseShortPacketRetryCounter =
        ((M802->state == M802_11_S_WFCTS) ||
         (MESSAGE_ReturnPacketSize(M802->currentMessage) <=
          M802->rtsThreshold));

    // If not greater than maximum retry count allowed, retransmit frame
    if (UseShortPacketRetryCounter) {
        M802->SSRC++;
        if (M802->SSRC < M802->shortRetryLimit) {
            RetryTheTransmission = TRUE;

            if ((M802->useDvcs) &&
                (M802->SSRC >= M802->directionalShortRetryLimit))
            {
                AngleOfArrivalCache_Delete(
                    &M802->directionalInfo->angleOfArrivalCache,
                    M802->currentNextHopAddress);
            }//if//
        }
        else {
            RetryTheTransmission = FALSE;
            M802->SSRC = 0;
        }//if//
    }
    else {
        M802->SLRC++;
        if (M802->SLRC < M802->longRetryLimit) {
            RetryTheTransmission = TRUE;
        }
        else {
            RetryTheTransmission = FALSE;
            M802->SLRC = 0;
        }//if//
    }//if/


    if (RetryTheTransmission) {
        Mac802_11IncreaseCW(node, M802);
        Mac802_11SetBackoffIfZero(node, M802);
        Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(node, M802);
    }
    else {
        // Exceeded maximum retry count allowed, so drop frame
        M802->pktsDroppedDcf++;

        // Drop frame from queue

        Mac802_11Trace(node, M802, NULL, "Drop, exceeds retransmit count");

        Mac802_11InformNetworkOfPktDrop(node, M802, M802->currentMessage);

        if (M802->useDvcs) {
            AngleOfArrivalCache_Delete(
                &M802->directionalInfo->angleOfArrivalCache,
                M802->currentNextHopAddress);
        }

        Mac802_11ResetCurrentMessageVariables(M802);
        Mac802_11SetState(node, M802, M802_11_S_IDLE);
        Mac802_11ResetCW(node, M802);
        Mac802_11CheckForOutgoingPacket(node, M802, TRUE);
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CTSTransmitted.
//  PURPOSE:     Hold for reply after sending CTS.
//  PARAMETERS:  Node* node
//                  Pointer to node that sent the CTS
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CTSTransmitted(
    Node* node,
    MacData802_11* M802)
{

    Mac802_11SetState(node, M802, M802_11_S_WFDATA);
    Mac802_11StartTimer(node, M802, M802->noResponseTimeoutDuration);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11UnicastedTransmitted.
//  PURPOSE:     Hold for ACK after data frame is transmitted.
//  PARAMETERS:  Node* node
//                  Pointer to node that sent the data
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11UnicastTransmitted(Node* node, MacData802_11* M802) {
    clocktype holdForAck;

    Mac802_11SetState(node, M802, M802_11_S_WFACK);

    holdForAck =
        M802->extraPropDelay +
        M802->sifs +
        PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            PHY_GetTxDataRateType(node, M802->myMacData->phyNumber),
            M802_11_SHORT_CTRL_FRAME_SIZE) +
        M802->extraPropDelay +
        M802->slotTime;

    Mac802_11StartTimer(node, M802, holdForAck);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11AckTransmitted.
//  PURPOSE:     Ack sent. Check for packets in IP queue.
//  PARAMETERS:  Node* node
//                  Pointer to node that sent the ACK
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11AckTransmitted(
    Node* node,
    MacData802_11* M802)
{
    if (M802->useDvcs) {

        // For smart antennas, go back to sending/receiving
        // omnidirectionally.
        PHY_UnlockAntennaDirection(
            node, M802->myMacData->phyNumber);
    }//if//

    Mac802_11SetState(node, M802, M802_11_S_IDLE);
    Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11RTSTransmitted.
//  PURPOSE:     Hold for reply after sending RTS.
//  PARAMETERS:  Node* node
//                  Pointer to node that sent the RTS
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11RTSTransmitted(Node* node, MacData802_11* M802)
{
    clocktype holdForCts;

    Mac802_11SetState(node, M802, M802_11_S_WFCTS);

    holdForCts =
        M802->extraPropDelay +
        M802->sifs +
        PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            PHY_GetTxDataRateType(node, M802->myMacData->phyNumber),
            M802_11_SHORT_CTRL_FRAME_SIZE) +
        M802->extraPropDelay +
        M802->slotTime;

    Mac802_11StartTimer(node, M802, holdForCts);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11PauseBackoff.
//  PURPOSE:     Stop the backoff counter once carrier is sensed.
//  PARAMETERS:  Node* node
//                  Pointer to node that is stopping its own backoff.
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11PauseBackoff(Node* node, MacData802_11* M802)
{
    M802->BO = M802->BO - (getSimTime(node) - M802->lastBOTimeStamp);

    //printf("Node- %d BO = %I64d (%I64d)\n", node->id, M802->BO, M802->CW);

    if (M802->BO < 0) {
        ERROR_ReportError("Mac802_11PauseBackoff: "
            "Backoff is less than 0.\n");
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ProcessNotMyFrame.
//  PURPOSE:     Handle frames that don't belong to this node.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               clocktype duration
//                  time
//               BOOL isARtsPacket
//                  Is a RTS packet
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11ProcessNotMyFrame(
   Node* node, MacData802_11* M802,
   clocktype duration, BOOL isARtsPacket)
{
    clocktype currentTime = getSimTime(node);
    BOOL navHasChanged = FALSE;
    clocktype NewNAV;

    if (duration == M802_11_CF_FRAME_DURATION * MICRO_SECOND) {
        return;
    }

    if (!M802->useDvcs) {
        NewNAV = currentTime + duration + EPSILON_DELAY;
        if (NewNAV > M802->NAV) {
            M802->NAV = NewNAV;
            navHasChanged = TRUE;
        }//if//
    }
    else {
        double angleOfArrival =
            Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802);
        double delta = M802->directionalInfo->defaultNavedDeltaAngle;

        DirectionalNav_Add(
            &M802->directionalInfo->directionalNav,
            NormalizedAngle(angleOfArrival - delta),
            NormalizedAngle(angleOfArrival + delta),
            (currentTime + duration + EPSILON_DELAY),
            currentTime);

        DirectionalNav_GetEarliestNavExpirationTime(
            &M802->directionalInfo->directionalNav,
            currentTime,
            &NewNAV);

        if (M802->NAV != NewNAV) {
            M802->NAV = NewNAV;
            navHasChanged = TRUE;
        }//if//
        Mac802_11SetPhySensingDirectionForNextOutgoingPacket(node, M802);
    }//if//

    ERROR_Assert(
        M802->state != M802_11_S_WF_DIFS_OR_EIFS,
        "Mac802_11ProcessNotMyFrame: "
        "Invalid state -- waiting for DIFS or EIFS.\n");
    ERROR_Assert(
        M802->state != M802_11_S_BO,
        "Mac802_11ProcessNotMyFrame: "
        "Invalid state -- waiting for Backoff.\n");
    ERROR_Assert(
        M802->state != M802_11_S_NAV_RTS_CHECK_MODE,
        "Mac802_11ProcessNotMyFrame: "
        "Invalid state -- waiting for NAV RTS check mode.\n");
    ERROR_Assert(
        M802->state != M802_11_S_WFNAV,
        "Mac802_11ProcessNotMyFrame: "
        "Invalid state -- waiting for NAV.\n");

    if ((navHasChanged) &&
        (Mac802_11PhyStatus(node, M802) == PHY_IDLE) &&
        (M802->currentMessage != NULL))
    {
        if (M802->state == M802_11_S_WFCTS) {
            Mac802_11Trace(node, M802, NULL, "Wait for CTS aborted.");
            M802->retxAbortedRtsDcf++;
        }

        // This is what we should do.
        //
        //if (isARtsPacket) {
            // If RTS-ing node failed to get a CTS
            // and start sending then reset the NAV
            // (MAC layer virtual carrier sense) for this
            // bystander node.
        //   Mac802_11SetState(node,
        //      M802, M802_11_S_NAV_RTS_CHECK_MODE);
        //   Mac802_11StartTimer(node, M802,
        //      (M802->extraPropDelay + M802->sifs +
        //       PHY_GetTransmissionDuration(
        //           node, M802->myMacData->phyNumber,
        //           PHY_GetRxDataRateType(node, M802->myMacData->phyNumber),
        //           M802_11_SHORT_CTRL_FRAME_SIZE) +
        //       M802->extraPropDelay + 2 * M802->slotTime));
        //} else {
        //
        //   Mac802_11SetState(node, M802, M802_11_S_WFNAV);
        //   Mac802_11StartTimer(node, M802,
        //      (M802->NAV - currentTime));
        //  }//if//

        // This is for ns-2 comparison.
        Mac802_11SetState(node, M802, M802_11_S_WFNAV);
        Mac802_11StartTimer(node, M802, (M802->NAV - currentTime));
    }
}//void Mac802_11ProcessNotMyFrame//



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CorrectSequenceNumber.
//  PURPOSE:     Check if ACK contains the right sequence number.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAdd
//                  Node address of station
//               int seqNo
//                  Sequence number of the frame that is being acked
//  RETURN:      TRUE if sequence number being ACKed is correct.
//               FALSE otherwise.
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CorrectSeqenceNumber(
    Node* node,
    MacData802_11* M802,
    NodeAddress sourceAddr,
    int seqNo)
{
    M802_11SeqNoEntry* entry;

    entry = Mac802_11GetSeqNo(node, M802, sourceAddr);
    ERROR_Assert(entry,
        "Mac802_11CorrectSeqenceNumber: "
        "Sequence number entry not found.\n");

    return (entry->fromSeqNo == seqNo);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmitCTSFrame.
//  PURPOSE:     Transmit a CTS frame.
//  PARAMETERS:  Node* node
//                  Pointer to node that is transmitting CTS frame
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* Rts
//                  Pointer holding the RTS
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11TransmitCTSFrame(Node* node, MacData802_11* M802,
                               Message* Rts)
{
    NodeAddress destAddr;
    Message* pktToPhy;
    M802_11LongControlFrame* hdr =
        (M802_11LongControlFrame*)MESSAGE_ReturnPacket(Rts);
    M802_11ShortControlFrame* newHdr;
    int dataRateType = PHY_GetRxDataRateType(node, M802->myMacData->phyNumber);

    destAddr = hdr->sourceAddr;

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node,
                        pktToPhy,
                        M802_11_SHORT_CTRL_FRAME_SIZE,
                        TRACE_802_11);

    newHdr = (M802_11ShortControlFrame *) MESSAGE_ReturnPacket(pktToPhy);
    memset(newHdr, 0, sizeof(M802_11ShortControlFrame));

    newHdr->frameType = M802_11_CTS;
    newHdr->destAddr = destAddr;

    // Subtract RTS transmit time.
    newHdr->duration =
        Mac802_11NanoToMicrosecond(
            Mac802_11MicroToNanosecond(hdr->duration)
            - M802->extraPropDelay
            - M802->sifs
            - PHY_GetTransmissionDuration(
                  node, M802->myMacData->phyNumber,
                  dataRateType,
                  M802_11_SHORT_CTRL_FRAME_SIZE));

    // Subtract off CTS transmit time from the total duration
    // to get the CTS timeout time.
    M802->noResponseTimeoutDuration =
        Mac802_11MicroToNanosecond(newHdr->duration)
        - M802->extraPropDelay - M802->sifs
        - PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            dataRateType,
            M802_11_SHORT_CTRL_FRAME_SIZE)
        - M802->extraPropDelay
        + M802->slotTime + EPSILON_DELAY;

    Mac802_11SetState(node, M802, M802_11_X_CTS);

    M802->ctsPacketsSentDcf++;

    PHY_SetTxDataRateType(
        node,
        M802->myMacData->phyNumber,
        MIN(dataRateType, M802->highestDataRateTypeForBC));

    if (!M802->useDvcs) {
        Mac802_11StartTransmittingPacket(node, M802, pktToPhy, M802->sifs);
    }
    else {
        Mac802_11StartTransmittingPacketDirectionally(
            node, M802, pktToPhy, M802->sifs,
            Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802));

        PHY_LockAntennaDirection(node, M802->myMacData->phyNumber);
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11HandlePromiscuousMode.
//  PURPOSE:     Send remote packet to network layer.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message *frame
//                  Frame
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11HandlePromiscuousMode(
    Node* node,
    MacData802_11* M802,
    Message *frame,
    NodeAddress nextHop,
    NodeAddress destAddr)
{
    MESSAGE_RemoveHeader(
        node,
        frame,
        M802_11_DATA_FRAME_HDR_SIZE,
        TRACE_802_11);

    MAC_SneakPeekAtMacPacket(
        node,
        M802->myMacData->interfaceIndex,
        frame,
        nextHop,
        destAddr);

    MESSAGE_AddHeader(
        node,
        frame,
        M802_11_DATA_FRAME_HDR_SIZE,
        TRACE_802_11);
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11ProcessAck.
//  PURPOSE:     Process local ACKs.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message *Ack
//                  Acknowledgement message
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------

static
void Mac802_11ProcessAck(
    Node* node,
    MacData802_11* M802,
    Message *Ack)
{
    NodeAddress sourceAddr;

    sourceAddr = M802->waitingForAckOrCtsFromAddress;

    // Reset retry counts since frame is acknowledged
    Mac802_11ResetAckVariables(M802);

    if (M802->currentMessage == NULL) {
        ERROR_ReportError("Mac802_11ProcessAck: "
            "There is no message in local buffer.\n");
    }

    if (M802->useDvcs) {
        //
        // For smart antennas, go back to sending/receiving
        // omnidirectionally.
        //
        PHY_UnlockAntennaDirection(
            node, M802->myMacData->interfaceIndex);
    }//if//

    switch (M802->state) {
        case M802_11_S_WFACK: {
            Mac802_11UpdateForSuccessfullyReceivedAck(
                node,
                M802,
                sourceAddr);

            M802->unicastPacketsSentDcf++;

// ANTS01 begin:
            Mac802_11UpdateAvgSendTimeANTS01(node,M802);
            Mac802_11NotifySendReceiveANTS01(node,sourceAddr);
// ANTS01 end

            Mac802_11ResetCW(node, M802);
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CheckForOutgoingPacket(node, M802, TRUE);

            break;
        }

        default: {
            Mac802_11SetState(node, M802, M802_11_S_IDLE);

            Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
            break;
        }
    }//switch//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmitAck
//  PURPOSE:     Transmit ACK for frames.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress detsAddr
//                  Node address of destination station
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11TransmitAck(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr)
{
    Message* pktToPhy;
    M802_11ShortControlFrame* newHdr;
    int dataRateType =
        PHY_GetRxDataRateType(node, M802->myMacData->phyNumber);

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node,
                        pktToPhy,
                        M802_11_SHORT_CTRL_FRAME_SIZE,
                        TRACE_802_11);

    newHdr = (M802_11ShortControlFrame *) MESSAGE_ReturnPacket(pktToPhy);

    newHdr->frameType = M802_11_ACK;
    newHdr->destAddr = destAddr;

    newHdr->duration = 0;

    Mac802_11SetState(node, M802, M802_11_X_ACK);

    M802->ackPacketsSentDcf++;

    PHY_SetTxDataRateType(node, M802->myMacData->phyNumber, dataRateType);

    if (!M802->useDvcs) {
        Mac802_11StartTransmittingPacket(node, M802, pktToPhy, M802->sifs);
    }
    else {
        Mac802_11StartTransmittingPacketDirectionally(
            node, M802, pktToPhy, M802->sifs,
            Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802));
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ProcessFrame.
//  PURPOSE:     Process incoming frame.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message *frame
//                  Frame being processed
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11ProcessFrame(
    Node* node, MacData802_11* M802, Message *frame)
{
    M802_11FrameHdr* hdr = (M802_11FrameHdr*)MESSAGE_ReturnPacket(frame);
    NodeAddress sourceAddr = hdr->sourceAddr;

    if ((M802->state != M802_11_S_WFDATA) &&
        (Mac802_11IsWaitingForResponseState(M802->state)))
    {

        // Waiting for Another Packet Type Ignore This one.
        Mac802_11Trace(node, M802, NULL,
            "Drop, waiting for non-data response");

        MESSAGE_Free(node, frame);
    }
    else {
// ANTS01 begin:
        Mac802_11NotifySendReceiveANTS01(node,sourceAddr);
        if (NetworkIpGetRoutingProtocol(node, ROUTING_PROTOCOL_ANTHOCNET) != NULL)
        {
          ANTHOCNET_Mac802_11_UpdateSignalData(node,sourceAddr,M802->currentPower,
                       M802->currentSensitivity, M802->currentInterference,
                       M802->currentNoise,M802->currentSinr);
        }
// ANTS01 end
        if (hdr->destAddr == ANY_DEST) {
            Mac802_11Trace(node, M802, frame, "Receive");

            Mac802_11HandOffSuccessfullyReceivedBroadcast(
                node, M802, frame);

            M802->broadcastPacketsGotDcf++;
            Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
        }
        else {
            Mac802_11CancelTimer(node, M802);
            Mac802_11TransmitAck(node, M802, sourceAddr);

            if (Mac802_11CorrectSeqenceNumber(node, M802, sourceAddr,
                                              hdr->seqNo))
            {
                Mac802_11Trace(node, M802, frame, "Receive");

                // Update the poll list for received data frame
                Mac802_11CfPollListUpdateForUnicastReceived(
                    node, M802, sourceAddr);

                Mac802_11HandOffSuccessfullyReceivedUnicast(
                    node, M802, frame);

                M802->unicastPacketsGotDcf++;
            }
            else {
                // Wrong sequence number, Drop.

                Mac802_11Trace(node, M802, NULL,
                    "Drop, wrong sequence number");

                MESSAGE_Free(node, frame);
            }//if//
        }//if//
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ContinueBackoff.
//  PURPOSE:     Continues decreasing backoff counter.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11ContinueBackoff(Node* node, MacData802_11* M802)
{
    M802->lastBOTimeStamp = getSimTime(node);

    if (M802->BO < 0)
    {
        char errString[MAX_STRING_LENGTH];
        char clockStr[MAX_STRING_LENGTH];

        ctoa(M802->BO, clockStr);
        sprintf(errString,
            "Mac802_11ContinueBackoff: "
            "Node %u continuing backoff to a negative timer: %s\n",
            node->nodeId, clockStr);
        ERROR_ReportError(errString);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmitRTSFrame.
//  PURPOSE:     Send RTS to reserve channel for data frame.
//  PARAMETERS:  Node* node that wants to reserve channel for data frame.
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11TransmitRTSFrame(
    Node* node,
    MacData802_11* M802)
{
    M802_11LongControlFrame* hdr;
    Message* pktToPhy;
    NodeAddress destAddr;
    int dataRateType;

    destAddr = M802->currentNextHopAddress;
    ERROR_Assert(M802->currentNextHopAddress ==
                 M802->dataRateInfo->ipAddress,
                 "Address does not match");

    PHY_SetTxDataRateType(
        node,
        M802->myMacData->phyNumber,
        MIN(M802->dataRateInfo->dataRateType,
            M802->highestDataRateTypeForBC));

    dataRateType = PHY_GetTxDataRateType(node, M802->myMacData->phyNumber);

    if (M802->currentMessage == NULL) {
        ERROR_ReportError("Mac802_11TransmitRTSFrame: "
            "There is no message in the local buffer.\n");
    }

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node,
                        pktToPhy,
                        M802_11_LONG_CTRL_FRAME_SIZE,
                        TRACE_802_11);

    hdr = (M802_11LongControlFrame*)MESSAGE_ReturnPacket(pktToPhy);

    hdr->frameType = M802_11_RTS;

    hdr->sourceAddr = M802->selfAddr;
    hdr->destAddr = destAddr;

    M802->waitingForAckOrCtsFromAddress = destAddr;

    if (MESSAGE_ReturnPacketSize(M802->currentMessage)
        <= M802_11_FRAG_THRESH)
    {
        clocktype packetTransmissionDuration =
            PHY_GetTransmissionDuration(
                node, M802->myMacData->phyNumber,
                M802->dataRateInfo->dataRateType,
                MESSAGE_ReturnPacketSize(M802->currentMessage) +
                M802_11_DATA_FRAME_HDR_SIZE);

        hdr->duration =
            Mac802_11NanoToMicrosecond(
                M802->extraPropDelay + M802->sifs +
                PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    M802_11_SHORT_CTRL_FRAME_SIZE) +
                M802->extraPropDelay + M802->sifs +
                packetTransmissionDuration +
                M802->extraPropDelay + M802->sifs +
                PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    M802->dataRateInfo->dataRateType,
                    M802_11_SHORT_CTRL_FRAME_SIZE) +
                M802->extraPropDelay);

    }
    else {
        // Fragmentation Needs to be rewritten. (Jay)
        ERROR_ReportError("Mac802_11TransmitRTSFrame: "
            "Fragmentation not currently supported.\n");
    }//if//

    Mac802_11SetState(node, M802, M802_11_X_RTS);

    M802->rtsPacketsSentDcf++;

    if (!M802->useDvcs) {
        Mac802_11StartTransmittingPacket(
            node, M802, pktToPhy, M802->delayUntilSignalAirborn);
    }
    else {
        BOOL isCached = FALSE;
        double directionToSend = 0.0;

        AngleOfArrivalCache_GetAngle(
            &M802->directionalInfo->angleOfArrivalCache,
            destAddr,
            getSimTime(node),
            &isCached,
            &directionToSend);

        if (isCached) {
            Mac802_11StartTransmittingPacketDirectionally(
                node,
                M802,
                pktToPhy, M802->delayUntilSignalAirborn,
                directionToSend);
            PHY_LockAntennaDirection(node, M802->myMacData->phyNumber);
        }
        else {
            // Don't know which way to send, send omni.

            Mac802_11StartTransmittingPacket(node, M802, pktToPhy,
                                             M802->delayUntilSignalAirborn);
        }//if//
    }//if//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmitDataFrame.
//  PURPOSE:     Transmit the data frame.
//  PARAMETERS:  Node* node transmitting the data frame.
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11TransmitDataFrame(
    Node* node,
    MacData802_11* M802)
{
    NodeAddress destAddr;
    int dataRateType;

    destAddr = M802->currentNextHopAddress;
    ERROR_Assert(M802->currentNextHopAddress ==
                 M802->dataRateInfo->ipAddress,
                 "Address does not match");

    PHY_SetTxDataRateType(node, M802->myMacData->phyNumber,
                          M802->dataRateInfo->dataRateType);
    dataRateType = PHY_GetTxDataRateType(node, M802->myMacData->phyNumber);

    ERROR_Assert(M802->currentMessage != NULL,
                 "Mac802_11TransmitDataFrame: "
                 "There is no message in the local buffer.\n");

    if (destAddr == ANY_DEST) {
        Message* dequeuedPacket;
        M802_11FrameHdr* hdr;

        dequeuedPacket = M802->currentMessage;

        M802->broadcastPacketsSentDcf++;

        MESSAGE_AddHeader(node,
                          dequeuedPacket,
                          M802_11_DATA_FRAME_HDR_SIZE,
                          TRACE_802_11);

        hdr = (M802_11FrameHdr*)MESSAGE_ReturnPacket(dequeuedPacket);

        Mac802_11SetFieldsInDataFrameHdr(M802, hdr, destAddr, M802_11_DATA);
        hdr->duration = 0;

        Mac802_11ResetCurrentMessageVariables(M802);

        Mac802_11SetState(node, M802, M802_11_X_BROADCAST);

        ERROR_Assert(dataRateType == M802->highestDataRateTypeForBC,
                     "Wrong data rate");

        Mac802_11StartTransmittingPacket(node, M802, dequeuedPacket,
            M802->delayUntilSignalAirborn);
    }
    else {
        clocktype transmitDelay;
        M802_11SeqNoEntry *entry;
        M802_11FrameHdr* hdr;
        Message* pktToPhy = MESSAGE_Duplicate(node, M802->currentMessage);

        MESSAGE_AddHeader(node,
                          pktToPhy,
                          M802_11_DATA_FRAME_HDR_SIZE,
                          TRACE_802_11);

        hdr = (M802_11FrameHdr*)MESSAGE_ReturnPacket(pktToPhy);

        Mac802_11SetFieldsInDataFrameHdr(M802, hdr, destAddr, M802_11_DATA);

        M802->waitingForAckOrCtsFromAddress = destAddr;

        entry = Mac802_11GetSeqNo(node, M802, destAddr);
        ERROR_Assert(entry,
            "Mac802_11TransmitDataFrame: "
            "Unable to find or create sequence number entry.\n");
        hdr->seqNo = entry->toSeqNo;
        hdr->duration =
            Mac802_11NanoToMicrosecond(
                M802->extraPropDelay + M802->sifs +
                PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    M802_11_SHORT_CTRL_FRAME_SIZE) +
                M802->extraPropDelay);

        if (MESSAGE_ReturnPacketSize(M802->currentMessage) >
            M802_11_FRAG_THRESH)
        {
            // Fragmentation Needs to be rewritten. (Jay)
            ERROR_ReportError("Mac802_11TransmitDataFrame: "
                "Fragmentation is not currently supported.\n");
        }
        else {
            hdr->fragId = 0;

            Mac802_11SetState(node, M802, M802_11_X_UNICAST);

            if (MESSAGE_ReturnPacketSize(M802->currentMessage) >
                M802->rtsThreshold)
            {
                // Since using RTS-CTS, data packets have to wait
                // an additional SIFS.
                transmitDelay = M802->sifs;
            }
            else {
                // Not using RTS-CTS, so don't need to wait for SIFS
                // since already waited for DIFS or BO.
                transmitDelay = M802->delayUntilSignalAirborn;
            }

            if (!M802->useDvcs) {
                Mac802_11StartTransmittingPacket(
                    node, M802, pktToPhy, transmitDelay);
            }
            else {
                BOOL isCached = FALSE;
                double directionToSend = 0.0;

                AngleOfArrivalCache_GetAngle(
                    &M802->directionalInfo->angleOfArrivalCache,
                    destAddr,
                    getSimTime(node),
                    &isCached,
                    &directionToSend);

                if (isCached) {
                    Mac802_11StartTransmittingPacketDirectionally(
                        node, M802, pktToPhy, transmitDelay,
                        directionToSend);

                    PHY_LockAntennaDirection(node,
                                             M802->myMacData->phyNumber);
                }
                else {
                    // Don't know which way to send, send omni.

                    Mac802_11StartTransmittingPacket(node, M802, pktToPhy,
                                                     transmitDelay);
                }//if//
            }//if//
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmitFrame.
//  PURPOSE:     Initiate transmit of RTS or data frame.
//  PARAMETERS:  Node* node that wants to send a data frame.
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11TransmitFrame(
    Node* node,
    MacData802_11* M802)
{
    ERROR_Assert(Mac802_11PhyStatus(node, M802) == PHY_IDLE,
        "Mac802_11TransmitFrame: "
        "Physical status is not IDLE.\n");

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11TransmitFrame: "
        "There is no message in the local buffer.\n");

    if ((MESSAGE_ReturnPacketSize(M802->currentMessage) >
        M802->rtsThreshold)
        && (M802->currentNextHopAddress != ANY_DEST))
    {
        Mac802_11TransmitRTSFrame(node, M802);
    }
    else {
        Mac802_11TransmitDataFrame(node, M802);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11TransmissionHasFinished
//  PURPOSE:     Signal the end of transmission
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11TransmissionHasFinished(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->state) {
        case M802_11_X_ACK:
            Mac802_11AckTransmitted(node, M802);
            break;

        case M802_11_X_BROADCAST:
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11ResetCW(node, M802);
            Mac802_11CheckForOutgoingPacket(node, M802, TRUE);
            break;

        case M802_11_X_UNICAST:
            Mac802_11UnicastTransmitted(node, M802);
            break;

        case M802_11_X_RTS:
            Mac802_11RTSTransmitted(node, M802);
            break;

        case M802_11_X_CTS:
            Mac802_11CTSTransmitted(node, M802);
            break;

        case M802_11_X_CFP_BEACON:
            Mac802_11CfpPcBeaconTransmitted(node, M802);
            break;

        case M802_11_CFP_START:
            Mac802_11CfpFrameTransmitted(node, M802);
            break;

        default:
            ERROR_ReportError("Mac802_11TransmissionHasFinished: "
                "Invalid transmit state.\n");
            break;
    }//switch//
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CancelSomeTimersBecauseChannelHasBecomeBusy
//  PURPOSE:     Cancel timers since the channel is busy.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11CancelSomeTimersBecauseChannelHasBecomeBusy(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->state)
    {
        case M802_11_S_WF_DIFS_OR_EIFS:
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);
            break;

        case M802_11_S_BO:
            Mac802_11PauseBackoff(node, M802);
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);
            break;

        case M802_11_S_WFNAV:
            // Timer no longer need timer for NAV (may need to
            // set the timer if the channel becomes busy again.

            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);
            break;

        case M802_11_S_WF_CFP_BEACON:
            if (Mac802_11IsPC(M802))
            {
                // Cancel the PIFS beacon timer
                Mac802_11CancelTimer(node, M802);
                Mac802_11SetState(node, M802, M802_11_S_IDLE);
            }
            break;

        case M802_11_CFP_START:
            Mac802_11CfpCancelSomeTimersBecauseChannelHasBecomeBusy
                (node, M802);
            break;

        default:
            // Can't happen or ignore.
            break;
    }//switch//
}//Mac802_11CancelSomeTimersBecauseChannelHasBecomeBusy//



//--------------------------------------------------------------------------
//  NAME:        Mac802_11ExaminePotentialIncomingMessage
//  PURPOSE:     Examine incoming message.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               PhyStatusType phyStatus
//                  Status of the physical medium
//               clocktype receiveDuration
//                  Receive duration
//               const Message* thePacketIfItGetsThrough
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11ExaminePotentialIncomingMessage(
    Node* node,
    MacData802_11* M802,
    PhyStatusType phyStatus,
    clocktype receiveDuration,
    const Message* thePacketIfItGetsThrough)
{
    M802_11ShortControlFrame* hdr = (M802_11ShortControlFrame*)
        MESSAGE_ReturnPacket(thePacketIfItGetsThrough);

    M802->potentialIncomingMessage = thePacketIfItGetsThrough;

    ERROR_Assert(phyStatus == PHY_RECEIVING,
        "Mac802_11ExaminePotentialIncomingMessage: "
        "Physical is not in RECEIVING state.\n");

    M802->IsInExtendedIfsMode = TRUE;


    if (M802->state == M802_11_S_NAV_RTS_CHECK_MODE) {
        // The RTS (Request To Send) is getting a CTS so continue the
        // NAV wait and cancel the timer that would reset the NAV.

        Mac802_11SetState(node, M802, M802_11_S_IDLE);
        Mac802_11CancelTimer(node, M802);
    }//if//

    if (M802->stopReceivingAfterHeaderMode) {
        if ((hdr->frameType == M802_11_DATA) &&
            (!Mac802_11IsWaitingForResponseState(M802->state)))
        {
            clocktype headerCheckDelay =
                PHY_GetTransmissionDuration(node,
                                     M802->myMacData->phyNumber,
                                     PHY_GetRxDataRateType(
                                         node, M802->myMacData->phyNumber),
                                     M802_11_SHORT_CTRL_FRAME_SIZE);

            Mac802_11SetState(node, M802, M802_11_S_HEADER_CHECK_MODE);
            Mac802_11StartTimer(node, M802, headerCheckDelay);
        } else if (M802->state == M802_11_S_HEADER_CHECK_MODE) {
            Mac802_11SetState(node, M802, M802_11_S_IDLE);
            Mac802_11CancelTimer(node, M802);
        }//if//
    }//if//


}//Mac802_11ExaminePotentialIncomingMessage//



//--------------------------------------------------------------------------
//  NAME:        Mac802_11PhyStatusIsNowIdleStartSendTimers
//  PURPOSE:     Start send timer since PHY indicates idle state.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static //inline//
void Mac802_11PhyStatusIsNowIdleStartSendTimers(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->state) {
        case M802_11_S_IDLE:
        case M802_11_S_HEADER_CHECK_MODE:
        {
            if (M802->cfpBeaconIsDue
                && Mac802_11CanHandleDueCfpBeacon(node, M802))
            {
                return;
            }
            else if (M802->currentMessage == NULL) {
                // Nothing to send. Stay Idle.
            }
            else {
                // Start up sending process again.
                // Note: uses current Backoff delay.
                Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(node, M802);
            }//if//
            break;
        }

        case M802_11_CFP_START: {
            Mac802_11CfpPhyStatusIsNowIdleStartSendTimers(node, M802);
            break;
        }

        // In these states, we don't care about the state of the phy.

        case M802_11_S_WFCTS:
        // M802_11_S_WFACK:
        case M802_11_S_WFDATA:
        // M802_11_S_WFACK:
        case M802_11_S_WFACK:
        {

            break;
        }

        // These states can't happen.

        // Currently Disabled
        case M802_11_S_NAV_RTS_CHECK_MODE:
        // default:

        case M802_11_S_WFNAV:
        // default:

        case M802_11_S_WF_DIFS_OR_EIFS:
        // default:

        case M802_11_S_BO:
        // default:

        case M802_11_S_WF_CFP_BEACON:
        // default:

        default:
            ERROR_ReportError("Mac802_11PhyStatusIsNowIdleStartSendTimers: "
                "Invalid state encountered.\n");
            break;
    }//switch//

}//Mac802_11PhyStatusIsNowIdleStartSendTimers//



//--------------------------------------------------------------------------
// Inter-Layer Interface Routines
//--------------------------------------------------------------------------





//--------------------------------------------------------------------------



//--------------------------------------------------------------------------
//  NAME:        Mac802_11NetworkLayerHasPacketToSend
//  PURPOSE:     Called when network layer buffers transition from empty.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11NetworkLayerHasPacketToSend(
    Node* node,
    MacData802_11* M802)
{
    if (M802->currentMessage != NULL) {
        // Ignore this signal if 802.11 is already holding a packet
        return;
    }

    Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer(node, M802);
    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11NetworkLayerHasPacketToSend: "
        "Local buffer does not have the network packet.\n");


    if (M802->state == M802_11_S_IDLE) {
        if ((M802->useDvcs) &&
            ((Mac802_11PhyStatus(node, M802) == PHY_IDLE) ||
             (Mac802_11PhyStatus(node, M802) == PHY_SENSING)))
        {
            Mac802_11SetPhySensingDirectionForNextOutgoingPacket(node, M802);
        }//if//

        Mac802_11SetBackoffIfZero(node, M802);
        Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(node, M802);

    }//if//
}//Mac802_11NetworkLayerHasPacketToSend//


//--------------------------------------------------------------------------


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ReceivePacketFromPhy
//  PURPOSE:     Receive a packet from physical layer
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress address
//                  Node address of station
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11ReceivePacketFromPhy(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11ShortControlFrame* hdr =
        (M802_11ShortControlFrame*) MESSAGE_ReturnPacket(msg);

    M802->IsInExtendedIfsMode = FALSE;

    //
    // Since in QualNet it's possible to have two events occurring
    // at the same time, enforce the fact that when a node is
    // transmitting, a node can't be receiving a frame at the same time.
    //
    ERROR_Assert(!Mac802_11IsTransmittingState(M802->state),
        "Mac802_11ReceivePacketFromPhy: "
        "Cannot receive packet while in transmit state.\n");

    if (Mac802_11IsInCfp(M802->state)) {
        Mac802_11CfpReceivePacketFromPhy(node, M802, msg);
        return;
    }

    if (M802->selfAddr == hdr->destAddr) {
        switch (hdr->frameType) {
            case M802_11_RTS: {
                if (!M802->useDvcs) {
                    if (Mac802_11IsWaitingForResponseState(M802->state)) {
                    }
                    else if ((!Mac802_11WaitForNAV(node, M802)) &&
                             (Mac802_11PhyStatus(node, M802) == PHY_IDLE))
                    {
                        // Transmit CTS only if NAV (software carrier sense)
                        // and the phy says the channel is idle.

                        Mac802_11Trace(node, M802, msg, "Receive");

                        Mac802_11CancelTimer(node, M802);
                        Mac802_11TransmitCTSFrame(node, M802, msg);
                    }//if//
                }
                else {
                    NodeAddress sourceNodeAddress =
                        ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                        ->sourceAddr;
                    BOOL nodeIsInCache;

                    if (M802->useDvcs) {
                        Mac802_11UpdateDirectionCacheWithCurrentSignal(
                            node, M802, sourceNodeAddress, &nodeIsInCache);
                    }//if//

                    if (Mac802_11IsWaitingForResponseState(M802->state)) {
                        // Do Nothing.
                    } else if (
                        (!DirectionalNav_SignalIsNaved(
                            &M802->directionalInfo->directionalNav,
                            Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802),
                            getSimTime(node))) &&
                        (PHY_MediumIsIdleInDirection(
                            node,
                            M802->myMacData->phyNumber,
                            Mac802_11GetLastSignalsAngleOfArrivalFromRadio(node, M802))))
                    {
                        // Transmit CTS only if NAV (software carrier sense)
                        // and the phy says the channel is idle.

                        Mac802_11Trace(node, M802, msg, "Receive");

                        Mac802_11CancelTimer(node, M802);
                        Mac802_11TransmitCTSFrame(node, M802, msg);
                    }//if//
                }

                MESSAGE_Free(node, msg);
                break;
            }

            case M802_11_CTS: {
                Mac802_11Trace(node, M802, msg, "Receive");

                if (M802->state == M802_11_S_WFNAV) {
                    // Cannot respond to this CTS
                    // because NAV is set
                    MESSAGE_Free(node, msg);
                    break;
                }

                ERROR_Assert(M802->state == M802_11_S_WFCTS,
                    "Mac802_11ReceivePacketFromPhy: "
                    "Received unexpected CTS packet.\n");

                if (M802->useDvcs) {
                    BOOL nodeIsInCache;

                    Mac802_11UpdateDirectionCacheWithCurrentSignal(
                        node, M802, M802->waitingForAckOrCtsFromAddress,
                        &nodeIsInCache);

                }//if//

                Mac802_11CancelTimer(node, M802);

                M802->SSRC = 0;

                Mac802_11TransmitDataFrame(node, M802);

                MESSAGE_Free(node, msg);
                break;
            }

            case M802_11_DATA:
            case M802_11_CF_DATA_ACK: {
                if (Mac802_11IsMyUnicastDataFrame(M802, msg)) {
                    NodeAddress sourceNodeAddress =
                        ((M802_11FrameHdr*)MESSAGE_ReturnPacket(msg))
                        ->sourceAddr;

                    if (M802->useDvcs) {
                        BOOL nodeIsInCache;

                        Mac802_11UpdateDirectionCacheWithCurrentSignal(
                            node, M802, sourceNodeAddress,
                            &nodeIsInCache);
                    }//if//

                    // This is not in the standard, but ns-2 does.
                    // Mac802_11ResetCW(node, M802);
                    // This is not in the standard, but ns-2 does.
                    // M802->SSRC = 0;

                    Mac802_11ProcessFrame(node, M802, msg);
                }
                else {
                    Mac802_11Trace(node, M802, NULL,
                        "Ignored directed data frame.");
                    MESSAGE_Free(node, msg);
                }
                break;
            }

            case M802_11_ACK: {
                Mac802_11Trace(node, M802, msg, "Receive");

                ERROR_Assert(M802->state == M802_11_S_WFACK,
                    "Mac802_11ReceivePacketFromPhy: "
                    "Received unexpected ACK packet.\n");

                if (M802->useDvcs) {
                    BOOL nodeIsInCache;

                    Mac802_11UpdateDirectionCacheWithCurrentSignal(
                        node, M802, M802->waitingForAckOrCtsFromAddress,
                        &nodeIsInCache);

                }//if//

                ERROR_Assert(M802->dataRateInfo->ipAddress ==
                             M802->currentNextHopAddress,
                             "Address does not match");

                ERROR_Assert(M802->dataRateInfo->ipAddress ==
                             M802->waitingForAckOrCtsFromAddress,
                             "Address does not match");

                M802->dataRateInfo->numAcksInSuccess++;

                Mac802_11CancelTimer(node, M802);

                Mac802_11ProcessAck(node, M802, msg);
                MESSAGE_Free(node, msg);
                break;
            }

            default:
            {
                char errString[MAX_STRING_LENGTH];

                // For a BSS station that could have missed a beacon,
                // enter CFP and process CF frame as normal.
                if (Mac802_11IsBssStation(M802) &&
                    Mac802_11IsFrameFromMyAP(M802,
                        ((M802_11LongControlFrame*)MESSAGE_ReturnPacket(msg))
                        ->sourceAddr) &&
                    hdr->duration == M802_11_CF_FRAME_DURATION)
                {
                    Mac802_11Trace(node, M802, msg,
                        "BSS station misses beacon, goes into CFP");

                    // Enter CFP and process frame as normal
                    Mac802_11CfpStationInitiateCfPeriod(node, M802);
                    Mac802_11CfpReceivePacketFromPhy(node, M802, msg);
                    return;
                }

                sprintf(errString,
                    "Mac802_11ReceivePacketFromPhy: "
                    "Received unknown unicast frame type %d\n",
                    hdr->frameType);
                ERROR_ReportError(errString);
                break;
            }
        }//switch
    }
    else if (hdr->destAddr == ANY_DEST) {
        switch (hdr->frameType) {
            case M802_11_DATA:
            case M802_11_CF_DATA_ACK: {
                if (Mac802_11IsMyBroadcastDataFrame(M802, msg)) {
                    NodeAddress sourceNodeAddress =
                        ((M802_11FrameHdr*)MESSAGE_ReturnPacket(msg))
                        ->sourceAddr;

                    if (M802->useDvcs) {
                        BOOL nodeIsInCache;

                        Mac802_11UpdateDirectionCacheWithCurrentSignal(
                            node, M802, sourceNodeAddress,
                            &nodeIsInCache);
                    }//if//

                    Mac802_11ProcessFrame(node, M802, msg);
                }
                else {
                    Mac802_11Trace(node, M802, NULL,
                        "Ignored data broadcast");
                    MESSAGE_Free(node, msg);
                }
                break;
            }

            case M802_11_BEACON:
            {
                // Currently handling only PCF beacons.
                Mac802_11ProcessCfpBeacon(node, M802, msg);
                MESSAGE_Free(node, msg);
                break;
            }

            case M802_11_CF_END:
            case M802_11_CF_END_ACK:
            {
                Mac802_11ProcessCfpEnd(node, M802, msg);
                MESSAGE_Free(node, msg);
                break;
            }

            default:
            {
                char errString[MAX_STRING_LENGTH];

                sprintf(errString,
                    "Mac802_11ReceivePacketFromPhy: "
                    "Received unknown broadcast frame type %d\n",
                    hdr->frameType);
                ERROR_ReportError(errString);
                break;
             }
        }//switch
    }
    else {
        // Does not belong to node

        Mac802_11ProcessNotMyFrame(
            node, M802, Mac802_11MicroToNanosecond(hdr->duration),
            (hdr->frameType == M802_11_RTS));

        if (M802->useDvcs) {
            NodeAddress sourceNodeAddress = INVALID_ADDRESS;

            if (hdr->frameType == M802_11_RTS) {
                sourceNodeAddress =
                    ((M802_11LongControlFrame*)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr;
            }
            else if (hdr->frameType == M802_11_DATA) {
                sourceNodeAddress =
                    ((M802_11FrameHdr*)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr;
            }

            if (sourceNodeAddress != INVALID_ADDRESS) {
                BOOL nodeIsInCache;

                Mac802_11UpdateDirectionCacheWithCurrentSignal(
                    node, M802, sourceNodeAddress,
                    &nodeIsInCache);
            }
        }

        if ((M802->myMacData->promiscuousMode == TRUE) &&
            Mac802_11IsFrameAUnicastDataTypeForPromiscuousMode(
                (M802_11MacFrameType) hdr->frameType, msg))
        {
            M802_11FrameHdr* fHdr = (M802_11FrameHdr*)hdr;

            Mac802_11HandlePromiscuousMode(node, M802, msg,
                fHdr->sourceAddr, fHdr->destAddr);
        }

        MESSAGE_Free(node,msg);
    }//if//

    if (M802->state == M802_11_S_IDLE) {
        if (M802->useDvcs) {
            Mac802_11SetPhySensingDirectionForNextOutgoingPacket(node, M802);
        }//if//

        if (Mac802_11PhyStatus(node, M802) == PHY_IDLE) {
            Mac802_11PhyStatusIsNowIdleStartSendTimers(node, M802);
        }
    }//if//


}//Mac802_11ReceivePacketFromPhy//


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ReceivePhyStatusChangeNotification
//  PURPOSE:     React to change in physical status.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               PhyStatusType oldPhyStatus
//                  The previous physical state
//               PhyStatusType newPhyStatus
//                  New physical state
//               clocktype receiveDuration
//                  Receiving duration
//               const Message* potentialIncomingPacket
//                  Incoming packet
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11ReceivePhyStatusChangeNotification(
    Node* node,
    MacData802_11* M802,
    PhyStatusType oldPhyStatus,
    PhyStatusType newPhyStatus,
    clocktype receiveDuration,
    const Message* potentialIncomingPacket)
{

    switch (oldPhyStatus) {
        case PHY_IDLE: {
            switch (newPhyStatus) {
                case PHY_SENSING: {
                    Mac802_11CancelSomeTimersBecauseChannelHasBecomeBusy
                        (node, M802);
                    break;
                }

                case PHY_RECEIVING: {
                    Mac802_11CancelSomeTimersBecauseChannelHasBecomeBusy
                        (node, M802);
                    Mac802_11ExaminePotentialIncomingMessage(
                        node, M802, newPhyStatus,
                        receiveDuration, potentialIncomingPacket);
                    break;
                }

                case PHY_TRANSMITTING: {
                    break;
                }

                default: {
                    ERROR_ReportError(
                        "Mac802_11ReceivePhyStatusChangeNotification: "
                        "Invalid physical state change from IDLE.\n");
                    break;
                }
            }//switch//
            break;
        }

        case PHY_SENSING: {
            switch (newPhyStatus) {
                case PHY_IDLE: {
                    Mac802_11PhyStatusIsNowIdleStartSendTimers(node, M802);
                    break;
                }
                case PHY_RECEIVING: {
                    Mac802_11ExaminePotentialIncomingMessage(node, M802,
                        newPhyStatus, receiveDuration,
                        potentialIncomingPacket);
                    break;
                }
                case PHY_TRANSMITTING: {
                    // 802.11 will transmit ACKs and CTS even when sensing.
                    break;
                }
                default: {
                    ERROR_ReportError(
                        "Mac802_11ReceivePhyStatusChangeNotification: "
                        "Invalid physical state change from SENSING.\n");
                    break;
                }
            }//switch//

            break;
        }

        case PHY_RECEIVING: {

            switch (newPhyStatus) {
                case PHY_IDLE: {
                    Mac802_11PhyStatusIsNowIdleStartSendTimers
                        (node, M802);
                    break;
                }

                case PHY_SENSING: {
                    // Receive was canceled and packet dropped.


                    if (M802->state == M802_11_S_HEADER_CHECK_MODE) {
                        Mac802_11SetState(node, M802, M802_11_S_IDLE);
                        Mac802_11CancelTimer(node, M802);
                    }//if//

                    break;
                }

                case PHY_RECEIVING: {
                    // "Captured" stronger packet.
                    Mac802_11ExaminePotentialIncomingMessage(
                        node, M802, newPhyStatus,
                        receiveDuration, potentialIncomingPacket);
                    break;
                }

                default: {
                    ERROR_ReportError(
                        "Mac802_11ReceivePhyStatusChangeNotification: "
                        "Invalid physical state change from RECEIVING.\n");
                    break;
                }
            }//switch//

            break;
        }

        case PHY_TRANSMITTING: {
            switch (newPhyStatus) {
                case PHY_IDLE:
                case PHY_SENSING:
                case PHY_RECEIVING:
                {
                    Mac802_11TransmissionHasFinished(node, M802);
                    break;
                }

                default: {
                    ERROR_ReportError(
                        "Mac802_11ReceivePhyStatusChangeNotification: "
                        "Invalid physical state change from TRANSMITTING.\n");
                    break;
                }
            }//switch//

            break;
        }

        default: {
            ERROR_ReportError(
                "Mac802_11ReceivePhyStatusChangeNotification: "
                "Invalid old physical state.\n");
            break;
        }
    }//switch//


}//Mac802_11ReceivePhyStatusChangeNotification//



//--------------------------------------------------------------------------



//--------------------------------------------------------------------------
//  NAME:        Mac802_11HandleTimeout.
//  PURPOSE:     Handle expiry of timer depending on state.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11HandleTimeout(
    Node* node,
    MacData802_11* M802)
{

    if (M802->cfpBeaconIsDue
        && Mac802_11CanHandleDueCfpBeacon(node, M802))
    {
        return;
    }

    if (M802->useDvcs) {
        //
        // A Packet sequence could possibly have failed,
        // go back to sending/receiving omnidirectionally if not already
        // doing so.
        //
        PHY_UnlockAntennaDirection(
            node, M802->myMacData->phyNumber);
    }//if//

    switch (M802->state) {
        case M802_11_S_WF_DIFS_OR_EIFS: {
            if (M802->BO == 0) {
                if (M802->currentMessage != NULL) {
                    Mac802_11TransmitFrame(node, M802);
                }
                else {
                    Mac802_11SetState(node, M802, M802_11_S_IDLE);
                }
            }
            else {
                Mac802_11ContinueBackoff(node, M802);
                Mac802_11SetState(node, M802, M802_11_S_BO);
                Mac802_11StartTimer(node, M802, M802->BO);
            }//if//

            break;
        }

        case M802_11_S_BO: {
            ERROR_Assert(
                (M802->lastBOTimeStamp + M802->BO) == getSimTime(node),
                "Mac802_11HandleTimeout: "
                "Backoff period does not match simulation time.\n");
            M802->BO = 0;

            if (M802->currentMessage != NULL) {
                Mac802_11TransmitFrame(node, M802);
            }
            else {
                Mac802_11SetState(node, M802, M802_11_S_IDLE);
            }

            break;
        }

        case M802_11_S_WFACK: {
            ERROR_Assert(M802->dataRateInfo->ipAddress ==
                         M802->currentNextHopAddress,
                         "Address does not match");

            M802->dataRateInfo->numAcksFailed++;
            Mac802_11AdjustDataRateForResponseTimeout(node, M802);

            M802->retxDueToAckDcf++;
            Mac802_11Trace(node, M802, NULL, "Retransmit wait for ACK");
            Mac802_11Retransmit(node, M802);

            break;
        }

        case M802_11_S_NAV_RTS_CHECK_MODE:
        case M802_11_S_WFNAV:
        {
            M802->NAV = 0;

            if (M802->currentMessage != NULL) {
                Mac802_11AttemptToGoIntoWaitForDifsOrEifsState(node, M802);
            }
            else {
                Mac802_11SetState(node, M802, M802_11_S_IDLE);
            }
            break;
        }

        case M802_11_S_WFCTS: {
            ERROR_Assert(M802->dataRateInfo->ipAddress ==
                         M802->currentNextHopAddress,
                         "Address does not match");

            M802->dataRateInfo->numAcksFailed++;
            Mac802_11AdjustDataRateForResponseTimeout(node, M802);

            M802->retxDueToCtsDcf++;
            Mac802_11Trace(node, M802, NULL, "Retransmit wait for CTS");
            Mac802_11Retransmit(node, M802);

            break;
        }

        case M802_11_S_WFDATA: {
            Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
            break;
        }

        case M802_11_CFP_START: {
            Mac802_11CfpHandleTimeout(node, M802);
            break;
        }

        case M802_11_S_HEADER_CHECK_MODE: {
            const M802_11LongControlFrame* hdr =
                (const M802_11LongControlFrame*)
                MESSAGE_ReturnPacket(M802->potentialIncomingMessage);

            BOOL frameHeaderHadError;
            clocktype endSignalTime;

            BOOL packetForMe =
                ((hdr->destAddr == node->nodeId) ||
                 (hdr->destAddr == ANY_DEST));

            ERROR_Assert((hdr->frameType != M802_11_CTS) &&
                         (hdr->frameType != M802_11_ACK),
                         "Frame type should not be CTS or ACK");

            PHY_TerminateCurrentReceive(
                node, M802->myMacData->phyNumber, packetForMe,
                &frameHeaderHadError, &endSignalTime);

            if (Mac802_11PhyStatus(node, M802) == PHY_RECEIVING) {
                // Keep Receiving (wait).
                Mac802_11SetState(node, M802, M802_11_S_IDLE);
            }
            else {
                M802->noOutgoingPacketsUntilTime = endSignalTime;

                if (DEBUG) {
                    char timeStr[MAX_STRING_LENGTH];
                    TIME_PrintClockInSecond(getSimTime(node), timeStr);

                    if (packetForMe) {
                        printf("%d : %s For Me/Error\n",
                               node->nodeId, timeStr);
                    }
                    else if (frameHeaderHadError) {
                        printf("%d : %s For Other/Error\n",
                               node->nodeId, timeStr);
                    }
                    else if (M802->currentMessage != NULL) {
                        printf("%d : %s For Other/Have Message\n",
                               node->nodeId, timeStr);
                    }
                    else {
                        printf("%d : %s For Other/No Message\n",
                               node->nodeId, timeStr);
                    }//if//
                }//if//

                if (!frameHeaderHadError) {
                    clocktype headerCheckDelay =
                        PHY_GetTransmissionDuration(
                            node,
                            M802->myMacData->phyNumber,
                            PHY_GetRxDataRateType(
                                node, M802->myMacData->phyNumber),
                            M802_11_SHORT_CTRL_FRAME_SIZE);


                    // Extended IFS mode if for backing off more when a
                    // collision happens and a packet is garbled.
                    // We are forcably terminating the packet and thus
                    // we should not use extended IFS delays.

                    M802->IsInExtendedIfsMode = FALSE;

                    // It should be able to learn AOA, but temporarily disabled.
                    // UpdateDirectionCacheWithCurrentSignal(
                    //     node, M802, hdr->sourceAddr, TRUE);

                    Mac802_11ProcessNotMyFrame(
                        node, M802,
                        (Mac802_11MicroToNanosecond(hdr->duration)
                            - headerCheckDelay),
                        (hdr->frameType == M802_11_RTS));
                }
                else {
                    // Header had error.  Assume no information received.
                }//if//

                Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
            }//if//

            break;
        }

        case M802_11_S_WF_CFP_BEACON: {
            ERROR_Assert(Mac802_11IsPC(M802),
                "Mac802_11HandleTimeout: "
                "Wait For Beacon state for an AP or non-BSS station.\n");

            Mac802_11CfpPcAttemptToTransmitCfBeacon(node, M802);

            break;
        }

        default: {
            printf("MAC_802_11: Node %u got unknown state type %d\n",
                   node->nodeId, M802->state);

            ERROR_ReportError("Mac802_11HandleTimeout: "
                "Timeout in unexpected state.\n");
        }
    }


}//Mac802_11HandleTimeout//



//--------------------------------------------------------------------------
//  NAME:        Mac802_11Layer.
//  PURPOSE:     Handle timers and layer messages.
//  PARAMETERS:  Node* node
//                  Node handling the incoming messages
//               int interfaceIndex
//                  Interface index
//               Message* msg
//                  Message for node to interpret.
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
void Mac802_11Layer(Node* node, int interfaceIndex, Message* msg)
{
    MacData802_11* M802 =
        (MacData802_11*)node->macData[interfaceIndex]->macVar;
    unsigned timerSequenceNumber = *(int*)MESSAGE_ReturnInfo(msg);

    ERROR_Assert(timerSequenceNumber <= M802->timerSequenceNumber,
        "Mac802_11Layer: Received invalid timer message.\n");

    switch (msg->eventType) {
        case MSG_MAC_TimerExpired: {
            if (timerSequenceNumber == M802->timerSequenceNumber) {
                Mac802_11HandleTimeout(node, M802);
            }
            break;
        }

        case MSG_MAC_802_11_Beacon: {
            ERROR_Assert(
                timerSequenceNumber == M802->cfpBeaconSequenceNumber,
                "Mac802_11Layer: Beacon timer has wrong sequence number.\n");
            ERROR_Assert(!Mac802_11IsInCfp(M802->state),
                "Mac802_11Layer: Beacon timer when in CFP state.\n");

            M802->cfpBeaconIsDue = TRUE;
            (void) Mac802_11CanHandleDueCfpBeacon(node, M802);
            break;
        }

        case MSG_MAC_802_11_CfpEnd: {
            if (timerSequenceNumber == M802->cfpEndSequenceNumber) {
                Mac802_11CfpHandleEndTimer(node, M802);
            }
            break;
        }

        default: {
            ERROR_ReportError("Mac802_11Layer: "
                "Unknown message event type.\n");
            break;
        }
    } //switch

    MESSAGE_Free(node, msg);
}


//--------------------------------------------------------------------------
// NAME:       Mac802_11Init
// PURPOSE:    Initialize 802.11, read user input.
// PARAMETERS  Node* node
//                Node being initialized.
//             NodeInput* nodeInput
//                Structure containing contents of input file.
//             MacSubnetMemberData* subnetList
//                Number of nodes in subnet.
//             int nodesInSubnet
//                Number of nodes in subnet.
//             int subnetListIndex
//             NodeAddress subnetAddress
//                Subnet address.
//             int numHostBits
//                number of host bits.
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
void Mac802_11Init(
    Node* node,
    int interfaceIndex,
    const NodeInput* nodeInput,
    PhyModel phyModel,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType,
    /*unsigned int tla,
    unsigned int nla,
    unsigned int sla*/
    in6_addr *subnetAddr,
    unsigned int subnetPrefixLen)
{
    BOOL wasFound;
    char yesOrNo[MAX_STRING_LENGTH];
    int anIntInput;
    Address address;
    MacData802_11* M802 =
        (MacData802_11*)MEM_malloc(sizeof(MacData802_11));

    Mac802_11CheckHeaderSizes();

    memset(M802, 0, sizeof(MacData802_11));

    M802->myMacData = node->macData[interfaceIndex];
    M802->myMacData->macVar = (void*)M802;

    RANDOM_SetSeed(M802->seed,
                   node->globalSeed,
                   node->nodeId,
                   MAC_PROTOCOL_802_11,
                   interfaceIndex);

    if (phyModel == PHY802_11a) {
        M802->cwMin = M802_11a_CW_MIN;
        M802->cwMax = M802_11a_CW_MAX;
        M802->slotTime = M802_11a_SLOT_TIME;
        M802->sifs = M802_11a_SIFS;
        M802->delayUntilSignalAirborn = M802_11a_DELAY_UNTIL_SIGNAL_AIRBORN;
    }
    else if (phyModel == PHY802_11b) {
        M802->cwMin = M802_11b_CW_MIN;
        M802->cwMax = M802_11b_CW_MAX;
        M802->slotTime = M802_11b_SLOT_TIME;
        M802->sifs = M802_11b_SIFS;
        M802->delayUntilSignalAirborn = M802_11b_DELAY_UNTIL_SIGNAL_AIRBORN;
    }
    M802->propagationDelay = M802_11_PROPAGATION_DELAY;

    M802->txSifs = M802->sifs - M802->delayUntilSignalAirborn;

    M802->difs = M802->sifs + (2 * M802->slotTime);
    M802->txDifs = M802->txSifs + (2 * M802->slotTime);

    M802->extendedIfsDelay =
        M802->sifs +
        PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            MAC_PHY_LOWEST_MANDATORY_RATE_TYPE,
            M802_11_SHORT_CTRL_FRAME_SIZE) +
        M802->difs;


    M802->state = M802_11_S_IDLE;
    M802->prevState = M802_11_S_IDLE;
    M802->IsInExtendedIfsMode = FALSE;

    M802->noResponseTimeoutDuration = 0;
    M802->CW = M802->cwMin;
    M802->BO = 0;
    Mac802_11ResetAckVariables(M802);
    M802->NAV = 0;
    M802->noOutgoingPacketsUntilTime = 0;

    M802->seqNoHead = NULL;
    M802->dataRateHead = NULL;

    M802->timerSequenceNumber = 0;

    M802->extraPropDelay =
        M802->myMacData->propDelay - M802->propagationDelay;


    Mac802_11ResetCurrentMessageVariables(M802);

    PHY_GetLowestTxDataRateType(
        node, M802->myMacData->phyNumber, &(M802->lowestDataRateType));
    PHY_GetHighestTxDataRateType(
        node, M802->myMacData->phyNumber, &(M802->highestDataRateType));
    PHY_GetHighestTxDataRateTypeForBC(
        node, M802->myMacData->phyNumber, &(M802->highestDataRateTypeForBC));
    M802->rateAdjustmentTime = M802_11_DATA_RATE_ADJUSTMENT_TIME;

    M802->stationType = M802_11_STATION_TYPE_DEFAULT;

    M802->bssAddr = INVALID_ADDRESS;
    M802->selfAddr = INVALID_ADDRESS;

    M802->pktsToSend = 0;
    //M802->pktsLostOverflow = 0;

    M802->unicastPacketsSentDcf = 0;
    M802->broadcastPacketsSentDcf = 0;

    M802->unicastPacketsGotDcf = 0;
    M802->broadcastPacketsGotDcf = 0;

    M802->ctsPacketsSentDcf = 0;
    M802->rtsPacketsSentDcf = 0;
    M802->ackPacketsSentDcf = 0;

    M802->retxDueToCtsDcf = 0;
    M802->retxDueToAckDcf = 0;

    M802->retxAbortedRtsDcf = 0;
    M802->pktsDroppedDcf = 0;


    M802->potentialIncomingMessage = NULL;
    M802->directionalInfo = NULL;

    // Read short retry count.
    // Format is :
    //      MAC-802.11-SHORT-PACKET-TRANSMIT-LIMIT <value>
    // Default is M802_11_SHORT_RETRY_LIMIT (value 7)
    //
    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);
    IO_ReadInt(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-SHORT-PACKET-TRANSMIT-LIMIT",
        &wasFound,
        &anIntInput);

    if (wasFound) {
        M802->shortRetryLimit = anIntInput;
        ERROR_Assert(M802->shortRetryLimit > 0,
            "Mac802_11Init: "
            "Value of MAC-802.11-SHORT-PACKET-TRANSMIT-LIMIT "
            "should be greater than zero.\n");
    }
    else {
        M802->shortRetryLimit = M802_11_SHORT_RETRY_LIMIT;
    }

    M802->directionalShortRetryLimit = M802->shortRetryLimit + 1;

    // Read long retry count.
    // Format is :
    //      MAC-802.11-LONG-PACKET-TRANSMIT-LIMIT <value>
    // Default is M802_11_LONG_RETRY_LIMIT (value 4)
    //
    IO_ReadInt(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-LONG-PACKET-TRANSMIT-LIMIT",
        &wasFound,
        &anIntInput);

    if (wasFound) {
        M802->longRetryLimit = anIntInput;
        ERROR_Assert(M802->longRetryLimit > 0,
            "Mac802_11Init: "
            "Value of MAC-802.11-LONG-PACKET-TRANSMIT-LIMIT "
            "should be greater than zero.\n");
    }
    else {
        M802->longRetryLimit = M802_11_LONG_RETRY_LIMIT;
    }

    // Read RTS threshold.
    // Format is :
    //      MAC-802.11-RTS-THRESHOLD <value>
    // Default is M802_11_RTS_THRESH (value 0, RTS/CTS for all packets)
    //
    IO_ReadInt(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-RTS-THRESHOLD",
        &wasFound,
        &anIntInput);

    if (wasFound) {
        M802->rtsThreshold = anIntInput;
        ERROR_Assert(M802->rtsThreshold >= 0,
            "Mac802_11Init: "
            "Value of MAC-802.11-RTS-THRESHOLD is negative.\n");
    }
    else {
        M802->rtsThreshold = M802_11_RTS_THRESH;
    }

    IO_ReadString(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-STOP-RECEIVING-AFTER-HEADER-MODE",
        &wasFound,
        yesOrNo);

    if ((!wasFound) || (strcmp(yesOrNo, "NO") == 0)) {
        M802->stopReceivingAfterHeaderMode = FALSE;
    }
    else if (strcmp(yesOrNo, "YES") == 0) {
        M802->stopReceivingAfterHeaderMode = TRUE;
    }//if//

    IO_ReadString(
        node->nodeId,
        &address, //M802->selfAddr,
        nodeInput,
        "MAC-802.11-DIRECTIONAL-ANTENNA-MODE",
        &wasFound,
        yesOrNo);

    if ((!wasFound) || (strcmp(yesOrNo, "NO") == 0)) {
        M802->useDvcs = FALSE;
    }
    else if (strcmp(yesOrNo, "YES") == 0) {
        clocktype directionCacheExpirationDelay;
        double defaultNavedDeltaAngle;

        M802->useDvcs = TRUE;
        IO_ReadTime(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-DIRECTION-CACHE-EXPIRATION-TIME",
            &wasFound,
            &directionCacheExpirationDelay);

        if (!wasFound) {
            ERROR_ReportError(
                "Expecting "
                "802.11-DIRECTION-CACHE-EXPIRATION-TIME "
                "parameter\n");
        }//if//

        IO_ReadDouble(
            node->nodeId,
            &address,//M802->selfAddr,
            nodeInput,
            "MAC-802.11-DIRECTIONAL-NAV-AOA-DELTA-ANGLE",
            &wasFound,
            &defaultNavedDeltaAngle);

        if (!wasFound) {
            ERROR_ReportError(
                "Expecting "
                "MAC-802.11-DIRECTIONAL-NAV-AOA-DELTA-ANGLE "
                "parameter\n");
        }//if//

        M802->directionalInfo = (DirectionalAntennaMacInfoType*)
            MEM_malloc(sizeof(DirectionalAntennaMacInfoType));

        InitDirectionalAntennalMacInfo(
            M802->directionalInfo,
            directionCacheExpirationDelay,
            defaultNavedDeltaAngle);

        IO_ReadInt(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-DIRECTIONAL-SHORT-PACKET-TRANSMIT-LIMIT",
            &wasFound,
            &M802->directionalShortRetryLimit);
    }
    else {
        ERROR_ReportError(
            "Expecting YES or NO for "
            "802.11-DIRECTIONAL-ANTENNA-MODE parameter\n");
    }//if//

    // Ipv6 Modification
    M802->selfAddr = NetworkIpGetLinkLayerAddress(node,
        M802->myMacData->interfaceIndex);

    // Initially, assume that interface is part of IBSS.
    // The use of a BSS address is not significant for
    // IBSSs in the implementation.
    M802->bssAddr = M802->selfAddr;

    // Ipv6 Modification
    if (networkType == NETWORK_IPV6)
    {
        Mac802_11BssInit(node, nodeInput, M802,
        subnetList, nodesInSubnet, subnetListIndex,
        0, 0,
        networkType, subnetAddr,subnetPrefixLen);
    }
    else
    {
    Mac802_11BssInit(node, nodeInput, M802,
        subnetList, nodesInSubnet, subnetListIndex,
        subnetAddress, numHostBits);
    }

    if (Mac802_11IsAP(M802) || Mac802_11IsBssStation(M802)) {

        if (networkType == NETWORK_IPV6)
        {
            Mac802_11CfpInit(node, nodeInput, M802,
                subnetList, nodesInSubnet, subnetListIndex,
                0, 0, networkType, subnetAddr,subnetPrefixLen);
        }
        else
        {
        Mac802_11CfpInit(node, nodeInput, M802,
            subnetList, nodesInSubnet, subnetListIndex,
            subnetAddress, numHostBits);
    }
    }

    Mac802_11TraceInit(node, nodeInput, M802);

// ANTS01 begin:
    M802->avgSendTimeANTS01 = 0;
// ANTS01

    #ifdef PARALLEL //Parallel
        PARALLEL_SetProtocolIsNotEOTCapable(node);
        PARALLEL_SetMinimumLookaheadForInterface(node,
                                                 M802->delayUntilSignalAirborn);
    #endif //endParallel
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11PrintStats
//  PURPOSE:     Print protocol statistics.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               int interfaceIndex
//                  interfaceIndex, Interface index
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11PrintStats(Node* node, MacData802_11* M802,
    int interfaceIndex)
{
    char buf[MAX_STRING_LENGTH];

    sprintf(buf, "Packets from network = %d",
           M802->pktsToSend);
    IO_PrintStat(node, "MAC", "802.11MAC", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "UNICAST packets sent to channel = %d",
           M802->unicastPacketsSentDcf + M802->unicastPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11MAC", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "BROADCAST packets sent to channel = %d",
           M802->broadcastPacketsSentDcf + M802->broadcastPacketsSentCfp);
    IO_PrintStat(node, "MAC", "802.11MAC", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "UNICAST packets received clearly = %d",
           M802->unicastPacketsGotDcf + M802->unicastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11MAC", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "BROADCAST packets received clearly = %d",
           M802->broadcastPacketsGotDcf + M802->broadcastPacketsGotCfp);
    IO_PrintStat(node, "MAC", "802.11MAC", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Unicasts sent = %d",
           M802->unicastPacketsSentDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Broadcasts sent = %d",
           M802->broadcastPacketsSentDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Unicasts received = %d",
           M802->unicastPacketsGotDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Broadcasts received = %d",
           M802->broadcastPacketsGotDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "CTS packets sent = %d",
           M802->ctsPacketsSentDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "RTS packets sent = %d",
           M802->rtsPacketsSentDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "ACK packets sent = %d",
           M802->ackPacketsSentDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "RTS retransmissions due to timeout = %d",
           M802->retxDueToCtsDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    // This value is captured but not printed. It is rarely non-zero.
    //sprintf(buf, "RTS retransmits aborted = %d",
    //       M802->retxAbortedRtsDcf);
    //IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Packet retransmissions due to ACK timeout = %d",
           M802->retxDueToAckDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

    sprintf(buf, "Packet drops due to retransmission limit = %d",
           M802->pktsDroppedDcf);
    IO_PrintStat(node, "MAC", "802.11DCF", ANY_DEST, interfaceIndex, buf);

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11Finalize.
//  PURPOSE:     Print stats and clear protocol variables.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               int interfaceIndex
//                  interfaceIndex, Interface index
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
void Mac802_11Finalize(
    Node* node,
    int interfaceIndex)
{
    MacData802_11* M802 =
        (MacData802_11*)node->macData[interfaceIndex]->macVar;
    M802_11SeqNoEntry* entry;
    M802_11DataRateEntry* rEntry;

    if (M802->myMacData->macStats == TRUE) {
        Mac802_11PrintStats(node, M802, interfaceIndex);
        Mac802_11CfpPrintStats(node, M802, interfaceIndex);
    }

    // Clear PC related variables
    Mac802_11CfpFreeStorage(node, interfaceIndex);

    // Clear sequence list
    while (M802->seqNoHead != NULL) {
        entry = M802->seqNoHead;
        M802->seqNoHead = entry->next;
        MEM_free(entry);
    }

    // Clear data rate list
    while (M802->dataRateHead != NULL) {
        rEntry = M802->dataRateHead;
        M802->dataRateHead = rEntry->next;
        MEM_free(rEntry);
    }

    // Free 802.11 data structure
    MEM_free(M802);
}




//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcGetTxDataRate
//  PURPOSE:     Determine transmit data rate based on frame type,
//               destination address and ack destination (if ack is due).
//  PARAMETERS:  Node* node
//                  Pointer to node
//               const MacData802_11 * const M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address of frame
//               M802_11MacFrameType frameToSend
//                  Frame type to send
//               int* dataRate
//                  Data rate to use for transmit.
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcGetTxDataRate(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr,
    M802_11MacFrameType frameToSend,
    int* dataRateType)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11DataRateEntry* dataRateEntry;

    *dataRateType = M802->lowestDataRateType;

    if (frameToSend == M802_11_CF_NONE) {
        return;
    }

    switch (frameToSend) {

        case M802_11_DATA:
        case M802_11_CF_DATA_POLL:
            *dataRateType = M802->dataRateInfo->dataRateType;
            break;

        case M802_11_CF_POLL:
            dataRateEntry =
                Mac802_11GetDataRateEntry(node, M802, destAddr);
            *dataRateType = dataRateEntry->dataRateType;
            break;

        case M802_11_CF_END:
            *dataRateType = M802->highestDataRateTypeForBC;
            break;

        case M802_11_CF_NULLDATA:
        case M802_11_CF_ACK:
        default:
            ERROR_ReportError("Mac802_11CfpPcSetDataRateForFrameType: "
                "Unknown or unexpected frame type.\n");
            break;
    }

    if (M802->cfpAckIsDue) {
        // It may occur that the destination for Ack is
        // also the destination for the current frame and
        // the current data rate is higher.
        // A conservative approach is taken here.
        *dataRateType = MIN(*dataRateType, pc->dataRateTypeForAck);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpResetBackoffAndAckVariables
//  PURPOSE:     (Re)set some DCF variables during transit between
//               DCF and PCF.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpResetBackoffAndAckVariables(
    Node* node,
    MacData802_11* M802)
{
    M802->BO = 0;
    Mac802_11ResetCW(node, M802);
    Mac802_11ResetAckVariables(M802);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpSetEndVariables
//  PURPOSE:     Set states and variables when ending CFP.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpSetEndVariables(
    Node* node,
    MacData802_11* M802)
{
    Mac802_11CancelTimer(node, M802);
    M802->cfpEndSequenceNumber = 0;

    // The variables were set when beacon was received.
    // Repeated here in case beacon was missed.
    Mac802_11CfpResetBackoffAndAckVariables(node, M802);

    Mac802_11SetState(node, M802, M802_11_S_IDLE);
    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcHasSufficientTime
//  PURPOSE:     Determine if PC has sufficient time to send a frame type.
//               The frame type determines the frame exchange sequence
//               that would occur. If time was insufficient, the PC would
//               attempt to send a CF-End to terminate the CFP.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame types
//               clocktype sendDelay
//                  Delay before sending the frame
//               NodeAddress destAddr
//                  Destination address of frame type
//  RETURN:      TRUE if there is sufficient time in CFP to send the frame
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpPcHasSufficientTime(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype sendDelay,
    NodeAddress destAddr,
    clocktype cfpBalanceDuration,
    clocktype cfEndTime,
    int dataRateType)
{
    clocktype cfpRequiredDuration = 0;
    clocktype currentMessageDuration;
    int dataRateTypeForResponse;

    dataRateTypeForResponse = dataRateType;

    switch (frameType) {
        case M802_11_DATA:
        case M802_11_CF_DATA_POLL:
            currentMessageDuration =
                PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    MESSAGE_ReturnPacketSize(M802->currentMessage) +
                    M802_11_DATA_FRAME_HDR_SIZE);

            if (destAddr == ANY_DEST) {
                // Broadcast needs sendDelay + msg time + SIFS + CfEnd
                cfpRequiredDuration =
                    sendDelay
                    + currentMessageDuration
                    + M802->extraPropDelay
                    + M802->sifs
                    + cfEndTime;
            }
            else {
                // Unicast needs
                //  sendDelay + msg time + SIFS + ack + SIFS + CfEnd
                cfpRequiredDuration =
                    sendDelay
                    + currentMessageDuration
                    + M802->extraPropDelay
                    + M802->sifs
                    + PHY_GetTransmissionDuration(
                        node, M802->myMacData->phyNumber,
                        dataRateTypeForResponse,
                        M802_11_DATA_FRAME_HDR_SIZE)
                    + M802->extraPropDelay
                    + M802->sifs
                    + cfEndTime;
            }
            break;

        case M802_11_CF_POLL:
            // PC poll needs
            //  sendDelay + Poll + SIFS + nullData + SIFS + CfEnd
            cfpRequiredDuration =
                sendDelay
                + PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    M802_11_DATA_FRAME_HDR_SIZE)
                + M802->extraPropDelay
                + M802->sifs
                + PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateTypeForResponse,
                    M802_11_DATA_FRAME_HDR_SIZE)
                + M802->extraPropDelay
                + M802->sifs
                + cfEndTime;
            break;

        case M802_11_CF_END:
            cfpRequiredDuration =
                sendDelay
                + cfEndTime;
            break;

        default:
            ERROR_ReportError("Mac802_11CfpPcHasSufficientTime: "
                "Unknown frame type.\n");
            break;
    } //switch

    return cfpRequiredDuration <= cfpBalanceDuration;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationHasSufficientTime
//  PURPOSE:     Determine if station has sufficient time to send frame type
//               The frame type determines the frame exchange sequence
//               that would occur. If time was insufficient, the station
//               would not send data bu would send an ack/nulldata.
//               send a CF-End to terminate the CFP.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame types
//               clocktype sendDelay
//                  Delay before sending the frame
//               int* dataRateType
//                  Data rate to use for transmit.
//  RETURN:      TRUE if there is sufficient time in CFP to send the frame
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpStationHasSufficientTime(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype sendDelay,
    int dataRateType)
{
    clocktype cfpBalanceDuration;
    clocktype cfpRequiredDuration = 0;
    clocktype currentMessageDuration;
    clocktype currentTime = getSimTime(node);
    clocktype cfEndTime;

    // Potential time for CF-End frame
    cfEndTime =
        PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            M802->lowestDataRateType,
            M802_11_LONG_CTRL_FRAME_SIZE)
        + M802->extraPropDelay
        + M802->slotTime;

    cfpBalanceDuration = M802->cfpEndTime - currentTime;

    switch (frameType) {
        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
            currentMessageDuration =
                PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    MESSAGE_ReturnPacketSize(M802->currentMessage) +
                    M802_11_DATA_FRAME_HDR_SIZE);

            // Station data needs sendDelay + msg time + SIFS + CfEnd
            cfpRequiredDuration =
                sendDelay
                + currentMessageDuration
                + M802->extraPropDelay
                + M802->sifs
                + cfEndTime;
            break;

        case M802_11_CF_NULLDATA:
        case M802_11_CF_ACK:
            // Need sendDelay + ack + SIF + CfEnd
            cfpRequiredDuration =
                sendDelay
                + PHY_GetTransmissionDuration(
                    node, M802->myMacData->phyNumber,
                    dataRateType,
                    M802_11_DATA_FRAME_HDR_SIZE)
                + M802->extraPropDelay
                + M802->sifs
                + cfEndTime;
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationHasSufficientTime: "
                "Unknown frame type.\n");
            break;

    } //switch

    return cfpRequiredDuration <= cfpBalanceDuration;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcUpdateForPollSentOrSkipped
//  PURPOSE:     Update tracking values when a poll is sent or skipped.
//               For poll save by-count mode, update counters.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollListItem *itemToUpdate
//                  Item to be updated
//               BOOL pollWasSkipped
//                  Was polling skipped, FALSE for transmit poll
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcUpdateForPollSentOrSkipped(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollListItem *itemToUpdate,
    BOOL pollWasSkipped)
{
    M802_11_PcVars* pc = M802->pcVars;

    ERROR_Assert(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc),
        "Mac802_11CfpPcUpdateForPollSentOrSkipped: "
        "Poll list not applicable to PC configuration.\n");

    // In poll+deliver mode, the itemToUpdate is null during
    // the deliver phase (once poll round is complete)
    if (itemToUpdate == NULL) {
        return;
    }

    // Check if this is the first poll of the CFP
    if (pc->firstPolledItem == NULL) {
        pc->firstPolledItem = itemToUpdate;
    }

    pc->prevPolledItem = itemToUpdate;

    if (pc->pollSaveMode == M802_11_POLL_SAVE_BYCOUNT) {
        itemToUpdate->data->pollCount++;
        if (!pollWasSkipped) {
            // Poll count wraps around once it reaches max
            itemToUpdate->data->pollCount = 0;
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcPollCanBeSkipped
//  PURPOSE:     Determine if poll can be skipped in case message
//               in buffer is not for the destination and poll-save
//               mode indicates accordingly.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11_CfPollListItem *itemToCheck
//                  Poll list item which is the destination
//  RETURN:      TRUE if poll can be skipped
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpPcPollCanBeSkipped(
    Node* node,
    MacData802_11* M802,
    M802_11_CfPollListItem *itemToCheck)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollStation *station = itemToCheck->data;

    if (pc->pollSaveMode == M802_11_POLL_SAVE_NONE) {
        // Not if the poll save mode is off.
        return FALSE;
    }
    else if (M802->currentMessage != NULL &&
             M802->currentNextHopAddress == station->macAddr)
    {
        // Not if current message in buffer is for the station
        return FALSE;
    }

    if (pc->pollSaveMode == M802_11_POLL_SAVE_BYCOUNT) {
        // Not if below min threshold or skip cylce has been reached
        if (station->nullDataCount <= pc->pollSaveMinCount
            || station->pollCount == station->nullDataCount)
        {
            return FALSE;
        }
        else {
            //char cancelComment[MAX_STRING_LENGTH];
            //sprintf(cancelComment, "CF-Poll cancelled "
            //    "for [%02X.%02X.%02X.%02X]",
            //    (station->macAddr & 0xff000000) >> 24,
            //    (station->macAddr & 0xff0000) >> 16,
            //    (station->macAddr & 0xff00) >> 8,
            //    (station->macAddr & 0xff));
            //Mac802_11Trace(node, M802, NULL, cancelComment);

            M802->pollsSavedCfp++;
            return TRUE;
        }
    }
    else {
        // for future poll save options
        return FALSE;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcGetNextItemToPoll
//  PURPOSE:     Determine the next item to poll from the poll list
//               considering the previously polled station, the
//               pollable status, the poll save mode and whether a
//               cycle of poll has completed.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      NULL if there are no pollable items
//                  NULL if the list has been cycled
//                  List item if other parameters permit polling
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
M802_11_CfPollListItem * Mac802_11CfpPcGetNextItemToPoll(
    Node* node,
    MacData802_11* M802)
{
    M802_11_PcVars* pc = M802->pcVars;
    M802_11_CfPollListItem* next;
    M802_11_CfPollListItem* itemToPoll = NULL;

    ERROR_Assert(Mac802_11IsPC(M802) && Mac802_11CfpPcSendsPolls(pc),
        "Mac802_11CfpPcGetNextItemToPoll: "
        "Poll list not applicable to PC configuration.\n");

    if (pc->pollableCount > 0 && !pc->allStationsPolled) {
        next = Mac802_11CfPollListGetNextItem(node, M802,
            pc->prevPolledItem);

        while (next != pc->firstPolledItem) {
            if (next->data->pollType == M802_11_STA_POLLABLE_POLL) {
                if (Mac802_11CfpPcPollCanBeSkipped(node, M802, next)) {
                    // TRUE parameter indicates skipped poll
                    Mac802_11CfpPcUpdateForPollSentOrSkipped(
                        node, M802, next, TRUE);
                }
                else {
                    itemToPoll = next;
                    break;
                }
            }
            next = Mac802_11CfPollListGetNextItem(node, M802, next);
        } //while

        if (next == pc->firstPolledItem) {
            pc->allStationsPolled = TRUE;
            itemToPoll = NULL;
        }
    }

    return itemToPoll;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpCheckForOutgoingPacket.
//  PURPOSE:     Check if a packet needs to be dequeued.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
static
void Mac802_11CfpCheckForOutgoingPacket(
    Node* node,
    MacData802_11* M802)
{
    if (M802->currentMessage == NULL) {
        Mac802_11MoveAPacketFromTheNetworkLayerToTheLocalBuffer(node, M802);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitEnd
//  PURPOSE:     Transmit a CF-End(Ack) frame marking end of CF period
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type to send
//               clocktype delay
//                  Send delay
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitEnd(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype delay)
{
    M802_11LongControlFrame* lHdr;
    Message *pktToPhy;

    ERROR_Assert(Mac802_11IsPC(M802),
        "Mac802_11TransmitCfEnd: Not a Point Coordinator.\n");

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node,
                        pktToPhy,
                        M802_11_LONG_CTRL_FRAME_SIZE,
                        TRACE_802_11);

    lHdr = (M802_11LongControlFrame*) MESSAGE_ReturnPacket(pktToPhy);

    lHdr->frameType = (unsigned char) frameType;
    lHdr->destAddr = ANY_DEST;
    lHdr->sourceAddr = M802->selfAddr;
    lHdr->duration = 0;

    Mac802_11CfpPcSetTransmitState(M802, frameType, ANY_DEST);

    Mac802_11StartTransmittingPacket(node, M802, pktToPhy, delay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcEndTransmitted
//  PURPOSE:     Transit to DCF state once the CF End is transmitted
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcEndTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->endPacketsSentCfp++;

    Mac802_11CfpSetEndVariables(node, M802);

    Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitPoll
//  PURPOSE:     Transmit a CF Poll (may be with Ack)
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type to send
//               NodeAddress destAddr
//                  Destination address of frame to be sent
//               clocktype delay
//                  Send delay
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitPoll(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    NodeAddress destAddr,
    clocktype delay)
{
    Message* pktToPhy;
    M802_11FrameHdr* fHdr;

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);
    MESSAGE_PacketAlloc(node, pktToPhy,
        M802_11_DATA_FRAME_HDR_SIZE, TRACE_802_11);
    fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(pktToPhy);

    Mac802_11SetFieldsInDataFrameHdr(M802, fHdr, destAddr, frameType);
    fHdr->address3 = M802->bssAddr;
    fHdr->duration = M802_11_CF_FRAME_DURATION;

    Mac802_11CfpPcSetTransmitState(M802, frameType, destAddr);

    Mac802_11StartTransmittingPacket(node, M802, pktToPhy, delay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcPollTransmitted
//  PURPOSE:     Handle states and timers once a Cf Poll
//               has been transmitted.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcPollTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->pollPacketsSentCfp++;

    Mac802_11CfpPcSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->cfpTimeout);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitBroadcast
//  PURPOSE:     Transmit a broadcast frame during CFP.
//               An ack may piggyback on the broadcast.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type to send
//               clocktype delay
//                  Send delay
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitBroadcast(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype delay)
{
    Message* dequeuedPacket;
    M802_11FrameHdr* fHdr;

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11CfpPcTransmitBroadcast: Current message is NULL.\n");

    dequeuedPacket = M802->currentMessage;

    MESSAGE_AddHeader(node,
                      dequeuedPacket,
                      M802_11_DATA_FRAME_HDR_SIZE,
                      TRACE_802_11);

    fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(dequeuedPacket);

    Mac802_11SetFieldsInDataFrameHdr(M802, fHdr, ANY_DEST, frameType);
    fHdr->duration = M802_11_CF_FRAME_DURATION;

    Mac802_11CfpPcSetTransmitState(M802, frameType, ANY_DEST);

    Mac802_11StartTransmittingPacket(node, M802, dequeuedPacket, delay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcBroadcastTransmitted
//  PURPOSE:     Clear message variables, set an SIFS timer after
//               transmit of broadcast.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcBroadcastTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->broadcastPacketsSentCfp++;

    Mac802_11ResetCurrentMessageVariables(M802);

    Mac802_11CfpPcSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->txSifs);
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitAck
//  PURPOSE:     Transmit an ack in case PC receives unicast data
//               from another BSS which is also in CFP.
//               This is an exception scenario.
//
//               Note that ACK is not CF-Ack.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitAck(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr)
{
    Message* pktToPhy;
    M802_11ShortControlFrame sHdr;

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node, pktToPhy,
        M802_11_SHORT_CTRL_FRAME_SIZE, TRACE_802_11);

    sHdr.frameType = M802_11_ACK;
    sHdr.destAddr = destAddr;
    sHdr.duration = 0;

    memcpy(MESSAGE_ReturnPacket(pktToPhy), &(sHdr),
           M802_11_SHORT_CTRL_FRAME_SIZE);

    Mac802_11CfpPcSetTransmitState(M802, M802_11_ACK, destAddr);

    Mac802_11StartTransmittingPacket(node, M802, pktToPhy, M802->sifs);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcAckTransmitted
//  PURPOSE:     Continue in PCF by setting an SIFS timer
//               after transmit of an ACK.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcAckTransmitted(
    Node* node,
    MacData802_11* M802)
{
    //M802->ackPacketsSentDcf++;

    Mac802_11CfpPcSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->txSifs);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitUnicast
//  PURPOSE:     Transmit a unicast frame during CFP.
//               An ack and/or a poll may piggyback on the data frame.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type
//               NodeAddress destAddr
//                  Destination address
//               clocktype delay
//                  Delay
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitUnicast(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    NodeAddress destAddr,
    clocktype delay)
{
    Message* pktToPhy;
    M802_11FrameHdr* fHdr;
    M802_11SeqNoEntry *entry;

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11CfpPcTransmitUnicast: Current message is NULL.\n");

    pktToPhy = MESSAGE_Duplicate(node, M802->currentMessage);

    MESSAGE_AddHeader(node,
                      pktToPhy,
                      M802_11_DATA_FRAME_HDR_SIZE,
                      TRACE_802_11);

    fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(pktToPhy);
    Mac802_11SetFieldsInDataFrameHdr(M802, fHdr, destAddr, frameType);
    fHdr->duration =  M802_11_CF_FRAME_DURATION;

    entry = Mac802_11GetSeqNo(node, M802, destAddr);
    ERROR_Assert(entry,
        "Mac802_11CfpPcTransmitUnicast: "
        "Unable to find or create sequence number entry.\n");
    fHdr->seqNo = entry->toSeqNo;

    M802->waitingForAckOrCtsFromAddress = destAddr;

    if (MESSAGE_ReturnPacketSize(M802->currentMessage) >
        M802_11_FRAG_THRESH)
    {
        // Fragmentation not currently supported
        ERROR_ReportError("Mac802_11CfpPcTransmitUnicast: "
            "Fragmentation not supported.\n");
    }
    else {
        fHdr->fragId = 0;
        Mac802_11CfpPcSetTransmitState(M802, frameType, destAddr);

        Mac802_11StartTransmittingPacket(node, M802, pktToPhy, delay);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcUnicastTransmitted
//  PURPOSE:     Set states and timers once a unicast has been transmitted.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcUnicastTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->dataPacketsSentCfp++;

    Mac802_11CfpPcSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->cfpTimeout);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcUnicastPollTransmitted
//  PURPOSE:     Handle states and timers once a unicast+poll
//               has been transmitted.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcUnicastPollTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->dataPollPacketsSentCfp++;

    Mac802_11CfpPcSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->cfpTimeout);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcAdjustFrameTypeForTime
//  PURPOSE:     Adjust frame type intended to be sent for balance CFP.
//               In case time is insufficient, possible transitions are
//                  data+poll   ->  poll
//                  data        ->  end
//                  poll        ->  end
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type
//               NodeAddress destAddr
//                  Destination address
//               clocktype delay
//                  Delay before transmit
//               int dataRateType
//                  Data rate at which to transmit
//  RETURN:      Modified frame type in case time is insufficient.
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcAdjustFrameTypeForTime(
    Node* node,
    MacData802_11* M802,
    NodeAddress* destAddr,
    M802_11MacFrameType* frameToSend,
    clocktype sendDelay,
    int* dataRateType)
{
    clocktype cfpBalanceDuration;
    clocktype cfEndTime;

    if (*frameToSend == M802_11_CF_NONE) {
        return;
    }

    // Potential time for CF-End frame
    cfEndTime =
        PHY_GetTransmissionDuration(
            node, M802->myMacData->phyNumber,
            M802->lowestDataRateType,
            M802_11_LONG_CTRL_FRAME_SIZE)
        + M802->extraPropDelay
        + M802->slotTime;

    // After sending a beacon, the PC beacon time indicates the
    // time for the next beacon.
    cfpBalanceDuration =
        M802->cfpBeaconTime
        - M802->cfpBeaconInterval
        + M802->cfpMaxDuration
        - getSimTime(node);

    if (! Mac802_11CfpPcHasSufficientTime(node, M802,
            *frameToSend, sendDelay, *destAddr,
            cfpBalanceDuration, cfEndTime, *dataRateType))
    {
        switch (*frameToSend) {
            case M802_11_CF_DATA_POLL:
                *frameToSend = M802_11_CF_POLL;
                if (! Mac802_11CfpPcHasSufficientTime(node, M802,
                        *frameToSend, sendDelay, *destAddr,
                        cfpBalanceDuration, cfEndTime, *dataRateType))
                {
                    *frameToSend = M802_11_CF_END;
                    *destAddr = ANY_DEST;
                }
                break;

            case M802_11_CF_POLL:
            case M802_11_DATA:
                *frameToSend = M802_11_CF_END;
                *destAddr = ANY_DEST;
                break;

            case M802_11_CF_END:
                //do nothing
                break;

            default:
                ERROR_ReportError("Mac802_11CfpPcAdjustFrameTypeForTime: "
                    "Unexpected frame type.\n");
                break;
        } //switch

        if (*frameToSend == M802_11_CF_END) {
            *dataRateType =
                MIN(*dataRateType, M802->highestDataRateTypeForBC);

            if (! Mac802_11CfpPcHasSufficientTime(node, M802,
                    *frameToSend, sendDelay, *destAddr,
                    cfpBalanceDuration, cfEndTime, *dataRateType))
            {
                *frameToSend = M802_11_CF_NONE;
            }
        }
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcAdjustFrameTypeForData
//  PURPOSE:     Adjust frame type intended to be sent for data in buffer.
//               In case there is no data, possible transitions are
//                  data+poll   ->  poll
//                  data        ->  none
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//               M802_11MacFrameType* frameToSend
//                  Frame to be sent
//  RETURN:      Modified frame type in case there is no data in buffer.
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcAdjustFrameTypeForData(
    Node* node,
    MacData802_11* M802,
    NodeAddress destAddr,
    M802_11MacFrameType* frameToSend)
{
    switch (*frameToSend) {
        case M802_11_CF_DATA_POLL:
            if (M802->currentMessage == NULL
                || (M802->currentNextHopAddress != destAddr))
            {
                *frameToSend = M802_11_CF_POLL;
            }
            break;

        case M802_11_DATA:
            if (M802->currentMessage == NULL) {
                *frameToSend = M802_11_CF_NONE;
            }
            break;

        default:
            ERROR_ReportError("Mac802_11CfpPcAdjustFrameTypeForData: "
                "Unexpected frame type.\n");
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitDeliverOnly
//  PURPOSE:     Determine the next frame to transmit when PC
//               is in DELIVER-ONLY mode.
//               Fills up frame type to send and the destination address.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType* frameToSend
//                  Frame type to be sent
//               NodeAddress destAddr
//                  Destination address
//               clocktype sendDelay
//                  Delay to be sent
//  RETURN:      Filled up destination address
//               Filled up frame type to send
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitDeliverOnly(
    Node* node,
    MacData802_11* M802,
    NodeAddress* destAddr,
    M802_11MacFrameType* frameToSend,
    clocktype sendDelay)
{
    M802_11_PcVars* pc = M802->pcVars;

    ERROR_Assert(pc->deliveryMode == M802_11_PC_DELIVER_ONLY,
        "Mac802_11CfpPcTransmitDeliverOnly: "
        "PC is not in DELIVER-ONLY mode.\n");

    // In this mode, there is no polling list to consider
    *frameToSend = M802_11_DATA;
    *destAddr = M802->currentNextHopAddress;

    Mac802_11CfpPcAdjustFrameTypeForData(node, M802,
        *destAddr, frameToSend);

    if (*frameToSend == M802_11_CF_NONE) {
        *frameToSend = M802_11_CF_END;
        *destAddr = ANY_DEST;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitPollAndDeliver
//  PURPOSE:     Determine the next frame to transmit when
//               PC is in POLL-AND-DELIVER mode.
//               Fills up frame type to send and destination address.
//
//               Gives priority to broadcast packets over polls.
//
//               This is a minimal algorith which can benefits from:
//               -  deeper look into network queue so that data
//                  and polls combine as far as possible.
//               -  poll ageing
//                  e.g. poll saving by count (this is implemented)
//               -  use of more data flag so that stations may be
//                  polled multipe time in a CFP.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//               M802_11MacFrameType* frameToSend
//                  Frame type
//               clocktype sendDelay
//                  Delay
//               M802_11_CfPollListItem **stationItem
//                  Polling list item
//  RETURN:      Filled up destination address
//               Filled up frame type to send
//               Filled up polling list item (may be NULL in case of
//                  broadcasts or when poll cycle is complete)
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitPollAndDeliver(
    Node* node,
    MacData802_11* M802,
    NodeAddress *destAddr,
    M802_11MacFrameType *frameToSend,
    clocktype sendDelay,
    M802_11_CfPollListItem **stationItem)
{
    M802_11_PcVars* pc = M802->pcVars;

    ERROR_Assert(pc->deliveryMode == M802_11_PC_POLL_AND_DELIVER,
        "Mac802_11CfpPcTransmitDeliverAndPoll: "
        "PC is not in POLL-AND-DELIVER mode.\n");

    // Send a broadcast if it has priority
    if (M802->currentMessage != NULL
        && pc->broadcastsHavePriority
        && M802->currentNextHopAddress == ANY_DEST)
    {
        *frameToSend = M802_11_DATA;
        *destAddr = ANY_DEST;
        *stationItem = NULL;
        return;
    }

    *stationItem = Mac802_11CfpPcGetNextItemToPoll(node, M802);

    if (*stationItem == NULL) {
        // All station polled at least once.
        // Operate in Deliver Only mode.
        //
        // Modifications related to more data flag can be made here.

        *frameToSend = M802_11_DATA;
        *destAddr = M802->currentNextHopAddress;

        Mac802_11CfpPcAdjustFrameTypeForData(node, M802,
            *destAddr, frameToSend);

        if (*frameToSend != M802_11_CF_NONE) {
            // Piggyback a poll if station type permits
            if (*frameToSend == M802_11_DATA
                && Mac802_11CfPollListIsAddressPollable(
                        node, M802, *destAddr))
            {
                *frameToSend = M802_11_CF_DATA_POLL;
            }
        }
        else {
            *frameToSend = M802_11_CF_END;
            *destAddr = ANY_DEST;
        }
    }
    else {
        // There is a pollable station unpolled this CFP
        *destAddr = (*stationItem)->data->macAddr;
        *frameToSend = M802_11_CF_DATA_POLL;

        Mac802_11CfpPcAdjustFrameTypeForData(node, M802,
            *destAddr, frameToSend);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitPollOnly
//  PURPOSE:     Determine the next frame to transmit when
//               PC is in POLL-ONLY mode.
//               Fills up frame type to send and destination address
//               Gives priority to broadcast packets over polls.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress destAddr
//                  Destination address
//               M802_11MacFrameType* frameToSend
//                  Frame type
//               clocktype sendDelay
//                  Delay
//               M802_11_CfPollListItem **stationItem
//                  Polling list item
//  RETURN:      Filled up destination address
//               Filled up frame type to send
//               Filled up polling list item (may be NULL in case of
//                  broadcasts or when poll cycle is complete)
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitPollOnly(
    Node* node,
    MacData802_11* M802,
    NodeAddress* destAddr,
    M802_11MacFrameType* frameToSend,
    clocktype sendDelay,
    M802_11_CfPollListItem** stationItem)
{
    M802_11_PcVars* pc = M802->pcVars;

    ERROR_Assert(pc->deliveryMode == M802_11_PC_POLL_ONLY,
        "Mac802_11CfpPcTransmitPollOnly: PC is not in POLL-ONLY mode.\n");

    if (M802->currentMessage != NULL
        && pc->broadcastsHavePriority
        && M802->currentNextHopAddress == ANY_DEST)
    {
        *frameToSend = M802_11_DATA;
        *destAddr = ANY_DEST;
        *stationItem = NULL;
        return;
    }

    *stationItem = Mac802_11CfpPcGetNextItemToPoll(node, M802);

    if (*stationItem != NULL) {
        *destAddr = (*stationItem)->data->macAddr;
        *frameToSend = M802_11_CF_DATA_POLL;

        Mac802_11CfpPcAdjustFrameTypeForData(node, M802,
            *destAddr, frameToSend);
    }
    else {
        *frameToSend = M802_11_CF_END;
        *destAddr = ANY_DEST;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmit
//  PURPOSE:     Governing routine for PC transmits during CFP.
//               Determines the send delay to use and calls the
//               configured delivery algorithm to determine what
//               is to be sent.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType receivedFrameType
//                  Received frame, none in case the previous frame
//                  sent by the PC was a beacon, a broadcast, or
//                  there was a timeout
//               clocktype delay
//                  Timer delay, 0 if not applicable.
//  PARAMETERS:
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmit(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType receivedFrameType,
    clocktype timerDelay)
{
    M802_11_PcVars* pc = M802->pcVars;
    clocktype sendDelay;
    M802_11MacFrameType frameType;
    NodeAddress destAddr = 0;
    M802_11_CfPollListItem* stationItem = NULL;
    int dataRateType;

    if (Mac802_11PhyStatus(node, M802) != PHY_IDLE) {
        Mac802_11Trace(node, M802, NULL, "CFP PC transmit, phy not idle.");

        return;
    }

    // In case of timeout, at a PC, the timer delay is PIFS.
    // In that case, a token delay of 1 is set for transmit.
    if (timerDelay >= M802->sifs) {
        sendDelay = EPSILON_DELAY;
    }
    else {
        sendDelay = M802->sifs - timerDelay;
    }

    switch (pc->deliveryMode) {

        case M802_11_PC_DELIVER_ONLY:
            switch (M802->cfpState) {
                case M802_11_S_CFP_NONE:
                    Mac802_11CfpPcTransmitDeliverOnly(node, M802,
                        &destAddr, &frameType, sendDelay);
                    break;

                case M802_11_S_CFP_WF_ACK:
                    // Since the Mac buffer can hold one packet, options are
                    // - attempt a retransmit
                    // - or drop packet and continue delivery
                    // - or end CFP (current implementation)
                    frameType = M802_11_CF_END;
                    destAddr = ANY_DEST;
                    break;

                default:
                    ERROR_ReportError("Mac802_11CfpPcTransmit: "
                        "Unknown CFP state for DELIVER-ONLY mode.\n");
                    break;
            } //switch
            break;

        case M802_11_PC_POLL_ONLY:
            switch (M802->cfpState) {
                case M802_11_S_CFP_NONE:
                    Mac802_11CfpPcTransmitPollOnly(node, M802,
                        &destAddr, &frameType, sendDelay, &stationItem);
                    break;

                case M802_11_S_CFP_WF_DATA:
                case M802_11_S_CFP_WF_DATA_ACK:
                    Mac802_11CfPollListUpdateForNullDataResponse(node,
                        M802, pc->prevPolledItem->data->macAddr);

                    // Continue polling
                    Mac802_11CfpPcTransmitPollOnly(node, M802,
                        &destAddr, &frameType, sendDelay, &stationItem);
                    break;

                default:
                    ERROR_ReportError("Mac802_11CfpPcTransmit: "
                        "Unknown CFP state for POLL-ONLY mode.\n");
                    break;
            } //switch
            break;

        case M802_11_PC_POLL_AND_DELIVER:
            switch (M802->cfpState) {
                case M802_11_S_CFP_NONE:
                    Mac802_11CfpPcTransmitPollAndDeliver(node, M802,
                        &destAddr, &frameType, sendDelay, &stationItem);
                    break;

                case M802_11_S_CFP_WF_ACK:
                    // Stop operating in delivery mode
                    frameType = M802_11_CF_END;
                    destAddr = ANY_DEST;
                    break;

                case M802_11_S_CFP_WF_DATA:
                case M802_11_S_CFP_WF_DATA_ACK:
                    if (!pc->allStationsPolled) {
                        Mac802_11CfPollListUpdateForNullDataResponse(node,
                            M802, pc->prevPolledItem->data->macAddr);

                        // Continue polling
                        Mac802_11CfpPcTransmitPollAndDeliver(node, M802,
                            &destAddr, &frameType, sendDelay, &stationItem);
                    }
                    else {
                        // In delivery mode, the packet in buffer timed out.
                        // End PCF.
                        frameType = M802_11_CF_END;
                        destAddr = ANY_DEST;
                    }
                    break;
                default:
                    ERROR_ReportError("Mac802_11CfpPcTransmit: "
                        "Unknown CFP state for POLL-AND-DELIVER mode.\n");
                    break;
            } //switch

            break;
        default:
            ERROR_ReportError("Mac802_11CfpPcTransmit: "
                "Unknown PC delivery mode.\n");
            break;
    } //switch

    Mac802_11CfpPcGetTxDataRate(
        node, M802, destAddr, frameType, &dataRateType);

    Mac802_11CfpPcAdjustFrameTypeForTime(node, M802,
        &destAddr, &frameType, sendDelay, &dataRateType);

    Mac802_11CfpAdjustFrameTypeForAck(M802, &frameType);

    PHY_SetTxDataRateType(
        node,
        M802->myMacData->phyNumber,
        dataRateType);


    switch (frameType) {

        case M802_11_CF_DATA_POLL:
        case M802_11_CF_DATA_POLL_ACK:
            Mac802_11CfpPcUpdateForPollSentOrSkipped(
                node, M802, stationItem, FALSE);

            Mac802_11CfpPcTransmitUnicast(node, M802,
                frameType, destAddr, sendDelay);
            break;

        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
            if (destAddr == ANY_DEST) {
                Mac802_11CfpPcTransmitBroadcast(node, M802,
                    frameType, sendDelay);
            }
            else {
                Mac802_11CfpPcTransmitUnicast(node, M802,
                    frameType, destAddr, sendDelay);
            }
            break;

        case M802_11_CF_POLL:
        case M802_11_CF_POLL_ACK:
            Mac802_11CfpPcUpdateForPollSentOrSkipped(
                node, M802, stationItem, FALSE);

            Mac802_11CfpPcTransmitPoll(node, M802,
                frameType, destAddr, sendDelay);
            break;

        case M802_11_CF_END:
        case M802_11_CF_END_ACK:
            Mac802_11CfpPcTransmitEnd(node, M802,
                frameType, sendDelay);
            break;

        case M802_11_CF_NONE:
            break;

        default:
            ERROR_ReportError("Mac802_11CfpPcTransmit: "
                "Unknown frame type.\n");
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcProcessAck
//  PURPOSE:     Process the received ack.
//               The ack could piggybacks on a data+ack
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Source address od Ack
//               BOOL isPureAck
//                  TRUE if it is a CF-Ack frame
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcProcessAck(
    Node* node,
    MacData802_11* M802,
    NodeAddress sourceAddr,
    BOOL isPureAck)
{
    if (M802->waitingForAckOrCtsFromAddress == INVALID_ADDRESS
        || M802->waitingForAckOrCtsFromAddress != sourceAddr)
    {
        return;
    }

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11CfpPcProcessAck: "
        "Processing CF Ack without a message in buffer.\n");

    M802->dataRateInfo->numAcksInSuccess++;

    Mac802_11UpdateForSuccessfullyReceivedAck(node, M802, sourceAddr);

    M802->unicastPacketsSentCfp++;

// ANTS01
	Mac802_11UpdateAvgSendTimeANTS01(node,M802);
	Mac802_11NotifySendReceiveANTS01(node,sourceAddr);
// ANTS01

    Mac802_11CancelTimer(node, M802);

    if (isPureAck) {
        //M802->ackPacketsGotCfp++;
        Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
    }

    Mac802_11CfpCheckForOutgoingPacket(node, M802);

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcProcessUnicast
//  PURPOSE:     Process a unicast data frame.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      TRUE if frame is acceptable
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpPcProcessUnicast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11_PcVars* pc = M802->pcVars;
    BOOL unicastIsProcessed = FALSE;
    M802_11FrameHdr *fHdr =
        (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = fHdr->sourceAddr;

    if (Mac802_11CorrectSeqenceNumber(node, M802,
            sourceAddr, fHdr->seqNo))
    {
        Mac802_11Trace(node, M802, msg, "Receive");

        M802->unicastPacketsGotCfp++;

        pc->dataRateTypeForAck =
            PHY_GetRxDataRateType(node, M802->myMacData->phyNumber);

        Mac802_11CfPollListUpdateForUnicastReceived(
            node, M802, sourceAddr);

        Mac802_11HandOffSuccessfullyReceivedUnicast(node, M802, msg);

        unicastIsProcessed = TRUE;
    }
    else {
        Mac802_11Trace(node, M802, NULL, "Drop, wrong sequence number");

        MESSAGE_Free(node, msg);

        unicastIsProcessed = FALSE;
    }

    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
    M802->cfpAckIsDue = TRUE;

    return unicastIsProcessed;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcReceiveUnicast
//  PURPOSE:     Process a received unicast frame.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcReceiveUnicast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11FrameHdr *fHdr =
        (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);
    M802_11MacFrameType frameType = (M802_11MacFrameType) fHdr->frameType;
    NodeAddress sourceAddr = fHdr->sourceAddr;
    NodeAddress destAddr = fHdr->destAddr;

    switch (frameType) {
        case M802_11_CF_DATA_ACK:
            Mac802_11CfpPcProcessAck(node, M802, sourceAddr, FALSE);

            // no break; continue to process DATA

        case M802_11_DATA:
            M802->dataPacketsGotCfp++;

            Mac802_11CfpPcProcessUnicast(node, M802, msg);
            if (Mac802_11IsFrameFromStationInMyBSS(M802,
                    destAddr, fHdr->frameFlags))
            {
                Mac802_11CfpPcTransmit(node, M802, frameType, 0);
            }
            else {
                Mac802_11CfpPcTransmitAck(node, M802, sourceAddr);
            }
            break;

        case M802_11_CF_NULLDATA:
            Mac802_11Trace(node, M802, msg, "Receive");

            M802->nulldataPacketsGotCfp++;

            Mac802_11CfPollListUpdateForNullDataResponse(
                node, M802, sourceAddr);

            Mac802_11CancelTimer(node, M802);
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);

            MESSAGE_Free(node, msg);

            Mac802_11CfpPcTransmit(node, M802, frameType, 0);
            break;

        case M802_11_CF_ACK:
            Mac802_11Trace(node, M802, msg, "Receive");

            Mac802_11CfpPcProcessAck(node, M802, sourceAddr, TRUE);

            MESSAGE_Free(node, msg);

            Mac802_11CfpPcTransmit(node, M802, frameType, 0);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpReceiveUnicastPacketFromPhy: "
                "Received unknown unicast frame type.\n");
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationProcessEnd
//  PURPOSE:     Process a CF-End frame from the PC
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationProcessEnd(
    Node* node,
    MacData802_11* M802)
{
    M802->endPacketsGotCfp++;

    Mac802_11CfpSetEndVariables(node, M802);

    M802->NAV = 0;

    Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationProcessAck
//  PURPOSE:     Station receives an ack as part of CF-End/Data/DataPoll/Poll.
//               Station cannot get a pure ack.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               NodeAddress sourceAddr
//                  Source address of frame containing ack
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationProcessAck(
    Node* node,
    MacData802_11* M802,
    NodeAddress sourceAddr)
{
    if (M802->waitingForAckOrCtsFromAddress == INVALID_ADDRESS
        || M802->waitingForAckOrCtsFromAddress != sourceAddr)
    {
        return;
    }

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11CfpStationProcessAck: "
        "Processing CF Ack without a message in buffer.\n");

    Mac802_11UpdateForSuccessfullyReceivedAck(node, M802, sourceAddr);

    M802->unicastPacketsSentCfp++;

// ANTS01
	Mac802_11UpdateAvgSendTimeANTS01(node,M802);
	Mac802_11NotifySendReceiveANTS01(node,sourceAddr);
// ANTS01

    Mac802_11CancelTimer(node, M802);
    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);

    Mac802_11CfpCheckForOutgoingPacket(node, M802);

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationTransmitAckOrNullData
//  PURPOSE:     Transmit of ack by a BSS station.
//               Also used to transmit a nulldata frame.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type to send
//               clocktype delay
//                  Delay
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationTransmitAckOrNullData(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype delay)
{
    Message* pktToPhy;
    M802_11FrameHdr* fHdr;
    NodeAddress destAddr = M802->bssAddr;

    pktToPhy = MESSAGE_Alloc(node, 0, 0, 0);

    MESSAGE_PacketAlloc(node, pktToPhy,
        M802_11_DATA_FRAME_HDR_SIZE, TRACE_802_11);
    fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(pktToPhy);

    Mac802_11SetFieldsInDataFrameHdr(M802, fHdr, destAddr, frameType);
    fHdr->address3 = M802->bssAddr;
    fHdr->duration = M802_11_CF_FRAME_DURATION;

    Mac802_11CfpStationSetTransmitState(M802, frameType);

    Mac802_11StartTransmittingPacket(node, M802, pktToPhy, delay);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationAckTransmitted
//  PURPOSE:     Set state once an ack transmit is complete.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationAckTransmitted(
    Node* node,
    MacData802_11* M802)
{
    //M802->ackPacketsSentCfp++;

    Mac802_11CfpStationSetPostTransmitState(M802);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationNullDataTransmitted
//  PURPOSE:     Set state once a nulldata transmit is complete.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationNullDataTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->nulldataPacketsSentCfp++;

    Mac802_11CfpStationSetPostTransmitState(M802);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationTransmitUnicast
//  PURPOSE:     Transmit a unicast frame in response to poll.
//               An ack may piggyback on the data frame.
//  PARAMETERS:  Frame type to send
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Frame type
//               clocktype delay
//                  Delay to be sent
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationTransmitUnicast(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType frameType,
    clocktype delay)
{
    Message* pktToPhy;
    M802_11FrameHdr* fHdr;
    M802_11SeqNoEntry* entry;
    NodeAddress destAddr = M802->bssAddr;

    ERROR_Assert(M802->currentMessage != NULL,
        "Mac802_11CfpStationTransmitUnicast: "
        "Transmit of data when message is NULL.\n");

    pktToPhy = MESSAGE_Duplicate(node, M802->currentMessage);

    MESSAGE_AddHeader(node,
                      pktToPhy,
                      M802_11_DATA_FRAME_HDR_SIZE,
                      TRACE_802_11);

    fHdr = (M802_11FrameHdr*) MESSAGE_ReturnPacket(pktToPhy);
    Mac802_11SetFieldsInDataFrameHdr(M802, fHdr, destAddr, frameType);
    fHdr->duration =  M802_11_CF_FRAME_DURATION;

    entry = Mac802_11GetSeqNo(node, M802, destAddr);
    ERROR_Assert(entry,
        "Mac802_11CfpStationTransmitUnicast: "
        "Unable to find or create sequence number entry.\n");
    fHdr->seqNo = entry->toSeqNo;

    M802->waitingForAckOrCtsFromAddress = destAddr;

    if (MESSAGE_ReturnPacketSize(M802->currentMessage) >
        M802_11_FRAG_THRESH)
    {
        // Fragmentation not currently supported
        ERROR_ReportError("Mac802_11CfpStationTransmitUnicast: "
            "Fragmentation not supported.\n");
    }
    else {
        fHdr->fragId = 0;
        Mac802_11CfpStationSetTransmitState(M802, frameType);
        Mac802_11StartTransmittingPacket(node, M802, pktToPhy, delay);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationUnicastTransmitted
//  PURPOSE:     Set states/timers after a station transmits a unicast
//               during CFP.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationUnicastTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802->dataPacketsSentCfp++;

    Mac802_11CfpStationSetPostTransmitState(M802);

    Mac802_11StartTimer(node, M802, M802->cfpTimeout);
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationTransmit
//  PURPOSE:     Determine the frame to transmit based on timer delay,
//               available balance CFP duration and received frame type.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               M802_11MacFrameType frameType
//                  Received frame type
//               clocktype delay
//                  Delay to be sent
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationTransmit(
    Node* node,
    MacData802_11* M802,
    M802_11MacFrameType receivedFrameType,
    clocktype timerDelay)
{
    clocktype sendDelay;
    M802_11MacFrameType frameType;
    int dataRateType;

    if (Mac802_11PhyStatus(node, M802) != PHY_IDLE) {
        Mac802_11Trace(node, M802, NULL,
            "CFP Station transmit, phy not idle.");
        return;
    }

    // The timerDelay should be 0 for a station
    if (timerDelay < M802->sifs) {
        sendDelay = M802->sifs - timerDelay;
    }
    else {
        sendDelay = EPSILON_DELAY;
    }

    dataRateType = PHY_GetRxDataRateType(node, M802->myMacData->phyNumber);

    switch (receivedFrameType) {
        case M802_11_CF_DATA_POLL:
        case M802_11_CF_DATA_POLL_ACK:
            // In response to the poll, send a data frame if possible,
            // else respond with an ACK.
            if (M802->currentMessage == NULL) {
                frameType = M802_11_CF_ACK;
            }
            else {
                frameType = M802_11_DATA;
                if (Mac802_11CfpStationHasSufficientTime(node, M802,
                    frameType, sendDelay, dataRateType))
                {
                    Mac802_11CfpAdjustFrameTypeForAck(M802, &frameType);
                }
                else {
                    frameType = M802_11_CF_ACK;
                }
            }
            break;

        case M802_11_CF_POLL:
        case M802_11_CF_POLL_ACK:
            // Send a data frame if possible
            // else respond with a null data response.
            if (M802->currentMessage == NULL) {
                frameType = M802_11_CF_NULLDATA;
            }
            else {
                frameType = M802_11_DATA;
                if (! Mac802_11CfpStationHasSufficientTime(node, M802,
                    frameType, sendDelay, dataRateType))
                {
                    frameType = M802_11_CF_NULLDATA;
                }
            }
            break;

        case M802_11_DATA:
            // Send a CF-ACK. An option is to send M802_11_ACK.
            frameType = M802_11_CF_ACK;
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationTransmit: "
                "Unexpected received frame type.\n");
            break;
    } //switch


    PHY_SetTxDataRateType(
        node,
        M802->myMacData->phyNumber,
        dataRateType);

    switch (frameType) {
        case M802_11_DATA:
        case M802_11_CF_DATA_ACK:
            Mac802_11CfpStationTransmitUnicast(node, M802,
                frameType, sendDelay);
            break;

        case M802_11_CF_NULLDATA:
        case M802_11_CF_ACK:
            Mac802_11CfpStationTransmitAckOrNullData(node, M802,
                frameType, sendDelay);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationTransmit: "
                "Unknown frame type.\n");
            break;
    } //switch

}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationProcessUnicast
//  PURPOSE:     Receive a directed unicast frame
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      TRUE is unicast is acceptable
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpStationProcessUnicast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    BOOL unicastIsProcessed = FALSE;
    M802_11FrameHdr* fHdr =
        (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = fHdr->sourceAddr;

    if (Mac802_11CorrectSeqenceNumber(node, M802, sourceAddr, fHdr->seqNo)) {
        Mac802_11Trace(node, M802, msg, "Receive");

        M802->unicastPacketsGotCfp++;

        Mac802_11HandOffSuccessfullyReceivedUnicast(node, M802, msg);

        unicastIsProcessed = TRUE;
    }
    else {
        Mac802_11Trace(node, M802, NULL, "Drop, wrong sequence number");

        unicastIsProcessed = FALSE;
        MESSAGE_Free(node, msg);
    }

    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);

    M802->cfpAckIsDue = TRUE;

    return unicastIsProcessed;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcTransmitBeacon
//  PURPOSE:     Send the beacon that signals start of PCF.
//
//               The implementation uses a simplified beacon mechanism:
//               -  a beacon starts a CFP,
//               -  the beacon interval equals CFP interval.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcTransmitBeacon(
    Node* node,
    MacData802_11* M802)
{
    Message* beaconMsg;
    M802_11_BeaconFrame* beacon;
    int cfpBalDuration = 0;
    BOOL canTransmit = FALSE;
    clocktype cfpStartTime;
    clocktype cfpEndTime;
    clocktype currentTime;
    int dataRateType = 0;

    if (Mac802_11PhyStatus(node, M802) != PHY_IDLE
        || Mac802_11WaitForNAV(node, M802))
    {
        Mac802_11Trace(node, M802, NULL,
            "PCF Beacon delayed - not idle or waiting for NAV.");

        // Either wait for timer to expire or for
        //  StartSendTimers to reattempt transmission
        Mac802_11SetState(node, M802, M802_11_S_IDLE);
        return;
    }

    currentTime = getSimTime(node);
    cfpStartTime = currentTime + M802->delayUntilSignalAirborn;
    cfpEndTime = M802->cfpBeaconTime + M802->cfpMaxDuration;

    // Has beacon been delayed for this CFP?
    if (cfpEndTime > cfpStartTime) {
        dataRateType = M802->highestDataRateTypeForBC;

        cfpBalDuration = (int) (cfpEndTime - cfpStartTime);

        // Is there sufficient time to send a beacon + CF-End?
        canTransmit = cfpBalDuration >=
            PHY_GetTransmissionDuration(
                node, M802->myMacData->phyNumber,
                dataRateType,
                sizeof(M802_11_BeaconFrame))
            + M802->extraPropDelay
            + M802->sifs
            + PHY_GetTransmissionDuration(
                node, M802->myMacData->phyNumber,
                dataRateType,
                M802_11_LONG_CTRL_FRAME_SIZE)
            + M802->extraPropDelay
            + M802->slotTime;
    }

    if (canTransmit) {
        Mac802_11CancelTimer(node, M802);

        PHY_SetTxDataRateType(node, M802->myMacData->phyNumber,
            dataRateType);

        beaconMsg = MESSAGE_Alloc(node, 0, 0, 0);
        MESSAGE_PacketAlloc(node, beaconMsg,
            sizeof(M802_11_BeaconFrame), TRACE_802_11);
        beacon = (M802_11_BeaconFrame*)MESSAGE_ReturnPacket(beaconMsg);
        memset(beacon, 0, sizeof(M802_11_BeaconFrame));

        beacon->hdr.frameType = M802_11_BEACON;
        beacon->hdr.frameFlags = 0;
        beacon->hdr.duration = M802_11_CF_FRAME_DURATION;
        beacon->hdr.destAddr = ANY_DEST;
        beacon->hdr.sourceAddr = M802->selfAddr;
        beacon->hdr.address3 = M802->bssAddr;

        beacon->timeStamp = currentTime + M802->delayUntilSignalAirborn;
        beacon->beaconInterval =
            Mac802_11ClocktypeToTUs(M802->cfpBeaconInterval);

        beacon->cfSet.elementId = M802_11_CF_PARAMETER_SET_ID;
        beacon->cfSet.length = M802_11_CF_PARAMETER_SET_LENGTH;
        beacon->cfSet.cfpMaxDuration =
            Mac802_11ClocktypeToTUs(M802->cfpMaxDuration);
        beacon->cfSet.cfpBalDuration = cfpBalDuration;

        Mac802_11SetState(node, M802, M802_11_X_CFP_BEACON);

        Mac802_11StartTransmittingPacket(node, M802, beaconMsg,
            M802->delayUntilSignalAirborn);
    }
    else {
        // Skip the beacon.
        // Set the timer for the next beacon period.
        clocktype nextBeaconTime;

        Mac802_11Trace(node, M802, NULL, "Beacon skipped");

        M802->cfpBeaconIsDue = FALSE;
        nextBeaconTime = M802->cfpBeaconTime + M802->cfpBeaconInterval;

        // In case a beacon was missed altogether
        while (nextBeaconTime <= currentTime)
        {
            nextBeaconTime += M802->cfpBeaconInterval;
        }
        Mac802_11StartTimerOfGivenType(node, M802,
            nextBeaconTime - currentTime, MSG_MAC_802_11_Beacon);
        M802->cfpBeaconSequenceNumber = M802->timerSequenceNumber;
        M802->cfpBeaconTime = nextBeaconTime;

        Mac802_11SetState(node, M802, M802_11_S_IDLE);
        Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcAttemptToTransmitCfBeacon
//  PURPOSE:     Attempt to send a beacon to signal start of CFP.
//               Checks permissible states and sets the beacon timer.
//               Where timer has elapsed, makes a call to transmit beacon.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE if states permit or beacon transmit occurs
//  ASSUMPTION:  Where DIFS has elapsed or PC is in back-off, no
//               additional delay is neccary.
//               The beacon can have precedance when waiting for CTS.
//--------------------------------------------------------------------------
BOOL Mac802_11CfpPcAttemptToTransmitCfBeacon(
    Node* node,
    MacData802_11* M802)
{
    BOOL couldTransmit = FALSE;

    if (Mac802_11PhyStatus(node, M802) != PHY_IDLE
        || Mac802_11IsTransmittingState(M802->state)
        || Mac802_11IsWaitingForResponseState(M802->state))
    {
        return FALSE;
    }

    if (Mac802_11WaitForNAV(node, M802)) {
        clocktype currentTime = getSimTime(node);
        //char comment[MAX_STRING_LENGTH];
        //sprintf(comment,
        //    "Call to transmit beacon when NAV=%f and time=%f",
        //    M802->NAV/(float)SECOND, currentTime/(float)SECOND);
        //Mac802_11Trace(node, M802, NULL, comment);

        Mac802_11SetState(node, M802, M802_11_S_WFNAV);
        Mac802_11StartTimer(node, M802, M802->NAV - currentTime);

        // prevent HandleTimeout from modifying state
        return TRUE;
    }

    switch (M802->state) {
        case M802_11_S_IDLE:
        case M802_11_S_WFNAV:
        case M802_11_S_NAV_RTS_CHECK_MODE:
        case M802_11_S_WF_DIFS_OR_EIFS:
            Mac802_11SetState(node, M802, M802_11_S_WF_CFP_BEACON);
            Mac802_11StartTimer(node, M802, M802->txPifs);
            couldTransmit = TRUE;
            break;

        case M802_11_S_BO:
            // Have waited enough, can transmit immediately
            M802->BO = 0;
            Mac802_11CfpPcTransmitBeacon(node, M802);
            couldTransmit = TRUE;
            break;

        case M802_11_S_WF_CFP_BEACON:
            Mac802_11CfpPcTransmitBeacon(node, M802);
            couldTransmit = TRUE;
            break;

        default:
            break;
    } //switch

    return couldTransmit;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcBeaconTransmitted
//  PURPOSE:     Set variables and states after the transmit of beacon
//               marking start of PCF. Set timer for next beacon.
//
//               Also set the self CF-End time to protect against
//               overlapping CFPs in overlapping BSSs.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11CfpPcBeaconTransmitted(
    Node* node,
    MacData802_11* M802)
{
    M802_11_PcVars* pc = M802->pcVars;
    clocktype currentTime = getSimTime(node);

    M802->cfpBeaconIsDue = FALSE;
    M802->beaconPacketsSentCfp++;

    // Set the Cf End timer. In overlapping BSSs, balance
    // duration can finish without being able to send CF End
    Mac802_11StartTimerOfGivenType(node, M802,
        M802->cfpBeaconTime
        + M802->cfpMaxDuration
        - currentTime
        + EPSILON_DELAY,
        MSG_MAC_802_11_CfpEnd);
    M802->cfpEndSequenceNumber = M802->timerSequenceNumber;

    Mac802_11CfpResetBackoffAndAckVariables(node, M802);

    // Set the timer for the next beacon period
    M802->cfpBeaconTime += M802->cfpBeaconInterval;
    Mac802_11StartTimerOfGivenType(node, M802,
        M802->cfpBeaconTime - currentTime, MSG_MAC_802_11_Beacon);
    M802->cfpBeaconSequenceNumber = M802->timerSequenceNumber;

    Mac802_11SetState(node, M802, M802_11_CFP_START);
    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
    M802->cfpAckIsDue = FALSE;

    // Initialize poll list variables if applicable
    if (Mac802_11CfpPcSendsPolls(pc)) {
        pc->allStationsPolled = FALSE;
        pc->firstPolledItem = NULL;
        pc->prevPolledItem = NULL;
        Mac802_11CfPollListResetAtStartOfCfp(node, M802);
    }

    Mac802_11StartTimer(node, M802, M802->txSifs);
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationProcessBeacon
//  PURPOSE:     A station receives a CFP beacon from within the BSS.
//               Set CFP end timers and enter PCF.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received beacon message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationProcessBeacon(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11_BeaconFrame* beacon =
        (M802_11_BeaconFrame*) MESSAGE_ReturnPacket(msg);
    clocktype beaconTimeStamp = beacon->timeStamp;
    clocktype currentTime = getSimTime(node);
    clocktype newNAV;

    // Record end time used for sufficient time computation.
    M802->cfpEndTime = beaconTimeStamp + beacon->cfSet.cfpBalDuration;

    // Set NAV for CFP duration.
    newNAV = M802->cfpEndTime + EPSILON_DELAY;

    if (newNAV > M802->NAV) {
        M802->NAV = newNAV;
    }
    else if (newNAV < M802->NAV) {
        // NAV is set for more than the CFP balance duration.
        // This is unlikely, unless a previous beacon from
        // another BSS had a longer CF period.
        Mac802_11Trace(node, M802, NULL, "Beacon NAV < set NAV.");
    }

    // Enter CFP
    Mac802_11CfpResetBackoffAndAckVariables(node, M802);
    Mac802_11SetState(node, M802, M802_11_CFP_START);
    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);

    M802->cfpBeaconInterval =
        Mac802_11TUsToClocktype(beacon->beaconInterval);
    M802->cfpMaxDuration =
        Mac802_11TUsToClocktype(beacon->cfSet.cfpMaxDuration);
    M802->cfpBeaconTime = M802->cfpEndTime - M802->cfpMaxDuration;

    // Start timer for next expected beacon
    Mac802_11StartTimerOfGivenType(node, M802,
        M802->cfpBeaconTime
        + M802->cfpBeaconInterval
        - currentTime,
        MSG_MAC_802_11_Beacon);
    M802->cfpBeaconSequenceNumber = M802->timerSequenceNumber;

    // Start timer for CFP end, in case CF End frame is missed.
    Mac802_11StartTimerOfGivenType(node, M802,
        newNAV - currentTime,
        MSG_MAC_802_11_CfpEnd);
    M802->cfpEndSequenceNumber = M802->timerSequenceNumber;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationReceiveBroadcast
//  PURPOSE:     Receive a CF broadcast, either data or CF-End frame
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationReceiveBroadcast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11LongControlFrame *fHdr =
        (M802_11LongControlFrame*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = fHdr->sourceAddr;
    M802_11MacFrameType frameType = (M802_11MacFrameType) fHdr->frameType;

    switch (frameType) {
        case M802_11_CF_DATA_ACK:
            Mac802_11CfpStationProcessAck(node, M802, sourceAddr);
            // no break; continue to process DATA

        case M802_11_DATA:
            Mac802_11Trace(node, M802, msg, "Receive");

            M802->broadcastPacketsGotCfp++;
            Mac802_11HandOffSuccessfullyReceivedBroadcast(node, M802, msg);

            break;

        case M802_11_CF_END_ACK:
            Mac802_11CfpStationProcessAck(node, M802, sourceAddr);
            // no break; continue to process M802_11_CF_END

        case M802_11_CF_END:
            Mac802_11Trace(node, M802, msg, "Receive");

            Mac802_11CfpStationProcessEnd(node, M802);

            MESSAGE_Free(node, msg);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationReceiveBroadcast: "
               "Unexpected frame type.\n");
            MESSAGE_Free(node, msg);
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationReceiveUnicast
//  PURPOSE:     Depending on the type of directed frame, process it
//               and arrange to respond appropriately
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpStationReceiveUnicast(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11FrameHdr* fHdr =
        (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);
    NodeAddress sourceAddr = fHdr->sourceAddr;
    M802_11MacFrameType frameType = (M802_11MacFrameType) fHdr->frameType;

    switch (frameType) {
        case M802_11_CF_DATA_POLL_ACK:
            Mac802_11CfpStationProcessAck(node, M802, sourceAddr);
            // no break; continue to process DATA

        case M802_11_CF_DATA_POLL:
            M802->dataPollPacketsGotCfp++;

            Mac802_11CfpStationProcessUnicast(node, M802, msg);

            Mac802_11CfpStationTransmit(node, M802, frameType, 0);
            break;

        case M802_11_CF_POLL_ACK:
            Mac802_11CfpStationProcessAck(node, M802, sourceAddr);
            // no break; continue to process M802_11_CF_POLL

        case M802_11_CF_POLL:
            Mac802_11Trace(node, M802, msg, "Receive");

            M802->pollPacketsGotCfp++;
            MESSAGE_Free(node, msg);

            Mac802_11CfpStationTransmit(node, M802, frameType, 0);
            break;

        case M802_11_CF_DATA_ACK:
            Mac802_11CfpStationProcessAck(node, M802, sourceAddr);
            // no break; continue to process DATA

        case M802_11_DATA:
            M802->dataPacketsGotCfp++;
            Mac802_11CfpStationProcessUnicast(node, M802, msg);

            Mac802_11CfpStationTransmit(node, M802, frameType, 0);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpStationProcessUnicast: "
                "Received unknown unicast frame type.\n");
            break;
    } //switch
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationInitiateCfPeriod
//  PURPOSE:     When a BSS station, which is not in PCF state,
//               receives a CF frame from within its BSS, it
//               implies that the beacon was missed.
//               It guesses a CFP duration and enters PCF state.
//               The guess is accurage if a beacon were previously received.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11CfpStationInitiateCfPeriod(
    Node* node,
    MacData802_11* M802)
{
    clocktype newNAV;
    clocktype currentTime = getSimTime(node);

    // Protect against previous missed beacons
    while (M802->cfpBeaconTime + M802->cfpMaxDuration < currentTime)
    {
        M802->cfpBeaconTime += M802->cfpBeaconInterval;
    }

    newNAV = M802->cfpBeaconTime + M802->cfpMaxDuration + EPSILON_DELAY;

    if (newNAV > M802->NAV) {
        M802->NAV = newNAV;
    }
    else if (newNAV < M802->NAV) {
        Mac802_11Trace(node, M802, NULL, "New guessed NAV < current NAV.");
    }

    // Enter PCF
    Mac802_11CfpResetBackoffAndAckVariables(node, M802);
    Mac802_11SetState(node, M802, M802_11_CFP_START);
    Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
    M802->cfpEndTime = M802->cfpBeaconTime + M802->cfpMaxDuration;

    // Start the end timer
    Mac802_11StartTimerOfGivenType(node, M802,
        newNAV - currentTime, MSG_MAC_802_11_CfpEnd);
    M802->cfpEndSequenceNumber = M802->timerSequenceNumber;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPcCheckBeaconOverlap
//  PURPOSE:     Checks to see if the CFP would would overlap with
//               its own, and, if so, issues a warning message.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received beacon message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpPcCheckBeaconOverlap(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11_BeaconFrame *beacon =
        (M802_11_BeaconFrame*) MESSAGE_ReturnPacket(msg);
    clocktype beaconTimeStamp =
        beacon->timeStamp;
    clocktype cfpMaxDuration =
        (clocktype) (beacon->cfSet.cfpBalDuration);
    clocktype cfpBalanceDuration =
        (clocktype) (beacon->cfSet.cfpBalDuration);

    // Check that CFPs would not overlap
    if (M802->cfpBeaconTime < beaconTimeStamp + cfpBalanceDuration) {
        Mac802_11Trace(node, M802, NULL,
            "WARNING Overlapping CFPs detected.");

        ERROR_ReportWarning(
            "Neighbouring PCs have overlapping Contention Free periods.\n"
            "Use MAC-802.11-BEACON-START-TIME for PC coordination.\n");
    }

    // Check that there is enough time for DCF.
    //  This check could be improved.
    if (cfpMaxDuration + M802->cfpMaxDuration >= M802->cfpBeaconInterval) {
        Mac802_11Trace(node, M802, NULL,
            "WARNING Overlapping PCF duration does not leave "
            "time for DCF.");

        ERROR_ReportWarning(
            "The PCF duration of neighbouring PCs do not leave time "
            "for the contention period.\n");
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpProcessNotMyFrame
//  PURPOSE:     Process a non-directed DCF frame received during CFP.
//               Set NAV if a BSS station.
//               Assume that it cannot happen for a PC.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               clocktype duration
//                  Duration of DCF frame
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
void Mac802_11CfpProcessNotMyFrame(
    Node* node,
    MacData802_11* M802,
    clocktype duration)
{
    clocktype currentTime;
    clocktype newNAV;

    if (duration == M802_11_CF_FRAME_DURATION * MICRO_SECOND) {
        return;
    }

    currentTime = getSimTime(node);
    newNAV = currentTime + duration + EPSILON_DELAY;
    if (newNAV > M802->NAV) {
        M802->NAV = newNAV;
    }
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpHandleTimeout
//  PURPOSE:     Handle timeouts during CFP depending on state and
//               occurance at PC or station.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
//static
void Mac802_11CfpHandleTimeout(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->cfpState) {
        case M802_11_S_CFP_NONE:
            ERROR_Assert(Mac802_11IsPC(M802),
                "Mac802_11CfpHandleTimeout: "
                "Unexpected station timeout in CFP_NONE state.\n");
            Mac802_11CfpPcTransmit(node, M802, M802_11_CF_NONE,
                M802->txSifs);
            break;

        case M802_11_S_CFP_WF_ACK:
            ERROR_Assert(M802->dataRateInfo->ipAddress ==
                         M802->currentNextHopAddress,
                         "Mac802_11CfpHandleTimeout: "
                         "Address does not match.\n");

            Mac802_11ResetAckVariables(M802);

            if (Mac802_11IsPC(M802))
            {
                Mac802_11Trace(node, M802, NULL,
                    "PC timeout waiting for ack.");

                M802->dataRateInfo->numAcksFailed++;
                Mac802_11AdjustDataRateForResponseTimeout(node, M802);

                // Continue with transmit
                Mac802_11CfpPcTransmit(node, M802, M802_11_CF_NONE, 0);
            }
            else
            {
                Mac802_11Trace(node, M802, NULL,
                    "Station timeout waiting for ack.");

                // For stations, wait for next opportunity to send the frame
                Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            }
            break;

        case M802_11_S_CFP_WF_DATA_ACK:
            ERROR_Assert(M802->dataRateInfo->ipAddress ==
                         M802->currentNextHopAddress,
                         "Mac802_11CfpHandleTimeout: "
                         "Address does not match.\n");

            M802->dataRateInfo->numAcksFailed++;
            Mac802_11AdjustDataRateForResponseTimeout(node, M802);

            Mac802_11ResetAckVariables(M802);

            // no break

        case M802_11_S_CFP_WF_DATA:
            ERROR_Assert(Mac802_11IsPC(M802),
                "Mac802_11CfpHandleTimeout: "
                "Non-PC times out waiting for data.\n");

            Mac802_11Trace(node, M802, NULL,
                "PC timeout waiting for data (+ack).");

            // Continue with transmit
            Mac802_11CfpPcTransmit(node, M802, M802_11_CF_NONE, 0);
            break;

        default:
            ERROR_ReportError("Mac802_11CfpHandleTimeout: "
                "Unknown CFP timeout.\n");
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpCancelSomeTimersBecauseChannelHasBecomeBusy
//  PURPOSE:     To cancel timers when channel changes from idle to
//               receiving state.
//
//               Note that unlike the corresponding function for DCF
//               states, this function is not called when channel changes
//               from idle to sensing state.
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
//static
void Mac802_11CfpCancelSomeTimersBecauseChannelHasBecomeBusy(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->cfpState) {
        case M802_11_S_CFP_NONE:
            break;

        case M802_11_S_CFP_WF_DATA:
        case M802_11_S_CFP_WF_DATA_ACK:
            // Assume that the PC is getting the expected frame.
            Mac802_11CancelTimer(node, M802);
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        case M802_11_S_CFP_WF_ACK:
            // Assume that the expected frame is coming in.
            Mac802_11CancelTimer(node, M802);
            Mac802_11CfpSetState(M802, M802_11_S_CFP_NONE);
            break;

        default:
            break;
    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPhyStatusIsNowIdleStartSendTimers
//  PURPOSE:     To restart send once the channel transits from busy
//               to idle. Applied only to a PC during PCF.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11CfpPhyStatusIsNowIdleStartSendTimers(
    Node* node,
    MacData802_11* M802)
{
    if (Mac802_11IsPC(M802)) {
        switch (M802->cfpState) {
            case M802_11_S_CFP_NONE:
                // Start a SIFS timer
                Mac802_11StartTimer(node, M802, M802->txSifs);
                break;

            case M802_11_S_CFP_WF_DATA:
            case M802_11_S_CFP_WF_ACK:
            case M802_11_S_CFP_WF_DATA_ACK:
                break;

            default:
                ERROR_ReportError(
                    "Mac802_11CfpPhyStatusIsNowIdleStartSendTimers: "
                    "Unexpected busy state.\n");
                break;
        } //switch
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpReceivePacketFromPhy
//  PURPOSE:     Receive a packet from PHY while in CFP state.
//               Process the frame depending on whether it is a
//               directed unicast or broadcast.
//               Call the appropriate routine for a DCF frame type.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11CfpReceivePacketFromPhy(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11ShortControlFrame* sHdr =
        (M802_11ShortControlFrame*) MESSAGE_ReturnPacket(msg);
    NodeAddress destAddr = sHdr->destAddr;
    M802_11MacFrameType frameType = (M802_11MacFrameType) sHdr->frameType;

    ERROR_Assert(!Mac802_11CfpIsInTransmitState(M802->cfpState),
        "Mac802_11CfpReceivePacketFromPhy: "
        "Receipt of frame while in transmit state.\n");

    if (destAddr == M802->selfAddr) {
        switch (frameType) {
            case M802_11_CF_DATA_POLL_ACK:
            case M802_11_CF_DATA_POLL:
            case M802_11_CF_POLL_ACK:
            case M802_11_CF_POLL:
                ERROR_Assert(Mac802_11IsFrameFromMyAP(M802,
                    ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr),
                    "Mac802_11CfpReceivePacketFromPhy: "
                    "Directed data and/or Poll frame is not from PC.\n");

                Mac802_11CfpStationReceiveUnicast(node, M802, msg);
                break;

            case M802_11_DATA:
            case M802_11_CF_DATA_ACK:
                if (Mac802_11IsFrameFromMyAP(M802,
                    ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr))
                {
                    Mac802_11CfpStationReceiveUnicast(node, M802, msg);
                }
                else {
                    ERROR_Assert(Mac802_11IsPC(M802),
                        "Mac802_11CfpReceivePacketFromPhy: "
                        "Non-PC receives unexpected data during CFP.\n");
                    Mac802_11CfpPcReceiveUnicast(node, M802, msg);
                }
                break;

            case M802_11_CF_NULLDATA:
                ERROR_Assert(Mac802_11IsFrameFromStationInMyBSS(M802,
                    destAddr, sHdr->frameFlags),
                    "Mac802_11CfpReceivePacketFromPhy: "
                    "Directed CF Null Data frame not from BSS.\n");

                Mac802_11CfpPcReceiveUnicast(node, M802, msg);
                break;

            case M802_11_CF_ACK:
                ERROR_Assert(Mac802_11IsFrameFromStationInMyBSS(M802,
                    destAddr, sHdr->frameFlags),
                    "Mac802_11CfpReceivePacketFromPhy: "
                    "Directed CF Ack frame not from BSS.\n");

                Mac802_11CfpPcReceiveUnicast(node, M802, msg);
                break;

            case M802_11_ACK:
                ERROR_Assert(Mac802_11IsPC(M802),
                    "Mac802_11CfpReceivePacketFromPhy: "
                    "BSS station receives directed Ack frame during CFP.\n");

                Mac802_11Trace(node, M802, msg, "Receive");

                // Tacit assumption that source is correct.
                Mac802_11CfpPcProcessAck(node, M802,
                    M802->waitingForAckOrCtsFromAddress, TRUE);

                Mac802_11CfpPcTransmit(node, M802, frameType, 0);

                MESSAGE_Free(node, msg);
                break;

            default:
                Mac802_11Trace(node, M802, NULL,
                    "Ignored directed frame during CFP");

                MESSAGE_Free(node, msg);
                break;
        } //switch
    }
    else if (destAddr == ANY_DEST) {
        switch (frameType) {
            case M802_11_DATA:
            case M802_11_CF_DATA_ACK:
                // Can be broadcast from PC or broadcast from outside BSS
                ERROR_Assert(!(Mac802_11IsPC(M802)
                    && Mac802_11IsFrameFromStationInMyBSS(M802,
                            destAddr, sHdr->frameFlags)),
                    "Mac802_11CfpReceivePacketFromPhy: "
                    "PC receives broadcast frame from BSS.\n");

                if (Mac802_11IsMyBroadcastDataFrame(M802, msg)) {
                    if (Mac802_11IsBssStation(M802)) {
                        Mac802_11CfpStationReceiveBroadcast(
                            node, M802, msg);
                    }
                    else if (Mac802_11IsPC(M802)) {
                        Mac802_11Trace(node, M802, msg, "Receive");

                        M802->broadcastPacketsGotCfp++;
                        Mac802_11HandOffSuccessfullyReceivedBroadcast(
                            node, M802, msg);
                    }
                }
                else {
                    Mac802_11Trace(node, M802, NULL,
                        "Drop, broadcast not for me");

                    M802->waitingForAckOrCtsFromAddress = INVALID_ADDRESS;

                    MESSAGE_Free(node, msg);
                }
                break;

            case M802_11_BEACON:
                Mac802_11Trace(node, M802, msg, "Receive");
                M802->beaconPacketsGotCfp++;

                if (Mac802_11IsBssStation(M802)) {
                    M802_11_BeaconFrame *beacon =
                        (M802_11_BeaconFrame*) MESSAGE_ReturnPacket(msg);
                    clocktype newNAV;

                    newNAV = beacon->timeStamp
                             + beacon->cfSet.cfpBalDuration
                             + EPSILON_DELAY;
                    if (newNAV > M802->NAV) {
                        M802->NAV = newNAV;
                        Mac802_11Trace(node, M802, NULL,
                            "NAV set for beacon from outside BSS");
                    }
                }
                else {
                    Mac802_11Trace(node, M802, NULL,
                        "PC receives beacon while in CFP.");

                    ERROR_ReportWarning("PC receives beacon while in CFP. "
                        "Overlapping PCF periods should be avoided.\n");
                }

                MESSAGE_Free(node, msg);
                break;

            case M802_11_CF_END:
            case M802_11_CF_END_ACK:
                if (Mac802_11IsFrameFromMyAP(M802,
                    ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr))
                {
                    Mac802_11CfpStationReceiveBroadcast(node, M802, msg);
                }
                else {
                    // CF End from outside BSS while in CFP.
                    //  There is no track of which beacon had
                    //  what beacon interval.
                    //  Ignore the frame.
                    //
                    Mac802_11Trace(node, M802, NULL,
                        "CF End from outside BSS during CFP");

                    M802->endPacketsGotCfp++;

                    MESSAGE_Free(node, msg);
                }
                break;

            default:
                ERROR_ReportError("Mac802_11CfpReceivePacketFromPhy: "
                    "Received invalid broadcast frame type.\n");
                break;
        } //switch
    }
    else {
        if (Mac802_11IsBssStation(M802)) {
            if (Mac802_11IsFrameFromMyAP(M802,
                    ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr)
                && Mac802_11IsCfFrameAnAckType(frameType))
            {
                Mac802_11CfpStationProcessAck(node, M802,
                    ((M802_11LongControlFrame *)MESSAGE_ReturnPacket(msg))
                    ->sourceAddr);
            }
        }

        Mac802_11CfpProcessNotMyFrame(node, M802,
            Mac802_11MicroToNanosecond(sHdr->duration));

        M802->waitingForAckOrCtsFromAddress = INVALID_ADDRESS;

        if (M802->myMacData->promiscuousMode
            && Mac802_11IsFrameAUnicastDataTypeForPromiscuousMode(
                    frameType, msg))
        {
            M802_11FrameHdr *fHdr =
                (M802_11FrameHdr*) MESSAGE_ReturnPacket(msg);

            Mac802_11HandlePromiscuousMode(node, M802, msg,
                fHdr->sourceAddr, fHdr->destAddr);
        }

        MESSAGE_Free(node,msg);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpFrameTransmitted
//  PURPOSE:     Once PHY indicates that a frame has been transmitted
//               during CFP, call the appropriate function for post
//               transmit processing.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
//static
void Mac802_11CfpFrameTransmitted(
    Node* node,
    MacData802_11* M802)
{
    switch (M802->cfpState) {

    case M802_11_X_CFP_UNICAST:
        if (Mac802_11IsPC(M802)) {
            Mac802_11CfpPcUnicastTransmitted(node, M802);
        }
        else {
            Mac802_11CfpStationUnicastTransmitted(node, M802);
        }
        break;

    case M802_11_X_CFP_BROADCAST:
        Mac802_11CfpPcBroadcastTransmitted(node, M802);
        break;

    case M802_11_X_CFP_ACK:
        if (Mac802_11IsPC(M802)) {
            Mac802_11CfpPcAckTransmitted(node, M802);
        }
        else {
            Mac802_11CfpStationAckTransmitted(node, M802);
        }
        break;

    case M802_11_X_CFP_POLL:
        Mac802_11CfpPcPollTransmitted(node, M802);
        break;

    case M802_11_X_CFP_UNICAST_POLL:
        Mac802_11CfpPcUnicastPollTransmitted(node, M802);
        break;

    case M802_11_X_CFP_NULLDATA:
        Mac802_11CfpStationNullDataTransmitted(node, M802);
        break;

    case M802_11_X_CFP_END:
        Mac802_11CfpPcEndTransmitted(node, M802);
        break;

    default:
        ERROR_ReportError("Mac802_11CfpFrameTransmitted: "
            "Unknown CFP state.\n");
        break;

    } //switch
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpInit
//  PURPOSE:     Initialize PCF related variables.
//               For an AP, read additional user inputs
//
//  PARAMETERS:  Node* node
//                  Pointer to current node
//               const NodeInput* nodeInput
//                  Pointer to node input
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               MacSubnetMemberData* subnetList
//                  Pointer to subnet list
//               int nodesInSubnet
//                  Number of nodes in subnet
//               int subnetListIndex
//                  Subnet list index
//               NodeAddress subnetAddress
//                  Subnet address
//               int numHostBits
//                  Number of host bits
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11CfpInit(
    Node* node,
    const NodeInput* nodeInput,
    MacData802_11* M802,
    MacSubnetMemberData* subnetList,
    int nodesInSubnet,
    int subnetListIndex,
    NodeAddress subnetAddress,
    int numHostBits,
    int networkType ,
    /*unsigned int tla,
    unsigned int nla,
    unsigned int sla*/
    in6_addr *subnetAddr,
    unsigned int subnetPrefixLen)
{
    char input[MAX_STRING_LENGTH];
    BOOL wasFound = FALSE;
    BOOL hasCfp = FALSE;
    Address address;

    M802->pifs = M802->sifs + M802->slotTime;
    M802->txPifs = M802->txSifs + M802->slotTime;

    M802->cfpState = M802_11_S_CFP_NONE;
    M802->cfpPrevState = M802_11_S_CFP_NONE;

    M802->cfpTimeout = 2 * M802->extraPropDelay + M802->pifs;

    M802->cfpLastFrameType = M802_11_CF_NONE;

    NetworkGetInterfaceInfo(
        node,
        M802->myMacData->interfaceIndex,
        &address);

    if (address.networkType == NETWORK_IPV6)
    {
        if (M802->selfAddr != INVALID_ADDRESS)
        {
            int interfaceIndex = M802->selfAddr & 0xff;
            MAPPING_GetIpv6GlobalAddressForInterface(
                node,
                node->nodeId,
                interfaceIndex,
                &address.interfaceAddr.ipv6);
        }
    }
    else
    {
        address.interfaceAddr.ipv4 = M802->selfAddr;
    }

    if (Mac802_11IsAP(M802))
    {
        // An AP can be a Point Coordinator. If an AP is a PC,
        // it will send beacon frames and initiate CF periods.
        // User input e.g.
        //       [0.6] MAC-802.11-PC  YES | NO
        // If not specified, the default is NO
        IO_ReadString(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-PC",
            &wasFound,
            input);

        if (wasFound)
        {
            if (!strcmp(input, "YES"))
            {
                M802->stationType = M802_11_STA_PC;
                hasCfp = TRUE;
            }
            else if (!strcmp(input, "NO"))
            {
                M802->stationType = M802_11_STA_AP;
            }
            else
            {
                ERROR_ReportError("Mac802_11CfpInit: "
                    "Invalid value for MAC-802.11-PC in configuration file.\n"
                    "Expecting YES or NO.\n");
            }
        }

        if (Mac802_11IsPC(M802))
        {
            // Initialize PC related variables
            M802->pcVars = (M802_11_PcVars*) MEM_malloc(sizeof(M802_11_PcVars));
            ERROR_Assert(M802->pcVars != NULL,
                "Mac802_11ApInit: Unable to allocate PC structure.\n");

            memset(M802->pcVars, 0, sizeof(M802_11_PcVars));

            M802->pcVars->broadcastsHavePriority =
                M802_11_BROADCASTS_HAVE_PRIORITY_DURING_CFP;
            M802->pcVars->cfPollList = NULL;
            M802->pcVars->pollableCount = 0;
            M802->pcVars->firstPolledItem = NULL;
            M802->pcVars->prevPolledItem = NULL;
            M802->pcVars->allStationsPolled = FALSE;

            if (networkType == NETWORK_IPV6)
            {
                Mac802_11CfpPcInit(node, nodeInput, M802,
                        subnetList, nodesInSubnet, subnetListIndex,
                        0, 0,
                        networkType,subnetAddr,subnetPrefixLen);
            }
            else
            {
            Mac802_11CfpPcInit(node, nodeInput, M802,
                    subnetList, nodesInSubnet, subnetListIndex,
                    subnetAddress, numHostBits);
            }

            // Start the first CFP beacon timer
            Mac802_11StartTimerOfGivenType(node, M802,
                M802->cfpBeaconTime, MSG_MAC_802_11_Beacon);
            M802->cfpBeaconSequenceNumber = M802->timerSequenceNumber;
        }
    }
    else if (Mac802_11IsBssStation(M802))
    {
        NodeId nodeId;
        Address tempAddress;
        in6_addr temp6 = {{{0}}};

        clocktype delay;
        if (networkType == NETWORK_IPV6)
        {
            int interfaceIndex = M802->selfAddr & 0xff;
                MAPPING_GetIpv6GlobalAddressForInterface(
                    node, node->nodeId, interfaceIndex,
                    &address.interfaceAddr.ipv6);
            nodeId = MAPPING_GetNodeIdFromLinkLayerAddr(
                        node, M802->bssAddr);
            SetIPv6AddressInfo(&tempAddress, temp6);
        }
        else
        {
            nodeId = MAPPING_GetNodeIdFromInterfaceAddress(
                        node, M802->bssAddr);
            SetIPv4AddressInfo(&tempAddress, M802->bssAddr);
        }
        // For static association, check if AP is a PC.
        IO_ReadString(
            nodeId,
            &tempAddress, //M802->bssAddr,
            nodeInput,
            "MAC-802.11-PC",
            &wasFound,
            input);

        if (wasFound)
        {
            hasCfp = TRUE;

            Mac802_11CfpStationInit(node, nodeInput, M802);

            // For a station, anticipate the first CFP beacon timer
            delay =
                M802->cfpBeaconTime
                - M802->delayUntilSignalAirborn
                - M802->extraPropDelay;
            if (delay <= 0)
            {
                delay = 1;
            }
            Mac802_11StartTimerOfGivenType(node, M802,
                delay,
                MSG_MAC_802_11_Beacon);
            M802->cfpBeaconSequenceNumber = M802->timerSequenceNumber;
        }
    }


    M802->printCfpStatistics = FALSE;

    if (hasCfp)
    {
        // Additional PCF statistics can be output with
        //     MAC-802.11-PCF-STATISTICS YES | NO
        // Default is M802_11_PRINT_STATS_DEFAULT
        // Stat values vary for different types of stations
        IO_ReadString(
            node->nodeId,
            &address, //M802->selfAddr,
            nodeInput,
            "MAC-802.11-PCF-STATISTICS",
            &wasFound,
            input);

        if (wasFound) {
            if (!strcmp(input, "YES")) {
                M802->printCfpStatistics = TRUE;
            }
            else if (!strcmp(input, "NO")) {
                M802->printCfpStatistics = FALSE;
            }
            else {
                ERROR_ReportError("Mac802_11CfpInit: "
                    "Invalid value for MAC-802.11-PCF-STATISTICS "
                    "in configuration file.\n"
                    "Expecting YES or NO.\n");
            }
        }
        else {
            M802->printCfpStatistics = M802_11_PRINT_STATS_DEFAULT;
        }
    }
}



//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpPrintStats
//  PURPOSE:     Print PCF related statistics.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               int interfaceIndex
//                  interfaceIndex, Interface index
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
//static
void Mac802_11CfpPrintStats(
    Node* node,
    MacData802_11* M802,
    int interfaceIndex)
{
    // Print PCF stats for PC and BSS station.
    // Skip stats if station is not part of coordinated subnet
    if (!M802->printCfpStatistics) {
        return;
    }

    if (Mac802_11IsPC(M802)) {
        Mac802_11CfpPcPrintStats(node, M802, interfaceIndex);
    }
    else if (Mac802_11IsBssStation(M802) &&
             (M802->dataPacketsGotCfp > 0 ||
              M802->dataPollPacketsGotCfp > 0 ||
              M802->pollPacketsGotCfp > 0))
    {
        Mac802_11CfpStationPrintStats(node, M802, interfaceIndex);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpStationAttemptToInitiateCfPeriod
//  PURPOSE:     Set NAV for a BSS station so that the station waits
//               for the CFP beacon.
//
//               This is typically done at the TBTT or the earliest
//               opportunity that the DCF state permits
//
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE if NAV has been set.
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
static
BOOL Mac802_11CfpStationAttemptToInitiateCfPeriod(
    Node* node,
    MacData802_11* M802)
{
    BOOL haveSetNAV = FALSE;
    clocktype newNAV;
    clocktype currentTime;

    if (Mac802_11PhyStatus(node, M802) != PHY_IDLE
        || Mac802_11IsTransmittingState(M802->state)
        || Mac802_11IsWaitingForResponseState(M802->state))
    {
        return FALSE;
    }

    switch (M802->state) {
        case M802_11_S_IDLE:
        case M802_11_S_WFNAV:
        case M802_11_S_WF_DIFS_OR_EIFS:
        case M802_11_S_BO:
        {
            currentTime = getSimTime(node);

            // Protect against previous missed beacons
            while (M802->cfpBeaconTime + M802->cfpMaxDuration
                    < currentTime)
            {
                M802->cfpBeaconTime += M802->cfpBeaconInterval;
            }

            newNAV = M802->cfpBeaconTime + M802->cfpMaxDuration
                     + EPSILON_DELAY;

            if (newNAV > M802->NAV) {
                if (M802->state == M802_11_S_BO) {
                        M802->BO = 0;
                }
                M802->NAV = newNAV;
                Mac802_11StartTimer(node, M802, newNAV - currentTime);
                Mac802_11SetState(node, M802, M802_11_S_WFNAV);
            }
            M802->cfpBeaconIsDue = FALSE;
            haveSetNAV = TRUE;

            break;
        }

        default:
            // In either transmit or wait-for state
            break;
    } //switch
    return haveSetNAV;
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CanHandleDueCfpBeacon
//  PURPOSE:     For a PC, attempt to send the beacon.
//               For a STA, set CFP NAV.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      TRUE if the due beacon could be handled.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
BOOL Mac802_11CanHandleDueCfpBeacon(
    Node* node,
    MacData802_11* M802)
{
    ERROR_Assert(M802->cfpBeaconIsDue,
        "Mac802_11CanHandleDueCfpBeacon: Beacon is not due.\n");

    if (Mac802_11IsPC(M802)) {
        return Mac802_11CfpPcAttemptToTransmitCfBeacon(node, M802);
    }
    else if (Mac802_11IsBssStation(M802)) {
        return Mac802_11CfpStationAttemptToInitiateCfPeriod(node, M802);
    }
    else {
        ERROR_ReportError("Mac802_11CfpHandleDueBeacon: "
            "Beacon should not become due for stations other "
            "than a PC or BSS station.\n");
        return FALSE;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpHandleEndTimer
//  PURPOSE:     Handle a timer if a PC was unable to send a CF End
//               or a STA did not receive a CF End
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
void Mac802_11CfpHandleEndTimer(
    Node* node,
    MacData802_11* M802)
{
    if (!Mac802_11IsInCfp(M802->state)) {
        return;
    }

    if (Mac802_11IsPC(M802)) {
        Mac802_11Trace(node, M802, NULL, "PC CF End timer");

        // CFP end timer expires but unable to send CF End.
        // Switch to DCF state.
        Mac802_11CfpSetEndVariables(node, M802);
        Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
    }
    else if (Mac802_11IsBssStation(M802)) {
        Mac802_11Trace(node, M802, NULL, "Station CF End timer");

        Mac802_11CfpSetEndVariables(node, M802);
        Mac802_11CheckForOutgoingPacket(node, M802, FALSE);
    }
}

//--------------------------------------------------------------------------
//  NAME:        Mac802_11CfpFreeStorage.
//  PURPOSE:     Free memory at the end of simulation.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               int interfaceIndex
//                  interfaceIndex, Interface index
//  RETURN:      None.
//  ASSUMPTION:  None.
//--------------------------------------------------------------------------
void Mac802_11CfpFreeStorage(
    Node* node,
    int interfaceIndex)
{
    MacData802_11* M802 =
        (MacData802_11*)node->macData[interfaceIndex]->macVar;

    if (Mac802_11IsPC(M802)) {
        M802_11_PcVars* pc = M802->pcVars;

        // Clear CF Polling list
        if (Mac802_11CfpPcSendsPolls(pc)) {
            Mac802_11CfPollListFree(node, M802, pc->cfPollList);
        }

        MEM_free(pc);
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ProcessCfpBeacon
//  PURPOSE:     A station (IBSS, BSS station, AP/PC) receives a CFP beacon.
//               If the beacon is from within the BSS, start PCF.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received beacon message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11ProcessCfpBeacon(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    M802_11_BeaconFrame* beacon =
        (M802_11_BeaconFrame*) MESSAGE_ReturnPacket(msg);
    clocktype newNAV;

    // Rule out non CFP beacon
    ERROR_Assert(beacon->cfSet.cfpBalDuration > 0,
        "Mac802_11StationProcessCfpBeacon: "
        "Received beacon with 0 or negative CFP duration.\n");

    Mac802_11Trace(node, M802, msg, "Receive");

    M802->beaconPacketsGotCfp++;

    if (Mac802_11IsBssStation(M802)
        && Mac802_11IsFrameFromMyAP(M802, beacon->hdr.sourceAddr))
    {
        Mac802_11CfpStationProcessBeacon(node, M802, msg);
        return;
    }

    if (Mac802_11IsPC(M802)) {
        Mac802_11CfpPcCheckBeaconOverlap(node, M802, msg);
    }

    // BSS station (also AP) receives beacon from outside BSS.
    // IBSS station receives beacon.

    newNAV =
        beacon->timeStamp
        + beacon->cfSet.cfpBalDuration
        + EPSILON_DELAY;

    if (newNAV > M802->NAV) {
        Mac802_11CancelTimer(node, M802);
        Mac802_11CfpResetBackoffAndAckVariables(node, M802);
        M802->NAV = newNAV;
    }
}


//--------------------------------------------------------------------------
//  NAME:        Mac802_11ProcessCfpEnd
//  PURPOSE:     Process a CF-End frame when not in CFP.
//               The frame can be received by a IBSS station or
//               station within a BSS, including AP.
//  PARAMETERS:  Node* node
//                  Pointer to node
//               MacData802_11* M802
//                  Pointer to 802.11 structure
//               Message* msg
//                  Received message
//  RETURN:      None
//  ASSUMPTION:  None
//--------------------------------------------------------------------------
void Mac802_11ProcessCfpEnd(
    Node* node,
    MacData802_11* M802,
    Message* msg)
{
    Mac802_11Trace(node, M802, msg, "Receive");

    M802->endPacketsGotCfp++;

    Mac802_11CfpSetEndVariables(node, M802);

    M802->NAV = 0;

    if (M802->currentMessage != NULL) {
        Mac802_11SetBackoffIfZero(node, M802);
    }
}

